import { IAssetAdapter } from "@awayjs/core";
import { Rectangle } from "../geom/Rectangle";
import { Point } from "../geom/Point";
import { Matrix } from "../geom/Matrix";
import { ColorTransform } from "../geom/ColorTransform";
import { BitmapFilter } from "../filters/BitmapFilter";
import { IBitmapDrawable } from "./IBitmapDrawable";
import { SceneImage2D } from "@awayjs/scene";
import { IBitmapDataOwner } from "./IBitmapDataOwner";
import { ASObject } from '../../avm2/nat/ASObject';
import { ByteArray } from '../../avm2/natives/byteArray';
import { Uint32Vector } from '../../avm2/natives/uint32Vector';
import { GenericVector } from '../../avm2/natives/GenericVector';
export declare class BitmapData extends ASObject implements IBitmapDrawable, IAssetAdapter {
    private _adaptee;
    private _owners;
    compare(other: BitmapData): boolean;
    get adaptee(): SceneImage2D;
    set adaptee(value: SceneImage2D);
    static loadBitmap(id: string): BitmapData;
    setPixels(rect: Rectangle, inputByteArray: ByteArray): void;
    getPixels(rect: Rectangle): ByteArray;
    copyPixelsToByteArray(rect: Rectangle, data: ByteArray): void;
    getVector(rect: Rectangle): Uint32Vector;
    setVector(rect: Rectangle, inputVector: Uint32Vector): void;
    histogram(hRect?: Rectangle): GenericVector;
    encode(rect: Rectangle, compressor: ASObject, byteArray?: ByteArray): ByteArray;
    drawWithQuality(source: IBitmapDrawable, matrix?: Matrix, colorTransform?: ColorTransform, blendMode?: string, clipRect?: Rectangle, smoothing?: boolean, quality?: string): void;
    constructor(width: number, height: number, transparent?: boolean, fillColor?: number);
    get transparent(): boolean;
    set transparent(value: boolean);
    get width(): number;
    set width(value: number);
    get height(): number;
    set height(value: number);
    clone(): BitmapData;
    get rect(): Rectangle;
    getPixel(x: number, y: number): number;
    getPixel32(x: number, y: number): number;
    setPixel(x: number, y: number, color: number): void;
    setPixel32(x: number, y: number, color: number): void;
    applyFilter(sourceBitmap: BitmapData, sourceRect: Rectangle, destPoint: Point, filter: BitmapFilter): number;
    colorTransform(rect: Rectangle, colorTransform: ColorTransform): void;
    copyChannel(sourceBitmap: BitmapData, sourceRect: Rectangle, destPoint: Point, sourceChannel: number, destChannel: number): void;
    copyPixels(sourceBitmap: any, sourceRect: Rectangle, destPoint: Point, alphaBitmapData?: BitmapData, alphaPoint?: Point, mergeAlpha?: boolean): void;
    dispose(): void;
    draw(source: any, matrix: Matrix, colorTransform?: ColorTransform, blendMode?: any, clipRect?: Rectangle, smooth?: boolean): void;
    fillRect(rect: Rectangle, color: number): void;
    floodFill(x: number, y: number, color: number): void;
    generateFilterRect(sourceRect: Rectangle, filter: BitmapFilter): Rectangle;
    getColorBoundsRect(mask: number, color: number, findColor: boolean): Rectangle;
    hitTest(firstPoint: Point, firstAlphaThreshold: number, secondObject: any, secondBitmapPoint?: Point, secondAlphaThreshold?: number): boolean;
    lock(): void;
    merge(sourceBitmap: BitmapData, sourceRect: Rectangle, destPoint: Point, redMult: number, greenMult: number, blueMult: number, alphaMult: number): void;
    noise(randomSeed: number, low: number, high: number, channelOptions: number, grayScale: boolean): void;
    paletteMap(sourceBitmap: BitmapData, sourceRect: Rectangle, destPoint: Point, redArray: any[], greenArray: any[], blueArray: any[], alphaArray: any[]): void;
    perlinNoise(baseX: number, baseY: number, numOctaves: number, randomSeed: number, stitch: boolean, fractalNoise: boolean, channelOptions: number, grayScale: boolean, offsets: any): void;
    pixelDissolve(sourceBitmap: BitmapData, sourceRect: Rectangle, destPoint: Point, randomSeed: number, numberOfPixels: number, fillColor: number): number;
    scroll(x: number, y: number): void;
    threshold(sourceBitmap: BitmapData, sourceRect: Rectangle, destPoint: Point, operation: string, threshold: number, color: number, mask: number, copySource: boolean): number;
    unlock(): void;
    _addOwner(owner: IBitmapDataOwner): void;
    _removeOwner(owner: IBitmapDataOwner): void;
}
