import { Graphics as AwayGraphics } from "@awayjs/graphics";
import { ASObject } from '../../avm2/nat/ASObject';
import { IAssetAdapter, Matrix } from '@awayjs/core';
import { ASArray } from '../../avm2/nat/ASArray';
import { BitmapData } from './BitmapData';
import { GenericVector } from '../../avm2/natives/GenericVector';
import { AXClass } from '../../avm2/run/AXClass';
import { LoaderInfo } from './LoaderInfo';
export declare class Graphics extends ASObject implements IAssetAdapter {
    static axClass: typeof Graphics & AXClass;
    static currentAwayGraphics: AwayGraphics;
    static classInitializer: any;
    private _adaptee;
    constructor(adaptee?: AwayGraphics);
    dispose(): void;
    get adaptee(): AwayGraphics;
    static FromData(data: any, loaderInfo: LoaderInfo): Graphics;
    clear(): void;
    /**
     * Sets a solid color and opacity as the fill for subsequent drawing commands.
     *
     * @see http://help.adobe.com/en_US/FlashPlatform/reference/actionscript/3/flash/display/Graphics.html#beginFill%28%29
     * @param color
     * @param alpha While any Number is a valid input, the value is clamped to [0,1] and then scaled
     * to an integer in the interval [0,0xff].
     */
    beginFill(color: number, alpha?: number): void;
    beginGradientFill(type: string, colors: ASArray, alphas: ASArray, ratios: ASArray, matrix?: Matrix, spreadMethod?: string, interpolationMethod?: string, focalPointRatio?: number): void;
    beginBitmapFill(bitmap: BitmapData, matrix?: Matrix, repeat?: boolean, smooth?: boolean): void;
    endFill(): void;
    lineStyle(thickness: number, color?: number, alpha?: number, pixelHinting?: boolean, scaleMode?: string, caps?: string, joints?: string, miterLimit?: number): void;
    lineGradientStyle(type: string, colors: ASArray, alphas: ASArray, ratios: ASArray, matrix?: Matrix, spreadMethod?: string, interpolationMethod?: string, focalPointRatio?: number): void;
    lineBitmapStyle(bitmap: BitmapData, matrix?: Matrix, repeat?: boolean, smooth?: boolean): void;
    drawRect(x: number, y: number, width: number, height: number): void;
    drawRoundRect(x: number, y: number, width: number, height: number, ellipseWidth: number, ellipseHeight: number): void;
    drawRoundRectComplex(x: number, y: number, width: number, height: number, topLeftRadius: number, topRightRadius: number, bottomLeftRadius: number, bottomRightRadius: number): void;
    drawCircle(x: number, y: number, radius: number): void;
    /**
     * Here x and y are the top-left coordinates of the bounding box of the
     * ellipse not the center as is the case for circles.
     */
    drawEllipse(x: number, y: number, width: number, height: number): void;
    moveTo(x: number, y: number): void;
    lineTo(x: number, y: number): void;
    curveTo(controlX: number, controlY: number, anchorX: number, anchorY: number): void;
    cubicCurveTo(controlX1: number, controlY1: number, controlX2: number, controlY2: number, anchorX: number, anchorY: number): void;
    copyFrom(sourceGraphics: Graphics): void;
    drawPath(commands: GenericVector, data: GenericVector, winding?: string): void;
    drawTriangles(vertices: GenericVector, indices?: GenericVector, uvtData?: GenericVector, culling?: string): void;
    drawGraphicsData(graphicsData: GenericVector): void;
}
