import { Event } from "./Event";
import { EventDispatcherBase } from "./EventDispatcherBase";
import { EventBase } from "@awayjs/core";
/**
     * [broadcast event] Dispatched when the Flash Player or AIR application operating
     * loses system focus and is becoming inactive.
     * @eventType	flash.events.Event.DEACTIVATE
    [Event(name="deactivate", type="flash.events.Event")]

     * [broadcast event] Dispatched when the Flash Player or AIR application gains
     * operating system focus and becomes active.
     * @eventType	flash.events.Event.ACTIVATE
    [Event(name="activate", type="flash.events.Event")]
 */
export declare class EventDispatcher extends EventDispatcherBase {
    toString(): string;
    hasEventListener(type: string, listener?: (event: EventBase) => void): boolean;
    willTrigger(): void;
    dispatchQueuedEvents(): void;
    getQueuedEvents(): EventBase[];
    dispatchEvent(event: EventBase): void;
    protected eventMapping: Object;
    protected eventMappingDummys: Object;
    protected eventMappingExtern: Object;
    protected eventMappingInvert: Object;
    _avm1Context: any;
    constructor(target?: any);
    private initActivateListener;
    private removeActivateListener;
    private _activateCallbackDelegate;
    private activateCallback;
    private initDeactivateListener;
    private removeDeactivateListener;
    private _deactivateCallbackDelegate;
    private deactivateCallback;
    addEventListener(type: string, listener: (event: EventBase) => void, useCapture?: boolean, priority?: number, useWeakReference?: boolean): void;
    /**
     * Remove an event listener
     * @method removeEventListener
     * @param {String} type of event to remove a listener for
     * @param {Function} listener function
     */
    removeEventListener(type: string, listener: (event: EventBase) => void): void;
}
export declare class BroadcastEventDispatchQueue {
    /**
     * The queues start off compact but can have null values if event targets are removed.
     * Periodically we compact them if too many null values exist.
     */
    private _queues;
    private static _instance;
    static getInstance(): BroadcastEventDispatchQueue;
    constructor();
    reset(): void;
    add(type: string, target: EventDispatcher): void;
    remove(type: string, target: EventDispatcher): void;
    dispatchEvent(event: Event): void;
    getQueueLength(type: string): any;
}
