import { __extends } from "tslib";
import { MovieClip as AwayMovieClip, Sprite as AwaySprite } from "@awayjs/scene";
import { DefaultSceneGraphFactory } from "@awayjs/scene";
import { Bitmap } from "../display/Bitmap";
import { BitmapData } from "../display/BitmapData";
import { Multiname } from '../../avm2/abc/lazy/Multiname';
import { constructClassFromSymbol } from '../../avm2/constructClassFromSymbol';
import { ABCFile } from '../../avm2/abc/lazy/ABCFile';
var FlashSceneGraphFactory = /** @class */ (function (_super) {
    __extends(FlashSceneGraphFactory, _super);
    function FlashSceneGraphFactory(sec) {
        var _this = _super.call(this) || this;
        _this.imageStore = {};
        _this.url = "";
        _this._sec = sec;
        return _this;
    }
    FlashSceneGraphFactory.prototype.executeABCBytes = function (abcBlocks) {
        for (var i = 0; i < abcBlocks.length; i++) {
            var abcBlock = abcBlocks[i];
            var abc = new ABCFile({ app: this._sec.application, url: "" }, abcBlock.data);
            if (abcBlock.flags) {
                // kDoAbcLazyInitializeFlag = 1 Indicates that the ABC block should not be executed
                // immediately.
                this._sec.application.loadABC(abc);
            }
            else {
                // TODO: probably delay execution until playhead reaches the frame.
                this._sec.application.loadAndExecuteABC(abc);
            }
        }
        return null;
    };
    FlashSceneGraphFactory.prototype.createSprite = function (prefab, graphics, symbol) {
        if (prefab === void 0) { prefab = null; }
        if (graphics === void 0) { graphics = null; }
        if (symbol === void 0) { symbol = null; }
        if (!symbol || !this._sec)
            throw ("no symbol provided");
        var symbolClass = null;
        if (symbol.className)
            symbolClass = this._sec.application.getClass(Multiname.FromFQNString(symbol.className, 0 /* Public */));
        else
            symbolClass = this._sec.flash.display.Sprite.axClass;
        symbol.symbolClass = symbolClass;
        //Graphicsadapter.currentAwayGraphics=graphics;
        // create the root for the root-symbol
        var asObj = constructClassFromSymbol(symbol, symbolClass);
        asObj.adaptee = new AwaySprite(graphics);
        // manually call the axInitializer for now:
        //asObj.axInitializer(AwaySprite.getNewSprite(new this._sec.flash.display.Graphics(graphics).adaptee));
        asObj.adaptee["fileurl"] = this.url;
        return asObj.adaptee;
    };
    FlashSceneGraphFactory.prototype.createDisplayObjectContainer = function (symbol) {
        if (symbol === void 0) { symbol = null; }
        return new this._sec.flash.display.DisplayObjectContainer().adaptee;
    };
    FlashSceneGraphFactory.prototype.createBinarySymbol = function (symbol) {
        if (symbol === void 0) { symbol = null; }
        this._sec.application.addBinarySymbol(symbol);
    };
    FlashSceneGraphFactory.prototype.createMovieClip = function (timeline, symbol) {
        if (timeline === void 0) { timeline = null; }
        if (symbol === void 0) { symbol = null; }
        if (!symbol || !this._sec)
            throw ("no symbol provided");
        var symbolClass = null;
        if (symbol.className) {
            symbolClass = this._sec.application.getClass(Multiname.FromFQNString(symbol.className, 0 /* Public */));
            symbolClass._symbol = symbol;
        }
        else
            symbolClass = this._sec.flash.display.MovieClip.axClass;
        //console.log("parsed symbolClass", symbolClass, symbol);
        symbol.symbolClass = symbolClass;
        // create the root for the root-symbol
        var asObj = constructClassFromSymbol(symbol, this._sec.flash.display.MovieClip.axClass);
        asObj.adaptee = new AwayMovieClip();
        symbol.timeline = asObj.adaptee.timeline;
        //(<AwayMovieClip>asObj.adaptee).timeline.resetScripts();
        //(<any>asObj).axInitializer();
        asObj.adaptee.timelineMC = true;
        asObj.adaptee["fileurl"] = this.url;
        return asObj.adaptee;
    };
    FlashSceneGraphFactory.prototype.createTextField = function (symbol) {
        if (symbol === void 0) { symbol = null; }
        var symbolClass = null;
        if (symbol.className)
            symbolClass = this._sec.application.getClass(Multiname.FromFQNString(symbol.className, 0 /* Public */));
        else
            symbolClass = this._sec.flash.text.TextField.axClass;
        symbol.symbolClass = symbolClass;
        // create the root for the root-symbol
        var asObj = constructClassFromSymbol(symbol, symbolClass);
        // 	manually call the axInitializer - this will run the constructor
        //	creating new Away-MovieClip and timeline, and registers framescripts on the timeline:
        asObj.axInitializer();
        asObj.adaptee["fileurl"] = this.url;
        return asObj.adaptee;
    };
    FlashSceneGraphFactory.prototype.createBillboard = function (material, symbol) {
        if (symbol === void 0) { symbol = null; }
        return new Bitmap(material.style.image.adapter).adaptee;
    };
    FlashSceneGraphFactory.prototype.createImage2D = function (width, height, transparent, fillColor, powerOfTwo, symbol) {
        if (transparent === void 0) { transparent = true; }
        if (fillColor === void 0) { fillColor = null; }
        if (powerOfTwo === void 0) { powerOfTwo = true; }
        if (symbol === void 0) { symbol = null; }
        return new BitmapData(width, height, transparent, fillColor).adaptee;
    };
    return FlashSceneGraphFactory;
}(DefaultSceneGraphFactory));
export { FlashSceneGraphFactory };
