import { ASObject } from '../../avm2/nat/ASObject';
import { AXClass } from '../../avm2/run/AXClass';
/**
 * The SoundTransform class contains properties for volume and panning.
 *
 */
export declare class SoundTransform extends ASObject {
    static axClass: typeof SoundTransform & AXClass;
    axCallPublicProperty(value1: any, value2: any): any;
    axGetPublicProperty(value: any): any;
    axSetPublicProperty(value: any, value2: any): any;
    axHasPublicProperty(value: any): any;
    axDeletePublicProperty(value: any): any;
    axGetEnumerableKeys(): string[];
    private _vol;
    private _pan;
    /**
     * Creates a SoundTransform object.
     * @param	vol	The volume, ranging from 0 (silent) to 1 (full volume).
     * @param	panning	The left-to-right panning of the sound, ranging from -1 (full pan left)
     *   to 1 (full pan right). A value of 0 represents no panning (center).
     * @langversion	3.0
     * @playerversion	Flash 9
     * @playerversion	Lite 4
     * @refpath
     */
    constructor(vol?: number, panning?: number);
    /**
     * A value, from 0 (none) to 1 (all), specifying how much of the left input is played in the
     * left speaker.
     */
    get leftToLeft(): number;
    set leftToLeft(leftToLeft: number);
    /**
     * A value, from 0 (none) to 1 (all), specifying how much of the left input is played in the
     * right speaker.
     */
    get leftToRight(): number;
    set leftToRight(leftToRight: number);
    /**
     * The left-to-right panning of the sound, ranging from -1 (full pan left)
     * to 1 (full pan right). A value of 0 represents no panning (balanced center between
     * right and left).
     */
    get pan(): number;
    set pan(value: number);
    /**
     * A value, from 0 (none) to 1 (all), specifying how much of the right input is played in the
     * left speaker.
     */
    get rightToLeft(): number;
    set rightToLeft(rightToLeft: number);
    /**
     * A value, from 0 (none) to 1 (all), specifying how much of the right input is played in the
     * right speaker.
     */
    get rightToRight(): number;
    set rightToRight(rightToRight: number);
    /**
     * The volume, ranging from 0 (silent) to 1 (full volume).
     */
    get volume(): number;
    set volume(value: number);
}
