/**
 * Copyright 2014 Mozilla Foundation
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
import { AVM1Context } from "../context";
import { IHasAS3ObjectReference } from "./AVM1Utils";
import { SceneImage2D as BitmapData } from "@awayjs/scene";
import { AVM1Object } from "../runtime/AVM1Object";
export declare function toAS3BitmapData(as2Object: AVM1BitmapData): BitmapData;
export declare class AVM1BitmapData extends AVM1Object implements IHasAS3ObjectReference {
    static createAVM1Class(context: AVM1Context): AVM1Object;
    get as3BitmapData(): BitmapData;
    avm1Constructor(width: number, height: number, transparent?: boolean, fillColor?: number): void;
    static fromAS3BitmapData(context: AVM1Context, awayObject: BitmapData): AVM1Object;
    static loadBitmap(context: AVM1Context, symbolId: string): AVM1BitmapData;
    getHeight(): number;
    getRectangle(): AVM1Object;
    getTransparent(): boolean;
    getWidth(): number;
    applyFilter(sourceBitmap: AVM1BitmapData, sourceRect: AVM1Object, destPoint: AVM1Object, filter: AVM1Object): number;
    clone(): AVM1BitmapData;
    colorTransform(rect: AVM1Object, colorTransform: AVM1Object): void;
    compare(other: AVM1BitmapData): boolean;
    copyChannel(sourceBitmap: AVM1BitmapData, sourceRect: AVM1Object, destPoint: AVM1Object, sourceChannel: number, destChannel: number): void;
    copyPixels(sourceBitmap: AVM1BitmapData, sourceRect: AVM1Object, destPoint: AVM1Object, alphaBitmap?: AVM1BitmapData, alphaPoint?: AVM1Object, mergeAlpha?: boolean): void;
    dispose(): void;
    draw(source: AVM1Object, matrix?: AVM1Object, colorTransform?: AVM1Object, blendMode?: any, clipRect?: AVM1Object, smooth?: boolean): void;
    fillRect(rect: AVM1Object, color: number): void;
    floodFill(x: number, y: number, color: number): void;
    generateFilterRect(sourceRect: AVM1Object, filter: AVM1Object): AVM1Object;
    getColorBoundsRect(mask: number, color: number, findColor?: boolean): AVM1Object;
    getPixel(x: number, y: number): number;
    getPixel32(x: number, y: number): number;
    hitTest(firstPoint: AVM1Object, firstAlphaThreshold: number, secondObject: AVM1Object, secondBitmapPoint?: AVM1Object, secondAlphaThreshold?: number): boolean;
    merge(sourceBitmap: AVM1BitmapData, sourceRect: AVM1Object, destPoint: AVM1Object, redMult: number, greenMult: number, blueMult: number, alphaMult: number): void;
    noise(randomSeed: number, low?: number, high?: number, channelOptions?: number, grayScale?: boolean): void;
    paletteMap(sourceBitmap: AVM1BitmapData, sourceRect: AVM1Object, destPoint: AVM1Object, redArray?: AVM1Object, greenArray?: AVM1Object, blueArray?: AVM1Object, alphaArray?: AVM1Object): void;
    perlinNoise(baseX: number, baseY: number, numOctaves: number, randomSeed: number, stitch: boolean, fractalNoise: boolean, channelOptions?: number, grayScale?: boolean, offsets?: AVM1Object): void;
    pixelDissolve(sourceBitmap: AVM1BitmapData, sourceRect: AVM1Object, destPoint: AVM1Object, randomSeed?: number, numberOfPixels?: number, fillColor?: number): number;
    scroll(x: number, y: number): void;
    setPixel(x: number, y: number, color: number): void;
    setPixel32(x: number, y: number, color: number): void;
    threshold(sourceBitmap: AVM1BitmapData, sourceRect: AVM1Object, destPoint: AVM1Object, operation: string, threshold: number, color?: number, mask?: number, copySource?: boolean): number;
}
