/**
 * Copyright 2014 Mozilla Foundation
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
import { IAVM1SymbolBase } from "./AVM1Utils";
import { AVM1Context } from "../context";
import { RandomSeed } from "random-seed";
import { AVM1Object } from "../runtime/AVM1Object";
import { AVM1Selection } from "./AVM1Selection";
import { ISoftKeyboardManager } from "../ISoftKeyboardManager";
export declare class TraceLevel {
    static ALL: string;
    static NONE: string;
    static IMPORTANT: string;
}
export declare class AVM1Globals extends AVM1Object {
    static _scenegraphFactory: any;
    static instance: AVM1Globals;
    static lessonStartTime: number;
    static _registeredCustomClasses: any;
    static _registeredCustomClassInstances: any;
    static randomProvider: RandomSeed;
    static setRandom(seed?: string): void;
    static softKeyboardManager: ISoftKeyboardManager;
    static registerCustomClass(name: string, avm1Class: any): void;
    static registerCustomClassInstance(name: string, avm1Class: AVM1Object): void;
    static createGlobalsObject(context: AVM1Context): AVM1Globals;
    static tracelevel: TraceLevel;
    static toStringPrecision: number;
    constructor(context: AVM1Context);
    flash: AVM1Object;
    ASSetPropFlags(obj: any, children: any, flags: any, allowFalse: any): any;
    clearIntervals(): void;
    clearInterval(id: number): void;
    clearTimeout(id: number): void;
    /**
     * AVM1 escapes slightly more characters than JS's encodeURIComponent, and even more than
     * the deprecated JS version of escape. That leaves no other option but to do manual post-
     * processing of the encoded result. :/
     *
     * Luckily, unescape isn't thus afflicted - it happily unescapes all the additional things
     * we escape here.
     */
    escape(str: string): string;
    unescape(str: string): string;
    setInterval(): any;
    setTimeout(): number;
    showRedrawRegions(enable: any, color: any): void;
    trace(expression: any): any;
    updateAfterEvent(): void;
    NaN: number;
    Infinity: number;
    isFinite(n: number): boolean;
    isNaN(n: any): boolean;
    parseFloat(s: string): number;
    parseInt(s: string, radix?: number): number;
    undefined: any;
    Object: AVM1Object;
    Function: AVM1Object;
    Array: AVM1Object;
    Number: AVM1Object;
    Math: AVM1Object;
    Boolean: AVM1Object;
    Date: AVM1Object;
    String: AVM1Object;
    Error: AVM1Object;
    ExternalInterface: AVM1Object;
    MovieClip: AVM1Object;
    AsBroadcaster: AVM1Object;
    System: AVM1Object;
    Stage: AVM1Object;
    Button: AVM1Object;
    TextField: AVM1Object;
    Color: AVM1Object;
    Key: AVM1Object;
    Mouse: AVM1Object;
    MovieClipLoader: AVM1Object;
    LoadVars: AVM1Object;
    Sound: AVM1Object;
    SharedObject: AVM1Object;
    ContextMenu: AVM1Object;
    ContextMenuItem: AVM1Object;
    TextFormat: AVM1Object;
    XMLNode: AVM1Object;
    XML: AVM1Object;
    filters: AVM1Object;
    BitmapData: AVM1Object;
    Matrix: AVM1Object;
    Point: AVM1Object;
    Rectangle: AVM1Object;
    Transform: AVM1Object;
    ColorTransform: AVM1Object;
    Selection: AVM1Selection;
    private _initBuiltins;
    private _initializeFlashObject;
}
export declare class AVM1NativeActions {
    context: AVM1Context;
    constructor(context: AVM1Context);
    asfunction(link: any): void;
    call(frame: any): void;
    chr(code: any): string;
    duplicateMovieClip(target: any, newname: any, depth: any): void;
    fscommand(command: string, args?: string): any;
    getTimer(): number;
    getURL(url: any, target?: any, method?: any): void;
    gotoAndPlay(label: any, frame?: any): void;
    gotoAndStop(label: any, frame?: any): void;
    private _gotoFrame;
    ifFrameLoaded(scene: any, frame?: any): boolean;
    length_(expression: any): number;
    loadMovie(url: string, target: any, method: string): void;
    loadMovieNum(url: any, level: any, method: any): void;
    loadVariables(url: string, target: any, method?: string): void;
    loadVariablesNum(url: string, level: number, method?: string): void;
    _loadVariables(nativeTarget: IAVM1SymbolBase, url: string, method: string): void;
    mbchr(code: any): string;
    mblength(expression: any): number;
    mbord(character: any): number;
    mbsubstring(value: any, index: any, count: any): string;
    nextFrame(): void;
    nextScene(): void;
    ord(character: any): number;
    play(): void;
    prevFrame(): void;
    prevScene(): void;
    print(target: any, boundingBox: any): void;
    printAsBitmap(target: any, boundingBox: any): void;
    printAsBitmapNum(level: any, boundingBox: any): void;
    printNum(level: any, bondingBox: any): void;
    random(value: any): number;
    removeMovieClip(target: any): void;
    startDrag(target?: any, ...args: any[]): void;
    stop(): void;
    stopAllSounds(): void;
    stopDrag(): void;
    substring(value: any, index: any, count: any): string;
    toggleHighQuality(): void;
    trace(expression: any): void;
    unloadMovie(target: any): void;
    unloadMovieNum(level: number): void;
}
