/**
 * Copyright 2014 Mozilla Foundation
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
import { IAVM1SymbolBase } from "./AVM1Utils";
import { AVM1Context } from "../context";
import { AVM1BitmapData } from "./AVM1BitmapData";
import { IMovieClipAdapter, DisplayObject, MovieClip } from "@awayjs/scene";
import { AVM1TextField } from "./AVM1TextField";
import { GradientType } from "@awayjs/graphics";
import { AVM1SymbolBase } from "./AVM1SymbolBase";
import { AVM1Object } from "../runtime/AVM1Object";
import { AVM1PropertyDescriptor } from "../runtime/AVM1PropertyDescriptor";
import { AVM1EventHandler } from "./AVM1EventHandler";
export interface IFrameScript {
    (any?: any): any;
    precedence?: number[];
    context?: MovieClip;
}
export declare const enum LookupChildOptions {
    DEFAULT = 0,
    IGNORE_CASE = 1,
    INCLUDE_NON_INITIALIZED = 2
}
export declare class AVM1MovieClip extends AVM1SymbolBase<MovieClip> implements IMovieClipAdapter {
    static currentMCAssetNameSpace: string;
    static currentDraggedMC: AVM1MovieClip;
    static pokiSDKonStopActionChildName: string;
    static pokiSDKonStopAction: any;
    static createAVM1Class(context: AVM1Context): AVM1Object;
    static capStyleMapStringToInt: any;
    static jointStyleMapStringToInt: any;
    private static noScaleDictionary;
    protected _mouseButtonListenerCount: number;
    _updateMouseEnabled(event: AVM1EventHandler, enabled: boolean): void;
    onLoaded: Function;
    executeConstructor: Function;
    clone(): AVM1MovieClip;
    private attachCustomConstructor;
    private compareAVM1FrameScripts;
    executeScript(actionsBlocks: any): void;
    addScript(source: any, frameIdx: number): any;
    stopAllSounds(): void;
    freeFromScript(): void;
    initAdapter(): void;
    registerScriptObject(child: DisplayObject, fromTimeline?: boolean): void;
    private unregisteredColors;
    unregisterScriptObject(child: DisplayObject): void;
    getLatestObjectForName(name: string): void;
    private _hitArea;
    private _lockroot;
    private get graphics();
    initAVM1SymbolInstance(context: AVM1Context, awayObject: any): void;
    _lookupChildByName(name: string): AVM1Object;
    private _lookupChildInAS3Object;
    get __targetPath(): string;
    getAwayJSID(): number;
    attachAudio(id: any): void;
    attachBitmap(bmp: AVM1BitmapData, depth: number, pixelSnapping?: string, smoothing?: boolean): void;
    _constructMovieClipSymbol(symbolId: string, name: string): MovieClip;
    get$version(): string;
    rgbaToArgb(float32Color: number): number;
    attachMovie(symbolId: any, name: any, depth: any, initObject: any): AVM1MovieClip;
    beginFill(color: number, alpha: number): void;
    beginBitmapFill(bmp: AVM1BitmapData, matrix?: AVM1Object, repeat?: boolean, smoothing?: boolean): void;
    beginGradientFill(fillType: GradientType, colors: AVM1Object, alphas: AVM1Object, ratios: AVM1Object, matrix: AVM1Object, spreadMethod?: string, interpolationMethod?: string, focalPointRatio?: number): void;
    _callFrame(frame: any): any;
    clear(): void;
    /**
     * This map stores the AVM1MovieClip's children keyed by their names. It's updated by all
     * operations that can cause different results for name-based lookups. these are
     * addition/removal of children and swapDepths.
     *
     * Using this map instead of always relaying lookups to the AVM2 MovieClip substantially
     * reduces the time spent in looking up children. In some cases by two orders of magnitude.
     */
    private _childrenByName;
    private _insertChildAtDepth;
    _updateChildName(child: AVM1MovieClip, oldName: string, newName: string): void;
    _removeChildName(child: IAVM1SymbolBase, name: string): void;
    _addChildName(child: IAVM1SymbolBase, name: string): void;
    createEmptyMovieClip(name: any, depth: any): AVM1MovieClip;
    createTextField(name: any, depth: any, x: any, y: any, width: any, height: any): AVM1TextField;
    get_currentframe(): number;
    curveTo(controlX: number, controlY: number, anchorX: number, anchorY: number): void;
    private _dropTarget;
    setDropTarget(dropTarget: DisplayObject): void;
    get_droptarget(): string;
    duplicateMovieClip(name: any, depth: any, initObject: any): AVM1MovieClip;
    endFill(): void;
    getForceSmoothing(): boolean;
    setForceSmoothing(value: boolean): void;
    get_framesloaded(): number;
    getBounds(bounds: any): AVM1Object;
    getBytesLoaded(): number;
    getBytesTotal(): number;
    getInstanceAtDepth(depth: number): AVM1MovieClip;
    getNextHighestDepth(): number;
    getRect(bounds: any): AVM1Object;
    getSWFVersion(): number;
    getTextSnapshot(): void;
    getURL(url: any, window: any, method: any): void;
    globalToLocal(pt: any): void;
    gotoAndPlay(frame: any): void;
    gotoAndStop(frame: any): void;
    private _gotoFrame;
    getHitArea(): any;
    setHitArea(value: any): void;
    hitTest(x: number, y: number, shapeFlag: boolean): boolean;
    lineGradientStyle(fillType: GradientType, colors: AVM1Object, alphas: AVM1Object, ratios: AVM1Object, matrix: AVM1Object, spreadMethod?: string, interpolationMethod?: string, focalPointRatio?: number): void;
    lineStyle(thickness?: number, rgb?: number, alpha?: number, pixelHinting?: boolean, noScale?: string, capsStyle?: string, jointStyle?: string, miterLimit?: number): void;
    lineTo(x: number, y: number): void;
    loadMovie(url: string, method: string): void;
    loadVariables(url: string, method?: string): void;
    localToGlobal(pt: any): void;
    get_lockroot(): boolean;
    set_lockroot(value: boolean): void;
    moveTo(x: number, y: number): void;
    nextFrame(): void;
    nextScene(): void;
    play(): void;
    prevFrame(): void;
    prevScene(): void;
    setMask(mc: Object): void;
    startDrag(lock?: boolean, left?: number, top?: number, right?: number, bottom?: number): void;
    private isDragging;
    private startDragPoint;
    private startDragMCPosition;
    private _dragBounds;
    dragListenerDelegate: (e: any) => void;
    dragListener(e: any): void;
    checkBounds(): void;
    stop(): void;
    stopDragDelegate: (e: any) => void;
    stopDrag(e?: any): void;
    swapDepths(target: any): void;
    getTabChildren(): boolean;
    setTabChildren(value: boolean): void;
    get_totalframes(): number;
    getTrackAsMenu(): boolean;
    setTrackAsMenu(value: boolean): void;
    unloadMovie(): void;
    getUseHandCursor(): boolean;
    setUseHandCursor(value: any): void;
    setParameters(parameters: any): any;
    private _resolveLevelNProperty;
    private _cachedPropertyResult;
    private _getCachedPropertyResult;
    alGetOwnProperty(name: any): AVM1PropertyDescriptor;
    alGetOwnPropertiesKeys(): any[];
    private _init;
    protected _initEventsHandlers(): void;
}
