/**
 * Copyright 2014 Mozilla Foundation
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
import { AVM1Context } from "../context";
import { DisplayObject, DisplayObjectContainer } from "@awayjs/scene";
import { AVM1SymbolBase } from "./AVM1SymbolBase";
import { AVM1Object } from "../runtime/AVM1Object";
export declare var DEPTH_OFFSET: number;
export interface IHasAS3ObjectReference {
    adaptee: any;
}
export interface IAVM1SymbolBase extends IHasAS3ObjectReference {
    context: AVM1Context;
    initAVM1SymbolInstance(context: AVM1Context, awayObject: DisplayObject): any;
    updateAllEvents(): any;
    getDepth(): number;
}
/**
 * Checks if an object contains a reference to a native AS3 object.
 * Returns false for MovieClip instances or instances of constructors with
 * MovieClip on their prototype chain that were created in script using,
 * e.g. new MovieClip(). Those lack the part of their internal structure
 * that makes them displayable.
 */
export declare function hasAwayJSAdaptee(obj: any): boolean;
/**
 * Returns obj's reference to a native AS3 object. If the reference
 * does not exist, returns undefined.
 */
export declare function getAwayJSAdaptee(obj: IHasAS3ObjectReference): any;
/**
 * Returns obj's reference to a native AS3 object. If the reference
 * doesn't exist, obj was created in script, e.g. with new MovieClip(),
 * and doesn't reflect a real, displayable display object. In that case,
 * an empty null-proto object is created and returned. This is used for
 * classes that are linked to embedded symbols that extend MovieClip. Their
 * inheritance chain is built by assigning new MovieClip to their prototype.
 * When a proper, displayable, instance of such a class is created via
 * attachMovie, initial values for properties such as tabEnabled
 * can be initialized from values set on the template object.
 */
export declare function getAwayObjectOrTemplate<T extends DisplayObjectContainer>(obj: AVM1SymbolBase<T>): T;
export declare var BlendModesMap: string[];
export declare function avm1HasEventProperty(context: AVM1Context, target: any, propertyName: string): boolean;
export declare function avm1BroadcastNativeEvent(context: AVM1Context, target: any, propertyName: string, args?: any[]): void;
export declare function avm1BroadcastEvent(context: AVM1Context, target: any, propertyName: string, args?: any[]): void;
export declare function getAVM1Object(awayObject: DisplayObject, context: AVM1Context): AVM1Object;
export declare function wrapAVM1NativeMembers(context: AVM1Context, wrap: AVM1Object, obj: any, members: string[], prefixFunctions?: boolean): void;
export declare function wrapAVM1NativeClass(context: AVM1Context, wrapAsFunction: boolean, cls: typeof AVM1Object, staticMembers: string[], members: string[], call?: Function, cstr?: Function): AVM1Object;
export declare function initializeAVM1Object(awayObject: any, context: AVM1Context, placeObjectTag: any): void;
export declare function toTwipFloor(value: number): number;
export declare function toTwipRound(value: number): number;
export declare function avm2AwayDepth(value: number): number;
export declare function away2avmDepth(value: number): number;
