/**
 * This file implements the AMF0 and AMF3 serialization protocols secified in:
 * http://wwwimages.adobe.com/www.adobe.com/content/dam/Adobe/en/devnet/amf/pdf/amf-file-format-spec.pdf
 */
import { ASObject } from "./nat/ASObject";
import { ByteArray } from "./natives/byteArray";
import { AXClass } from './run/AXClass';
export declare class ClassAliases {
    private _classMap;
    private _nameMap;
    getAliasByClass(axClass: AXClass): string;
    getClassByAlias(alias: string): AXClass;
    registerClassAlias(alias: string, axClass: AXClass): void;
}
export declare const enum AMF0Marker {
    NUMBER = 0,
    BOOLEAN = 1,
    STRING = 2,
    OBJECT = 3,
    NULL = 5,
    UNDEFINED = 6,
    REFERENCE = 7,
    ECMA_ARRAY = 8,
    OBJECT_END = 9,
    STRICT_ARRAY = 10,
    DATE = 11,
    LONG_STRING = 12,
    XML = 15,
    TYPED_OBJECT = 16,
    AVMPLUS = 17
}
export declare class AMF0 {
    static write(ba: ByteArray, value: any): void;
    static read(ba: ByteArray): any;
}
export declare const enum AMF3Marker {
    UNDEFINED = 0,
    NULL = 1,
    FALSE = 2,
    TRUE = 3,
    INTEGER = 4,
    DOUBLE = 5,
    STRING = 6,
    XML_DOC = 7,
    DATE = 8,
    ARRAY = 9,
    OBJECT = 10,
    XML = 11,
    BYTEARRAY = 12,
    VECTOR_INT = 13,
    VECTOR_UINT = 14,
    VECTOR_DOUBLE = 15,
    VECTOR_OBJECT = 16,
    DICTIONARY = 17
}
export declare class AMF3 {
    static write(ba: ByteArray, object: ASObject): void;
    static read(ba: ByteArray): any;
}
