/**
 * Option and Argument Management
 *
 * Options are configuration settings sprinkled throughout the code. They can be grouped into sets of
 * options called |OptionSets| which can form a hierarchy of options. For instance:
 *
 * var set = new OptionSet();
 * var opt = set.register(new Option("v", "verbose", "boolean", false, "Enables verbose logging."));
 *
 * creates an option set with one option in it. The option can be changed directly using |opt.value = true| or
 * automatically using the |ArgumentParser|:
 *
 * var parser = new ArgumentParser();
 * parser.addBoundOptionSet(set);
 * parser.parse(["-v"]);
 *
 * The |ArgumentParser| can also be used directly:
 *
 * var parser = new ArgumentParser();
 * argumentParser.addArgument("h", "help", "boolean", {parse: function (x) {
 *   printUsage();
 * }});
 */
export declare class Argument {
    shortName: string;
    longName: string;
    type: any;
    options: any;
    positional: boolean;
    parseFn: any;
    value: any;
    constructor(shortName: any, longName: any, type: any, options: any);
    parse(value: any): void;
}
export declare class ArgumentParser {
    args: any[];
    constructor();
    addArgument(shortName: any, longName: any, type: any, options: any): Argument;
    addBoundOption(option: any): void;
    addBoundOptionSet(optionSet: any): void;
    getUsage(): string;
    parse(args: any): any[];
}
export declare class OptionSet {
    name: string;
    settings: any;
    options: any;
    open: boolean;
    static isOptionSet(obj: any): boolean;
    constructor(name: string, settings?: any);
    register(option: any): any;
    trace(writer: any): void;
    getSettings(): {};
    setSettings(settings: any): void;
}
export declare class Option {
    longName: string;
    shortName: string;
    type: string;
    defaultValue: any;
    value: any;
    description: string;
    config: any;
    /**
     * Dat GUI control.
     */
    ctrl: any;
    constructor(shortName: any, longName: any, type: any, defaultValue: any, description: any, config?: any);
    parse(value: any): void;
    trace(writer: any): void;
}
