import { notImplemented } from './utilities/Debug';
export var FilterType;
(function (FilterType) {
    FilterType[FilterType["Blur"] = 0] = "Blur";
    FilterType[FilterType["DropShadow"] = 1] = "DropShadow";
    FilterType[FilterType["ColorMatrix"] = 2] = "ColorMatrix";
})(FilterType || (FilterType = {}));
export var MouseEventNames = [
    'click',
    'dblclick',
    'mousedown',
    'mousemove',
    'mouseup',
    'mouseover',
    'mouseout'
];
export var KeyboardEventNames = [
    'keydown',
    'keypress',
    'keyup'
];
/**
 * Implementation of ITransportPeer that uses standard DOM postMessage and
 * events to exchange data between messaging peers.
 */
var WindowTransportPeer = /** @class */ (function () {
    function WindowTransportPeer(window, target) {
        this.window = window;
        this.target = target;
        //
    }
    Object.defineProperty(WindowTransportPeer.prototype, "onAsyncMessage", {
        set: function (callback) {
            this.window.addEventListener('message', function (e) {
                Promise.resolve(e.data).then(function (msg) {
                    callback(msg);
                });
            });
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(WindowTransportPeer.prototype, "onSyncMessage", {
        set: function (callback) {
            this.window.addEventListener('syncmessage', function (e) {
                var wrappedMessage = e.detail;
                wrappedMessage.result = callback(wrappedMessage.msg);
            });
        },
        enumerable: true,
        configurable: true
    });
    WindowTransportPeer.prototype.postAsyncMessage = function (msg, transfers) {
        this.target.postMessage(msg, '*', transfers);
    };
    WindowTransportPeer.prototype.sendSyncMessage = function (msg, transfers) {
        var event = this.target.document.createEvent('CustomEvent');
        var wrappedMessage = {
            msg: msg,
            result: undefined
        };
        event.initCustomEvent('syncmessage', false, false, wrappedMessage);
        this.target.dispatchEvent(event);
        return wrappedMessage.result;
    };
    return WindowTransportPeer;
}());
export { WindowTransportPeer };
/**
 * Implementation of ITransportPeer that uses ShumwayCom API to exchange data
 * between messaging peers.
 */
var ShumwayComTransportPeer = /** @class */ (function () {
    function ShumwayComTransportPeer() {
    }
    Object.defineProperty(ShumwayComTransportPeer.prototype, "onAsyncMessage", {
        set: function (callback) {
            notImplemented("ShumwayComTransportPeer.onAsyncMessage not implemented yet");
            //ShumwayCom.setAsyncMessageCallback(callback);
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(ShumwayComTransportPeer.prototype, "onSyncMessage", {
        set: function (callback) {
            notImplemented("ShumwayComTransportPeer.onSyncMessage not implemented yet");
            //ShumwayCom.setSyncMessageCallback(callback);
        },
        enumerable: true,
        configurable: true
    });
    ShumwayComTransportPeer.prototype.postAsyncMessage = function (msg, transfers) {
        notImplemented("ShumwayComTransportPeer.postAsyncMessage not implemented yet");
        //ShumwayCom.postAsyncMessage(msg);
    };
    ShumwayComTransportPeer.prototype.sendSyncMessage = function (msg, transfers) {
        notImplemented("ShumwayComTransportPeer.sendSyncMessage not implemented yet");
        return null; //ShumwayCom.sendSyncMessage(msg);
    };
    return ShumwayComTransportPeer;
}());
export { ShumwayComTransportPeer };
