export declare class ArrayUtilities {
    static popManyInto: typeof popManyInto;
    static popMany: typeof popMany;
    static popManyIntoVoid: typeof popManyIntoVoid;
    static pushMany: typeof pushMany;
    static top: typeof top;
    static last: typeof last;
    static peek: typeof peek;
    static indexOf: typeof indexOf;
    static equals: typeof equals;
    static pushUnique: typeof pushUnique;
    static unique: typeof unique;
    static copyFrom: typeof copyFrom;
    static ensureTypedArrayCapacity: typeof ensureTypedArrayCapacity;
    static memCopy: typeof memCopy;
}
/**
 * Pops elements from a source array into a destination array. This avoids
 * allocations and should be faster. The elements in the destination array
 * are pushed in the same order as they appear in the source array:
 *
 * popManyInto([1, 2, 3], 2, dst) => dst = [2, 3]
 */
export declare function popManyInto(src: any[], count: number, dst: any[]): void;
export declare function popMany<T>(array: T[], count: number): T[];
/**
 * Just deletes several array elements from the end of the list.
 */
export declare function popManyIntoVoid(array: any[], count: number): void;
export declare function pushMany(dst: any[], src: any[]): void;
export declare function top(array: any[]): any;
export declare function last(array: any[]): any;
export declare function peek(array: any[]): any;
export declare function indexOf<T>(array: T[], value: T): number;
export declare function equals<T>(a: T[], b: T[]): boolean;
export declare function pushUnique<T>(array: T[], value: T): number;
export declare function unique<T>(array: T[]): T[];
export declare function copyFrom(dst: any[], src: any[]): void;
/**
 * Makes sure that a typed array has the requested capacity. If required, it creates a new
 * instance of the array's class with a power-of-two capacity at least as large as required.
 */
export declare function ensureTypedArrayCapacity<T extends TypedArray>(array: T, capacity: number): T;
export declare function memCopy<T extends TypedArray>(destination: T, source: T, doffset?: number, soffset?: number, length?: number): void;
export interface TypedArray {
    buffer: ArrayBuffer;
    length: number;
    set: (array: TypedArray, offset?: number) => void;
    subarray: (begin: number, end?: number) => TypedArray;
}
export interface IDataDecoder {
    onData: (data: Uint8Array) => void;
    onError: (e: any) => void;
    push(data: Uint8Array): any;
    close(): any;
}
