import { __extends } from "tslib";
import { DisplayObject } from "./DisplayObject";
import { assert, axCoerceString, Bounds } from "@awayjs/graphics";
import { release } from "../../base/utilities/Debug";
import { constructClassFromSymbol } from "../../avm2/constructClassFromSymbol";
import { BitmapData } from './BitmapData';
/**
 * Copyright 2014 Mozilla Foundation
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
// Class: Bitmap
var Bitmap = /** @class */ (function (_super) {
    __extends(Bitmap, _super);
    function Bitmap(bitmapData, pixelSnapping, smoothing) {
        if (bitmapData === void 0) { bitmapData = null; }
        if (pixelSnapping === void 0) { pixelSnapping = "auto"; }
        if (smoothing === void 0) { smoothing = false; }
        var _this = _super.call(this) || this;
        if (_this._symbol && !_this._fieldsInitialized) {
            _this.applySymbol();
        }
        if (!_this._symbol) {
            _this.bitmapData = bitmapData;
            _this._pixelSnapping = axCoerceString(pixelSnapping);
            _this._smoothing = !!smoothing;
        }
        return _this;
    }
    Bitmap.prototype.applySymbol = function () {
        release || assert(this._symbol);
        this._initializeFields();
        var symbol = this._symbol;
        var symbolClass = symbol.symbolClass;
        // If the symbol class inherits from Bitmap, we are already within its initializer.
        // Make sure to create a BitmapData instance here to avoid recursively calling the
        // initializer again.
        var bitmapClass = Bitmap.axClass;
        if (bitmapClass.dPrototype.isPrototypeOf(symbolClass.dPrototype)) {
            symbolClass = BitmapData.axClass;
        }
        // TODO: I don't think BitmapData symbol objects can change, so they don't need back
        // references to this Bitmap.
        this._bitmapData = constructClassFromSymbol(symbol, symbolClass);
        this._pixelSnapping = null;
        this._smoothing = null;
        this._setFillAndLineBoundsFromWidthAndHeight(symbol.width * 20 | 0, symbol.height * 20 | 0);
    };
    Object.defineProperty(Bitmap.prototype, "pixelSnapping", {
        get: function () {
            return this._pixelSnapping;
        },
        set: function (value) {
            console.log("todo: Bitmap.pixelSnapping");
            /*if (PixelSnapping.toNumber(value) < 0) {
              this.sec.throwError("ArgumentError", Errors.InvalidEnumError, "pixelSnapping");
            }*/
            this._pixelSnapping = axCoerceString(value);
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(Bitmap.prototype, "smoothing", {
        get: function () {
            return this._smoothing;
        },
        set: function (value) {
            this._smoothing = !!value;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(Bitmap.prototype, "bitmapData", {
        get: function () {
            return this._bitmapData;
        },
        set: function (value) {
            if (this._bitmapData !== value) {
                if (this._bitmapData) {
                    this._bitmapData._removeBitmapReferrer(this);
                }
                if (value) {
                    value._addBitmapReferrer(this);
                }
            }
            this._bitmapData = value;
            if (value) {
                this._setFillAndLineBoundsFromWidthAndHeight(value.width * 20 | 0, value.height * 20 | 0);
            }
            this._invalidateParentFillAndLineBounds(true, true);
            this._setDirtyFlags(16 /* DirtyBitmapData */);
        },
        enumerable: true,
        configurable: true
    });
    Bitmap.prototype._getContentBounds = function (includeStrokes) {
        if (includeStrokes === void 0) { includeStrokes = true; }
        if (this._bitmapData) {
            return this._bitmapData._getContentBounds();
        }
        return new Bounds(0, 0, 0, 0);
    };
    Bitmap.prototype._containsPointDirectly = function (localX, localY, globalX, globalY) {
        // If this override is reached, the content bounds have already been checked, which is all
        // we need to do.
        release || assert(this._getContentBounds().contains(localX, localY));
        return true;
    };
    // Called whenever the class is initialized.
    Bitmap.classInitializer = null;
    // List of static symbols to link.
    Bitmap.classSymbols = null; // [];
    // List of instance symbols to link.
    Bitmap.instanceSymbols = null; // [];
    return Bitmap;
}(DisplayObject));
export { Bitmap };
