import { __extends } from "tslib";
import { mixHash } from "../../base/utilities/HashUtilities";
import { InteractiveObject } from "./InteractiveObject";
import { DisplayObject } from "./DisplayObject";
import { Event } from "../events/Event";
import { release, warning, notImplemented } from "../../base/utilities/Debug";
import { checkParameterType } from "../../avm2/run/checkParameterType";
import { axCoerceString } from "../../avm2/run/axCoerceString";
import { Errors } from "../../avm2/errors";
import { clamp } from "@awayjs/graphics";
import { assert } from "@awayjs/graphics";
import { AVM1Movie } from "./AVM1Movie";
import { counter } from '../../avm2/module';
import { initializeAVM1Object } from '../../avm1/lib/AVM1Utils';
var DisplayObjectContainer = /** @class */ (function (_super) {
    __extends(DisplayObjectContainer, _super);
    function DisplayObjectContainer() {
        var _this = _super.call(this) || this;
        if (!_this._fieldsInitialized) {
            _this._initializeFields();
        }
        _this._setDirtyFlags(2 /* DirtyChildren */);
        return _this;
    }
    DisplayObjectContainer.prototype._initializeFields = function () {
        _super.prototype._initializeFields.call(this);
        this._tabChildren = true;
        this._mouseChildren = true;
        // Might already have been initialized from a symbol.
        this._children = this._children || [];
    };
    /**
     * This object's children have changed.
     */
    DisplayObjectContainer.prototype._invalidateChildren = function () {
        this._setDirtyFlags(2 /* DirtyChildren */);
        this._invalidateFillAndLineBounds(true, true);
    };
    /**
     * Propagates flags down the display list. Propagation stops if all flags are already set.
     */
    DisplayObjectContainer.prototype._propagateFlagsDown = function (flags) {
        if (this._hasFlags(flags)) {
            return;
        }
        this._setFlags(flags);
        var children = this._children;
        for (var i = 0; i < children.length; i++) {
            children[i]._propagateFlagsDown(flags);
        }
    };
    /**
     * Calls the constructors of new children placed by timeline commands.
     */
    DisplayObjectContainer.prototype._constructChildren = function () {
        release || counter.count("DisplayObjectContainer::_constructChildren");
        var children = this._children;
        for (var i = 0; i < children.length; i++) {
            var child = children[i];
            if (child._hasFlags(256 /* Constructed */)) {
                continue;
            }
            try {
                child.axInitializer();
            }
            catch (e) {
                warning('caught error executing child constructor in constructChildren: ', e);
            }
            //child.class.instanceConstructorNoInitialize.call(child);
            child._removeReference();
            if (child._name) {
                this.axSetPublicProperty(child._name, child);
                //child._addReference();
            }
            child._setFlags(256 /* Constructed */);
            var eventClass = Event.axClass;
            if (child._hasFlags(131072 /* HasPlaceObjectInitPending */)) {
                child._removeFlags(131072 /* HasPlaceObjectInitPending */);
                var avm1Context = child._symbol.avm1Context;
                initializeAVM1Object(child, avm1Context, child._placeObjectTag);
                try {
                    child.dispatchEvent(eventClass.getInstance(Event.AVM1_INIT));
                }
                catch (e) {
                    warning('caught error under DisplayObjectContainer AVM1_INIT event: ', e);
                }
                try {
                    child.dispatchEvent(eventClass.getInstance(Event.AVM1_CONSTRUCT));
                }
                catch (e) {
                    warning('caught error under DisplayObjectContainer AVM1_CONSTRUCT event: ', e);
                }
                if (child.hasEventListener(Event.AVM1_LOAD)) {
                    child._setFlags(1024 /* NeedsLoadEvent */);
                    this._propagateFlagsUp(16384 /* ContainsFrameScriptPendingChildren */);
                }
            }
            try {
                child.dispatchEvent(eventClass.getInstance(Event.ADDED, true));
            }
            catch (e) {
                warning('caught error under DisplayObject ADDED event: ', e);
            }
            if (child.stage) {
                try {
                    child.dispatchEvent(eventClass.getInstance(Event.ADDED_TO_STAGE));
                }
                catch (e) {
                    warning('caught error under DisplayObject ADDED_TO_STAGE event: ', e);
                }
            }
        }
    };
    DisplayObjectContainer.prototype._enqueueFrameScripts = function () {
        if (this._hasFlags(16384 /* ContainsFrameScriptPendingChildren */)) {
            this._removeFlags(16384 /* ContainsFrameScriptPendingChildren */);
            var children = this._children;
            for (var i = 0; i < children.length; i++) {
                var child = children[i];
                if (DisplayObjectContainer.axIsType(child) ||
                    AVM1Movie.axIsType(child)) {
                    child._enqueueFrameScripts();
                }
            }
        }
    };
    Object.defineProperty(DisplayObjectContainer.prototype, "numChildren", {
        get: function () {
            return this._getNumChildren();
        },
        enumerable: true,
        configurable: true
    });
    // `get numChildren` is overridden in `Stage` and has to be able to call this.
    DisplayObjectContainer.prototype._getNumChildren = function () {
        return this._children.length;
    };
    Object.defineProperty(DisplayObjectContainer.prototype, "textSnapshot", {
        get: function () {
            release || notImplemented("public DisplayObjectContainer::get textSnapshot");
            return;
            // return this._textSnapshot;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(DisplayObjectContainer.prototype, "tabChildren", {
        get: function () {
            return this._getTabChildren();
        },
        set: function (enable) {
            this._setTabChildren(enable);
        },
        enumerable: true,
        configurable: true
    });
    // `get tabChildren` is overridden in `Stage` and has to be able to call this.
    DisplayObjectContainer.prototype._getTabChildren = function () {
        return this._tabChildren;
    };
    // `set tabChildren` is overridden in `Stage` and has to be able to call this.
    DisplayObjectContainer.prototype._setTabChildren = function (enable) {
        enable = !!enable;
        var old = this._tabChildren;
        this._tabChildren = enable;
        if (old !== enable) {
            this.dispatchEvent(Event.axClass.getInstance(Event.TAB_CHILDREN_CHANGE, true));
        }
    };
    Object.defineProperty(DisplayObjectContainer.prototype, "mouseChildren", {
        get: function () {
            return this._getMouseChildren();
        },
        set: function (enable) {
            this._setMouseChildren(enable);
        },
        enumerable: true,
        configurable: true
    });
    // `get mouseChildren` is overridden in `Stage` and has to be able to call this.
    DisplayObjectContainer.prototype._getMouseChildren = function () {
        return this._mouseChildren;
    };
    // `set mouseChildren` is overridden in `Stage` and has to be able to call this.
    DisplayObjectContainer.prototype._setMouseChildren = function (enable) {
        this._mouseChildren = !!enable;
    };
    DisplayObjectContainer.prototype.addChild = function (child) {
        return this.addChildAt(child, this._children.length);
    };
    /**
     * Adds a child at a given index. The index must be within the range [0 ... children.length].
     * Note that this is different than the range setChildIndex expects.
     */
    DisplayObjectContainer.prototype.addChildAt = function (child, index /*int*/) {
        checkParameterType(child, "child", DisplayObject.axClass);
        release || counter.count("DisplayObjectContainer::addChildAt");
        index = index | 0;
        release || assert(child._hasFlags(256 /* Constructed */), "Child is not fully constructed.");
        if (child === this) {
            this.sec.throwError('ArgumentError', Errors.CantAddSelfError);
        }
        if (DisplayObjectContainer.axIsType(child) &&
            child.contains(this)) {
            this.sec.throwError('ArgumentError', Errors.CantAddParentError);
        }
        var children = this._children;
        if (index < 0 || index > children.length) {
            this.sec.throwError('RangeError', Errors.ParamRangeError);
        }
        if (child._parent === this) {
            this.setChildIndex(child, clamp(index, 0, children.length - 1));
            return child;
        }
        // TODO: check what happens if a listener for REMOVE throws an error. Does the element get
        // reparented nevertheless?
        if (child._parent) {
            // Loader overrides removeChildAt to throw an exception. We still want to use it, so we
            // always call the original version here.
            DisplayObjectContainer.prototype.removeChildAt.call(child._parent, child._parent.getChildIndex(child));
            // The children list could have been mutated as a result of |removeChild|.
            index = clamp(index, 0, children.length);
        }
        for (var i = children.length - 1; i >= index; i--) {
            children[i]._index++;
        }
        children.splice(index, 0, child);
        child._setParent(this, -1);
        child._index = index;
        child._invalidatePosition();
        this._invalidateChildren();
        child._addReference();
        child.dispatchEvent(Event.axClass.getInstance(Event.ADDED, true));
        // ADDED event handlers may remove the child from the stage, in such cases
        // we should not dispatch the ADDED_TO_STAGE event.
        if (child.stage) {
            child._propagateEvent(Event.axClass.getInstance(Event.ADDED_TO_STAGE));
        }
        return child;
    };
    /**
     * Adds a timeline object to this container. The new child is added after the last object that
     * exists at a smaller depth, or before the first object that exists at a greater depth. If no
     * other timeline object is found, the new child is added to the front(top) of all other
     * children.
     *
     * Note that this differs from `addChildAt` in that the depth isn't an index in the `children`
     * array, and doesn't have to be in the dense range [0..children.length].
     */
    DisplayObjectContainer.prototype.addTimelineObjectAtDepth = function (child, depth /*int*/) {
        release || counter.count("DisplayObjectContainer::addTimelineObjectAtDepth");
        depth = depth | 0;
        var children = this._children;
        var maxIndex = children.length - 1;
        var index = maxIndex + 1;
        for (var i = maxIndex; i >= 0; i--) {
            var current = children[i];
            if (current._depth > -1) {
                if (current._depth < depth) {
                    index = i + 1;
                    break;
                }
                index = i;
            }
        }
        if (index > maxIndex) {
            children.push(child);
            child._index = index;
        }
        else {
            children.splice(index, 0, child);
            for (var i = index; i < children.length; i++) {
                children[i]._index = i;
            }
        }
        child._setParent(this, depth);
        child._invalidatePosition();
        this._invalidateChildren();
    };
    DisplayObjectContainer.prototype.removeChild = function (child) {
        checkParameterType(child, "child", DisplayObject.axClass);
        return this.removeChildAt(this.getChildIndex(child));
    };
    DisplayObjectContainer.prototype.removeChildAt = function (index) {
        release || counter.count("DisplayObjectContainer::removeChildAt");
        index = index | 0;
        var children = this._children;
        if (index < 0 || index >= children.length) {
            this.sec.throwError('RangeError', Errors.ParamRangeError);
        }
        var child = children[index];
        if (child._hasFlags(256 /* Constructed */)) {
            child.dispatchEvent(Event.axClass.getInstance(Event.REMOVED, true));
            if (this.stage) {
                child._propagateEvent(Event.axClass.getInstance(Event.REMOVED_FROM_STAGE));
            }
            // Children list might have been mutated by the REMOVED or REMOVED_FROM_STAGE event,
            // we may need to operate on the new index of the child.
            index = this.getChildIndex(child);
        }
        children.splice(index, 1);
        for (var i = children.length - 1; i >= index; i--) {
            children[i]._index--;
        }
        child._setParent(null, -1);
        child._index = -1;
        child._invalidatePosition();
        this._invalidateChildren();
        return child;
    };
    DisplayObjectContainer.prototype.getChildIndex = function (child) {
        checkParameterType(child, "child", DisplayObject.axClass);
        if (child._parent !== this) {
            this.sec.throwError('ArgumentError', Errors.NotAChildError);
        }
        return child._index;
    };
    /**
     * Sets the index of a child. The index must be within the range [0 ... children.length - 1].
     */
    DisplayObjectContainer.prototype.setChildIndex = function (child, index /*int*/) {
        index = index | 0;
        checkParameterType(child, "child", DisplayObject.axClass);
        var children = this._children;
        if (index < 0 || index >= children.length) {
            this.sec.throwError('RangeError', Errors.ParamRangeError);
        }
        if (child._parent !== this) {
            this.sec.throwError('ArgumentError', Errors.NotAChildError);
        }
        child._setDepth(-1);
        var currentIndex = this.getChildIndex(child);
        if (children.length === 1 || currentIndex === index) {
            return;
        }
        if (index === currentIndex + 1 || index === currentIndex - 1) {
            // We can't call |swapChildrenAt| here because we don't want to affect the depth value.
            this._swapChildrenAt(currentIndex, index);
        }
        else {
            children.splice(currentIndex, 1);
            children.splice(index, 0, child);
            var i = currentIndex < index ? currentIndex : index;
            while (i < children.length) {
                children[i]._index = i++;
            }
        }
        this._invalidateChildren();
    };
    DisplayObjectContainer.prototype.getChildAt = function (index) {
        index = index | 0;
        var children = this._children;
        if (index < 0 || index >= children.length) {
            this.sec.throwError('RangeError', Errors.ParamRangeError);
        }
        var child = this._lookupChildByIndex(index, 0 /* DEFAULT */);
        if (!child) {
            return null;
        }
        child._addReference();
        return child;
    };
    /**
     * Returns the timeline object that exists at the specified depth.
     */
    DisplayObjectContainer.prototype.getTimelineObjectAtDepth = function (depth /*int*/) {
        depth = depth | 0;
        var children = this._children;
        for (var i = 0; i < children.length; i++) {
            var child = children[i];
            if (child._depth > depth) {
                break;
            }
            if (child._depth === depth) {
                return child;
            }
        }
        return null;
    };
    /**
     * Returns the last child index that is covered by the clip depth.
     */
    DisplayObjectContainer.prototype.getClipDepthIndex = function (depth) {
        depth = depth | 0;
        var children = this._children;
        var index = this._children.length - 1;
        var first = true;
        for (var i = index; i >= 0; i--) {
            var child = children[i];
            // Ignore children that don't have a depth value.
            if (child._depth < 0) {
                continue;
            }
            // Usually we return the index of the first child that has a depth value less than or
            // equal to the specified depth. However, Flash seems to clip all remaining children,
            // including those that don't have a depth value if the clip appears at the end.
            if (child._depth <= depth) {
                return first ? index : i;
            }
            first = false;
        }
        return 0;
    };
    DisplayObjectContainer.prototype.getChildByName = function (name) {
        name = axCoerceString(name);
        var child = this._lookupChildByName(name, 0 /* DEFAULT */);
        if (child) {
            child._addReference();
            return child;
        }
        return null;
    };
    /**
     * Returns the child display object instance that exists at given index without creating a
     * reference nor taking ownership.
     */
    DisplayObjectContainer.prototype._lookupChildByIndex = function (index, options) {
        var child = this._children[index];
        if (child && (child._hasFlags(256 /* Constructed */) ||
            options & 2 /* INCLUDE_NON_INITIALIZED */)) {
            return child;
        }
        return null;
    };
    /**
     * Returns the child display object that exists with given name without creating a reference
     * nor taking ownership.
     */
    DisplayObjectContainer.prototype._lookupChildByName = function (name, options) {
        var children = this._children;
        if (children.length === 0) {
            return null;
        }
        for (var i = 0; i < children.length; i++) {
            var child = children[i];
            if (!child._hasFlags(256 /* Constructed */) &&
                !(options & 2 /* INCLUDE_NON_INITIALIZED */)) {
                continue;
            }
            if (child._name === name) {
                return child;
            }
        }
        if (!(options & 1 /* IGNORE_CASE */)) {
            return null;
        }
        // Trying again in non-case sensitive mode (mostly for AVM1).
        name = name.toLowerCase();
        for (var i = 0; i < children.length; i++) {
            var child = children[i];
            if (!child._hasFlags(256 /* Constructed */) &&
                !(options & 2 /* INCLUDE_NON_INITIALIZED */)) {
                continue;
            }
            if (child._name.toLowerCase() === name) {
                return child;
            }
        }
        return null;
    };
    /**
     * Override of DisplayObject#_containsPoint that takes children into consideration.
     */
    DisplayObjectContainer.prototype._containsPoint = function (globalX, globalY, localX, localY, testingType, objects) {
        return this._containsPointImpl(globalX, globalY, localX, localY, testingType, objects, false);
    };
    DisplayObjectContainer.prototype._containsPointImpl = function (globalX, globalY, localX, localY, testingType, objects, skipBoundsCheck) {
        var result;
        if (!skipBoundsCheck) {
            result = this._boundsAndMaskContainPoint(globalX, globalY, localX, localY, testingType);
            // Same as in the DisplayObject base case, we're done if we don't have a hit or are only
            // looking for bounds + mask checks.
            if (result === 0 /* None */ || testingType < 2 /* HitTestShape */) {
                return result;
            }
        }
        var anyChildHit = false;
        var children = this._getUnclippedChildren(testingType, globalX, globalY);
        for (var i = children ? children.length : 0; i--;) {
            var child = children[i];
            if (child._maskedObject) {
                continue;
            }
            result = child._containsGlobalPoint(globalX, globalY, testingType, objects);
            if (result !== 2 /* Shape */) {
                continue;
            }
            // For hit testing, a single match suffices.
            if (testingType < 3 /* Mouse */) {
                return result;
            }
            anyChildHit = true;
            if (testingType >= 4 /* ObjectsUnderPoint */) {
                continue;
            }
            release || assert(testingType === 3 /* Mouse */);
            release || assert(objects.length <= 1);
            // If this object itself is mouse-disabled, we have to ensure that no nested object is
            // returned as a result.
            if (!this._mouseEnabled) {
                objects.length = 0;
                return result;
            }
            // If this container disables mouseChildren, any matched child establish the container as
            // the match.
            if (!this._mouseChildren) {
                objects[0] = this;
            }
            if (objects.length !== 0) {
                release || assert(InteractiveObject.axIsType(objects[0]));
                return 2 /* Shape */;
            }
        }
        // We need to always test the container itself for getObjectsUnderPoint or when looking for a
        // drop target. Otherwise, it's only required if no child (interactive or not) was hit.
        if (anyChildHit && testingType < 4 /* ObjectsUnderPoint */) {
            if (testingType === 3 /* Mouse */ && objects.length === 0) {
                objects[0] = this;
            }
            return 2 /* Shape */;
        }
        var selfHit = this._containsPointDirectly(localX, localY, globalX, globalY);
        if (selfHit) {
            if (testingType === 5 /* Drop */) {
                // For Drop, replace previous hit with current one.
                objects[0] = this;
            }
            else if (testingType === 4 /* ObjectsUnderPoint */ || objects && this._mouseEnabled) {
                objects.push(this);
            }
        }
        return anyChildHit || selfHit ? 2 /* Shape */ : 0 /* None */;
    };
    DisplayObjectContainer.prototype._getUnclippedChildren = function (testingType, globalX, globalY) {
        // Clipping masks complicate hit testing: for mouse target finding, where performance is
        // most important, we want to test highest children first, then go down. OTOH, we want to
        // test clipping masks before the elements they clip, potentially saving costly tests against
        // lots of elements. However, clipping masks affect siblings with higher child indices, so
        // by going top-to-bottom, we discover them after the clipped content. To get around that,
        // we iterate over the children once and detect any clipping masks. If we find at least one,
        // we copy all the non-clipped elements into a new array, over which we then iterate without
        // having to test clipping.
        // Note: if speed is an issue, we could set a flag on containers that have at least one
        // clipping mask and do this step only if that flag is set.
        var children = this._children;
        if (!children) {
            return null;
        }
        var unclippedChildren; // Lazily created.
        for (var i = 0; children && i < children.length; i++) {
            var child = children[i];
            if (child._clipDepth !== -1) {
                if (!unclippedChildren) {
                    unclippedChildren = children.slice(0, i);
                }
                // Clipping masks are simply ignored for HitTestPoint purposes.
                if (testingType === 2 /* HitTestShape */) {
                    continue;
                }
                release || assert(testingType >= 3 /* Mouse */);
                // If the point isn't contained in the clipping mask, we can skip all the clipped objects.
                // We pass HitTestShape here because we never want to collect hit objects, which the
                // higher testing types would attempt to do.
                var containsPoint = child._containsGlobalPoint(globalX, globalY, 2 /* HitTestShape */, null);
                if (!containsPoint) {
                    i = this.getClipDepthIndex(child._clipDepth);
                }
                continue;
            }
            if (unclippedChildren) {
                unclippedChildren.push(child);
            }
        }
        return unclippedChildren || children;
    };
    /**
     * Override of DisplayObject#_getChildBounds that union all childrens's
     * bounds into the bounds.
     */
    DisplayObjectContainer.prototype._getChildBounds = function (bounds, includeStrokes) {
        var children = this._children;
        for (var i = 0; i < children.length; i++) {
            bounds.unionInPlace(children[i]._getTransformedBounds(this, includeStrokes));
        }
    };
    /**
     * Returns an array of all leaf objects under the given point in global coordinates.
     * A leaf node in this context is an object that itself contains visual content, so it can be
     * any of Shape, Sprite, MovieClip, Bitmap, Video, and TextField.
     * Note that, while the Flash documentation makes it sound like it doesn't, the result also
     * contains the receiver object if that matches the criteria above.
     */
    DisplayObjectContainer.prototype.getObjectsUnderPoint = function (globalPoint) {
        release || counter.count("DisplayObjectContainer::getObjectsUnderPoint");
        var globalX = globalPoint.x * 20 | 0;
        var globalY = globalPoint.y * 20 | 0;
        var objects = [];
        this._containsGlobalPoint(globalX, globalY, 4 /* ObjectsUnderPoint */, objects);
        // getObjectsUnderPoint returns results in exactly the opposite order we collect them in.
        return this.sec.createArrayUnsafe(objects.reverse());
    };
    DisplayObjectContainer.prototype.areInaccessibleObjectsUnderPoint = function (point) {
        point = point;
        release || notImplemented("public DisplayObjectContainer::areInaccessibleObjectsUnderPoint");
        return;
    };
    DisplayObjectContainer.prototype.contains = function (child) {
        checkParameterType(child, "child", DisplayObject.axClass);
        return this._isAncestor(child);
    };
    DisplayObjectContainer.prototype.swapChildrenAt = function (index1 /*int*/, index2 /*int*/) {
        index1 = index1 | 0;
        index2 = index2 | 0;
        var children = this._children;
        if (index1 < 0 || index1 >= children.length ||
            index2 < 0 || index2 >= children.length) {
            this.sec.throwError('RangeError', Errors.ParamRangeError);
        }
        // Always call _swapChildrenAt to make sure _setDepth(-1) is called on both children.
        this._swapChildrenAt(index1, index2);
        if (index1 !== index2) {
            this._invalidateChildren();
        }
    };
    DisplayObjectContainer.prototype._swapChildrenAt = function (index1, index2) {
        var children = this._children;
        var child1 = children[index1];
        var child2 = children[index2];
        children[index2] = child1;
        child1._setDepth(-1);
        child1._index = index2;
        children[index1] = child2;
        child2._setDepth(-1);
        child2._index = index1;
    };
    DisplayObjectContainer.prototype.swapChildren = function (child1, child2) {
        // Flash prints out 'child' for both non-null |child1| and |child2|.
        checkParameterType(child1, "child", DisplayObject.axClass);
        checkParameterType(child2, "child", DisplayObject.axClass);
        this.swapChildrenAt(this.getChildIndex(child1), this.getChildIndex(child2));
    };
    DisplayObjectContainer.prototype.removeChildren = function (beginIndex, endIndex) {
        if (beginIndex === void 0) { beginIndex = 0; }
        if (endIndex === void 0) { endIndex = 2147483647; }
        beginIndex = beginIndex | 0;
        endIndex = endIndex | 0;
        if (beginIndex < 0 || endIndex < 0 || endIndex < beginIndex || endIndex >= this._children.length) {
            this.sec.throwError('RangeError', Errors.ParamRangeError);
        }
        var count = endIndex - beginIndex + 1;
        if (count > 0) {
            while (count--) {
                this.removeChildAt(beginIndex);
            }
        }
    };
    DisplayObjectContainer.prototype.hashCode = function () {
        var hash = 0;
        for (var i = 0; i < this.numChildren; i++) {
            var child = this.getChildAt(i);
            if (child) {
                hash = mixHash(hash, this.getChildAt(i).hashCode());
            }
        }
        return mixHash(hash, this.getBounds(null).hashCode());
    };
    /**
     * This is a very slow recursive function that should not be used in performance critical code.
     */
    DisplayObjectContainer.prototype.getAncestorCount = function () {
        var count = 0;
        for (var i = 0; i < this.numChildren; i++) {
            var child = this.getChildAt(i);
            if (child) {
                count += 1 + this.getChildAt(i).getAncestorCount();
            }
        }
        return count;
    };
    DisplayObjectContainer.bindings = null;
    DisplayObjectContainer.classSymbols = null;
    DisplayObjectContainer.classInitializer = null;
    return DisplayObjectContainer;
}(InteractiveObject));
export { DisplayObjectContainer };
