import { __extends } from "tslib";
import { release, somewhatImplemented } from "../../base/utilities/Debug";
import { Event } from "../events/Event";
import { DisplayObject } from "./DisplayObject";
/**
 * Copyright 2014 Mozilla Foundation
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
// Class: InteractiveObject
var InteractiveObject = /** @class */ (function (_super) {
    __extends(InteractiveObject, _super);
    function InteractiveObject() {
        var _this = _super.call(this) || this;
        if (!_this._fieldsInitialized) {
            _this._initializeFields();
        }
        return _this;
    }
    InteractiveObject.prototype._initializeFields = function () {
        _super.prototype._initializeFields.call(this);
        this._tabEnabled = false;
        this._tabIndex = -1;
        this._focusRect = null;
        this._mouseEnabled = true;
        this._doubleClickEnabled = false;
        this._accessibilityImplementation = null;
        this._softKeyboardInputAreaOfInterest = null;
        this._needsSoftKeyboard = false;
        this._contextMenu = null;
    };
    Object.defineProperty(InteractiveObject.prototype, "tabEnabled", {
        get: function () {
            return this._tabEnabled;
        },
        set: function (enabled) {
            enabled = !!enabled;
            var old = this._tabEnabled;
            this._tabEnabled = enabled;
            if (old !== enabled) {
                this.dispatchEvent(Event.axClass.getInstance(Event.TAB_ENABLED_CHANGE, true));
            }
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(InteractiveObject.prototype, "tabIndex", {
        get: function () {
            return this._tabIndex;
        },
        set: function (index /*int*/) {
            index = index | 0;
            var old = this._tabIndex;
            this._tabIndex = index;
            if (old !== index) {
                this.dispatchEvent(Event.axClass.getInstance(Event.TAB_INDEX_CHANGE, true));
            }
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(InteractiveObject.prototype, "focusRect", {
        get: function () {
            return this._focusRect;
        },
        /**
         * The given |focusRect| can be one of: |true|, |false| or |null|.
         */
        set: function (focusRect) {
            release || somewhatImplemented("public flash.display.InteractiveObject::set focusRect");
            this._focusRect = focusRect;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(InteractiveObject.prototype, "mouseEnabled", {
        get: function () {
            return this._mouseEnabled;
        },
        set: function (enabled) {
            this._mouseEnabled = !!enabled;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(InteractiveObject.prototype, "doubleClickEnabled", {
        get: function () {
            return this._doubleClickEnabled;
        },
        set: function (enabled) {
            this._doubleClickEnabled = !!enabled;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(InteractiveObject.prototype, "accessibilityImplementation", {
        get: function () {
            return this._accessibilityImplementation;
        },
        set: function (value) {
            release || somewhatImplemented("public flash.display.InteractiveObject::set" +
                " accessibilityImplementation");
            this._accessibilityImplementation = value;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(InteractiveObject.prototype, "softKeyboardInputAreaOfInterest", {
        get: function () {
            return this._softKeyboardInputAreaOfInterest;
        },
        set: function (value) {
            release || somewhatImplemented("public flash.display.InteractiveObject::set" +
                " softKeyboardInputAreaOfInterest");
            this._softKeyboardInputAreaOfInterest = value;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(InteractiveObject.prototype, "needsSoftKeyboard", {
        get: function () {
            return this._needsSoftKeyboard;
        },
        set: function (value) {
            value = !!value;
            release || somewhatImplemented("public flash.display.InteractiveObject::set needsSoftKeyboard");
            this._needsSoftKeyboard = value;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(InteractiveObject.prototype, "contextMenu", {
        get: function () {
            return this._contextMenu;
        },
        set: function (cm) {
            cm = cm;
            release || somewhatImplemented("public flash.display.InteractiveObject::set contextMenu");
            this._contextMenu = cm;
        },
        enumerable: true,
        configurable: true
    });
    InteractiveObject.prototype.requestSoftKeyboard = function () {
        release || somewhatImplemented("public flash.display.InteractiveObject::requestSoftKeyboard");
        return false;
    };
    InteractiveObject.classInitializer = null;
    return InteractiveObject;
}(DisplayObject));
export { InteractiveObject };
