import { __extends } from "tslib";
import { Graphics } from "./Graphics";
import { ShapeSymbol } from "./Shape";
import { DisplayObject } from "./DisplayObject";
import { assert, release } from "../../base/utilities/Debug";
/**
 * Copyright 2014 Mozilla Foundation
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
// Class: MorphShape
var MorphShape = /** @class */ (function (_super) {
    __extends(MorphShape, _super);
    function MorphShape() {
        var _this = _super.call(this) || this;
        if (_this._symbol && !_this._fieldsInitialized) {
            _this.applySymbol();
        }
        release || assert(!_this._symbol);
        return _this;
    }
    MorphShape.prototype.applySymbol = function () {
        this._initializeFields();
        release || assert(this._symbol);
        this._setStaticContentFromSymbol(this._symbol);
        // TODO: Check what do do if the computed bounds of the graphics object don't
        // match those given by the symbol.
        this._setFlags(32768 /* ContainsMorph */);
    };
    MorphShape.prototype._canHaveGraphics = function () {
        return true;
    };
    MorphShape.prototype._getGraphics = function () {
        return this._graphics;
    };
    Object.defineProperty(MorphShape.prototype, "graphics", {
        get: function () {
            return this._ensureGraphics();
        },
        enumerable: true,
        configurable: true
    });
    MorphShape.prototype._containsPointDirectly = function (localX, localY, globalX, globalY) {
        var graphics = this._getGraphics();
        return graphics && graphics._containsPoint(localX, localY, true, this._ratio / 0xffff);
    };
    MorphShape.classSymbols = null; // [];
    MorphShape.instanceSymbols = null; // [];
    MorphShape.classInitializer = null;
    return MorphShape;
}(DisplayObject));
export { MorphShape };
var MorphShapeSymbol = /** @class */ (function (_super) {
    __extends(MorphShapeSymbol, _super);
    function MorphShapeSymbol(data, sec) {
        return _super.call(this, data, MorphShape.axClass) || this;
    }
    MorphShapeSymbol.FromData = function (data, loaderInfo) {
        var symbol = new MorphShapeSymbol(data, loaderInfo.sec);
        symbol._setBoundsFromData(data);
        symbol.graphics = Graphics.FromData(data, loaderInfo);
        symbol.processRequires(data.require, loaderInfo);
        symbol.morphFillBounds = data.morphFillBounds;
        symbol.morphLineBounds = data.morphLineBounds;
        return symbol;
    };
    return MorphShapeSymbol;
}(ShapeSymbol));
export { MorphShapeSymbol };
