import { __extends } from "tslib";
import { warning, abstractMethod, notImplemented } from "../../base/utilities/Debug";
import { SoundChannel } from "../media/SoundChannel";
import { Sound } from "../media/Sound";
/**
 * Copyright 2014 Mozilla Foundation
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
// Class: MovieClip
var MP3_MIME_TYPE = 'audio/mpeg';
var HTMLAudioElementAdapter = /** @class */ (function () {
    function HTMLAudioElementAdapter(sec, element) {
        this._sec = sec;
        this._element = element;
    }
    Object.defineProperty(HTMLAudioElementAdapter.prototype, "isReady", {
        get: function () {
            return !!this._channel;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(HTMLAudioElementAdapter.prototype, "element", {
        get: function () {
            return this._element;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(HTMLAudioElementAdapter.prototype, "currentTime", {
        get: function () {
            return this._element.currentTime;
        },
        enumerable: true,
        configurable: true
    });
    HTMLAudioElementAdapter.prototype.playFrom = function (time) {
        var element = this._element;
        if (element.paused) {
            element.play();
            element.addEventListener('playing', function setTime(e) {
                element.removeEventListener('playing', setTime);
                element.currentTime = time;
            });
        }
        else {
            element.currentTime = time;
        }
    };
    Object.defineProperty(HTMLAudioElementAdapter.prototype, "paused", {
        get: function () {
            return this._element.paused;
        },
        set: function (value) {
            var element = this._element;
            if (value) {
                if (!element.paused) {
                    element.pause();
                }
            }
            else {
                if (element.paused) {
                    element.play();
                }
            }
        },
        enumerable: true,
        configurable: true
    });
    HTMLAudioElementAdapter.prototype.createChannel = function () {
        this._channel = SoundChannel.initializeFromAudioElement(this._sec, this._element);
    };
    HTMLAudioElementAdapter.prototype.queueData = function (frame) {
        abstractMethod('HTMLAudioElementAdapter::queueData');
    };
    HTMLAudioElementAdapter.prototype.finish = function () {
        abstractMethod('HTMLAudioElementAdapter::finish');
    };
    return HTMLAudioElementAdapter;
}());
var MediaSourceStreamAdapter = /** @class */ (function (_super) {
    __extends(MediaSourceStreamAdapter, _super);
    function MediaSourceStreamAdapter(sec, element) {
        var _this = _super.call(this, sec, element) || this;
        _this._mediaSource = new MediaSource();
        _this._sourceBuffer = null;
        _this._updating = false;
        _this._loading = true;
        _this._rawFrames = [];
        _this._isReady = false;
        _this._mediaSource.addEventListener('sourceopen', _this._openMediaSource.bind(_this));
        _this.element.src = URL.createObjectURL(_this._mediaSource);
        return _this;
    }
    MediaSourceStreamAdapter.prototype._appendSoundData = function () {
        if (this._rawFrames.length === 0 || this._updating || !this._sourceBuffer) {
            return;
        }
        if (!this._loading) {
            this._mediaSource.endOfStream();
            return;
        }
        this._updating = true;
        // There is an issue when multiple appendBuffers are added in a sequence,
        // pushing frames one-by-one.
        this._sourceBuffer.appendBuffer(this._rawFrames.shift());
        // Making MediaSourceStreamAdapter be ready on first packet.
        if (!this._isReady) {
            this._isReady = true;
            this.createChannel();
        }
    };
    MediaSourceStreamAdapter.prototype._openMediaSource = function () {
        var sourceBuffer = this._mediaSource.addSourceBuffer(MP3_MIME_TYPE);
        sourceBuffer.addEventListener('update', function () {
            this._updating = false;
            this._appendSoundData();
        }.bind(this));
        this._sourceBuffer = sourceBuffer;
        this._appendSoundData();
    };
    MediaSourceStreamAdapter.prototype.queueData = function (frame) {
        this._rawFrames.push(frame.data);
        this._appendSoundData();
    };
    MediaSourceStreamAdapter.prototype.finish = function () {
        this._loading = false;
        this._appendSoundData();
    };
    return MediaSourceStreamAdapter;
}(HTMLAudioElementAdapter));
var BlobStreamAdapter = /** @class */ (function (_super) {
    __extends(BlobStreamAdapter, _super);
    function BlobStreamAdapter(sec, element) {
        var _this = _super.call(this, sec, element) || this;
        _this._rawFrames = [];
        return _this;
    }
    BlobStreamAdapter.prototype.queueData = function (frame) {
        this._rawFrames.push(frame.data);
    };
    BlobStreamAdapter.prototype.finish = function () {
        var blob = new Blob(this._rawFrames);
        this.element.src = URL.createObjectURL(blob);
        this.createChannel();
    };
    return BlobStreamAdapter;
}(HTMLAudioElementAdapter));
function syncTime(element, movieClip) {
    var initialized = false;
    var startMediaTime, startRealTime;
    element.addEventListener('timeupdate', function (e) {
        if (!initialized) {
            startMediaTime = element.currentTime;
            startRealTime = performance.now();
            initialized = true;
            //movieClip._stage._frameScheduler.startTrackDelta();
            return;
        }
        var mediaDelta = element.currentTime - startMediaTime;
        var realDelta = performance.now() - startRealTime;
        //movieClip._stage._frameScheduler.setDelta(realDelta - mediaDelta * 1000);
    });
    element.addEventListener('pause', function (e) {
        //movieClip._stage._frameScheduler.endTrackDelta();
        initialized = false;
    });
    element.addEventListener('seeking', function (e) {
        //movieClip._stage._frameScheduler.endTrackDelta();
        initialized = false;
    });
}
var WebAudioAdapter = /** @class */ (function () {
    function WebAudioAdapter(sec, data) {
        this._sec = sec;
        this._channel = null;
        this._sound = null;
        this._data = data;
        this._position = 0;
    }
    Object.defineProperty(WebAudioAdapter.prototype, "currentTime", {
        get: function () {
            return NaN;
        },
        enumerable: true,
        configurable: true
    });
    WebAudioAdapter.prototype.playFrom = function (time) {
        // tslint, noop
    };
    Object.defineProperty(WebAudioAdapter.prototype, "paused", {
        get: function () {
            return false;
        },
        set: function (value) {
            // tslint, noop
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(WebAudioAdapter.prototype, "isReady", {
        get: function () {
            return !!this._channel;
        },
        enumerable: true,
        configurable: true
    });
    WebAudioAdapter.prototype.queueData = function (frame) {
        this._data.pcm.set(frame.pcm, this._position);
        this._position += frame.pcm.length;
    };
    WebAudioAdapter.prototype.finish = function () {
        // TODO Start from some seek offset, stopping
        var sound = Sound.initializeFromPCMData(this._sec, this._data);
        var channel = sound.play();
        this._sound = sound;
        this._channel = channel;
    };
    return WebAudioAdapter;
}());
var WebAudioMP3Adapter = /** @class */ (function (_super) {
    __extends(WebAudioMP3Adapter, _super);
    function WebAudioMP3Adapter(sec, data) {
        var _this = _super.call(this, sec, data) || this;
        notImplemented("WebAudioMP3Adapter not implemented yet");
        return _this;
        /*
        this._decoderPosition = 0;
        this._decoderSession = new MP3DecoderSession();
        this._decoderSession.onframedata = function (frameData) {
          var position = this._decoderPosition;
          data.pcm.set(frameData, position);
          this._decoderPosition = position + frameData.length;
        }.bind(this);
        this._decoderSession.onclosed = function () {
          WebAudioAdapter.prototype.finish.call(this);
        }.bind(this);
        this._decoderSession.onerror = function (error) {
          warning('MP3DecoderSession error: ' + error);
        };*/
    }
    WebAudioMP3Adapter.prototype.queueData = function (frame) {
        this._decoderSession.pushAsync(frame.data);
    };
    WebAudioMP3Adapter.prototype.finish = function () {
        this._decoderSession.close();
    };
    return WebAudioMP3Adapter;
}(WebAudioAdapter));
var MovieClipSoundStream = /** @class */ (function () {
    function MovieClipSoundStream(streamInfo, movieClip) {
        this.movieClip = movieClip;
        this.decode = streamInfo.decode.bind(streamInfo);
        this.data = {
            sampleRate: streamInfo.sampleRate,
            channels: streamInfo.channels
        };
        this.seekIndex = [];
        this.position = 0;
        this.wasFullyLoaded = false;
        this.expectedFrame = 0;
        this.waitFor = 0;
        var sec = movieClip.sec;
        var isMP3 = streamInfo.format === 'mp3';
        /*
        if (isMP3 && !webAudioMP3Option.value) {
          var element = document.createElement('audio');
          element.preload = 'metadata'; // for mobile devices
          element.loop = false;
          syncTime(element, movieClip);
          if (element.canPlayType(MP3_MIME_TYPE)) {
            this.element = element;
            if (!mediaSourceMP3Option.value) {
              this.soundStreamAdapter = new BlobStreamAdapter(sec, element);
            } else if (typeof MediaSource !== 'undefined' &&
                        (<any>MediaSource).isTypeSupported(MP3_MIME_TYPE)) {
              this.soundStreamAdapter = new MediaSourceStreamAdapter(sec, element);
            } else {
              // Falls back to blob playback.
              warning('MediaSource is not supported');
              this.soundStreamAdapter = new BlobStreamAdapter(sec, element);
            }
            return;
          }
          // Falls through to WebAudio if element cannot play MP3.
        }
        */
        // TODO fix streamInfo.samplesCount name -- its actually average value
        var totalSamples = (streamInfo.samplesCount + 1) * this.movieClip.totalFrames * streamInfo.channels;
        this.data.pcm = new Float32Array(totalSamples);
        this.soundStreamAdapter = !isMP3 ? new WebAudioAdapter(sec, this.data) :
            new WebAudioMP3Adapter(sec, this.data);
    }
    MovieClipSoundStream.prototype.appendBlock = function (frameNum, streamBlock) {
        var decodedBlock = this.decode(streamBlock);
        var streamPosition = this.position;
        this.seekIndex[frameNum] = streamPosition + decodedBlock.seek * this.data.channels;
        this.position = streamPosition + decodedBlock.samplesCount * this.data.channels;
        this.soundStreamAdapter.queueData(decodedBlock);
    };
    MovieClipSoundStream.prototype.playFrame = function (frameNum) {
        if (isNaN(this.seekIndex[frameNum])) {
            return;
        }
        var PAUSE_WHEN_OF_SYNC_GREATER = 1.0;
        var PLAYBACK_ADJUSTMENT = 0.25;
        if (!this.wasFullyLoaded && this.movieClip._isFullyLoaded) {
            this.wasFullyLoaded = true;
            this.soundStreamAdapter.finish();
        }
        if (this.soundStreamAdapter.isReady &&
            !isNaN(this.soundStreamAdapter.currentTime)) {
            var soundStreamData = this.data;
            var time = this.seekIndex[frameNum] /
                soundStreamData.sampleRate / soundStreamData.channels;
            var elementTime = this.soundStreamAdapter.currentTime;
            if (this.expectedFrame !== frameNum) {
                this.soundStreamAdapter.playFrom(time);
            }
            else if (this.waitFor > 0) {
                if (this.waitFor <= time) {
                    this.soundStreamAdapter.paused = false;
                    this.waitFor = 0;
                }
            }
            else if (elementTime - time > PAUSE_WHEN_OF_SYNC_GREATER) {
                warning('Sound is faster than frames by ' + (elementTime - time));
                this.waitFor = elementTime - PLAYBACK_ADJUSTMENT;
                this.soundStreamAdapter.paused = true;
            }
            else if (time - elementTime > PAUSE_WHEN_OF_SYNC_GREATER) {
                warning('Sound is slower than frames by ' + (time - elementTime));
                this.soundStreamAdapter.playFrom(time + PLAYBACK_ADJUSTMENT);
            }
            this.expectedFrame = frameNum + 1;
        }
    };
    return MovieClipSoundStream;
}());
export { MovieClipSoundStream };
