import { __extends } from "tslib";
import { DisplayObject } from "./DisplayObject";
import { assert, release, warning } from "../../base/utilities/Debug";
import { Graphics } from "./Graphics";
import { DisplaySymbol } from "../symbol";
/**
 * Copyright 2014 Mozilla Foundation
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
// Class: Shape
var Shape = /** @class */ (function (_super) {
    __extends(Shape, _super);
    function Shape() {
        var _this = _super.call(this) || this;
        if (_this._symbol && !_this._fieldsInitialized) {
            _this.applySymbol();
        }
        if (!_this._fieldsInitialized) {
            _this._initializeFields();
        }
        return _this;
    }
    Shape.prototype.applySymbol = function () {
        this._initializeFields();
        release || assert(this._symbol);
        // TODO: Check what do do if the computed bounds of the graphics object don't
        // match those given by the symbol.
        this._setStaticContentFromSymbol(this._symbol);
    };
    Shape.prototype._initializeFields = function () {
        _super.prototype._initializeFields.call(this);
        this._graphics = null;
    };
    Shape.prototype._canHaveGraphics = function () {
        return true;
    };
    Shape.prototype._getGraphics = function () {
        return this._graphics;
    };
    Object.defineProperty(Shape.prototype, "graphics", {
        get: function () {
            return this._ensureGraphics();
        },
        enumerable: true,
        configurable: true
    });
    Shape.prototype._containsPointDirectly = function (localX, localY, globalX, globalY) {
        var graphics = this._getGraphics();
        return !!graphics && graphics._containsPoint(localX, localY, true, 0);
    };
    Shape.classInitializer = null;
    return Shape;
}(DisplayObject));
export { Shape };
var ShapeSymbol = /** @class */ (function (_super) {
    __extends(ShapeSymbol, _super);
    function ShapeSymbol(data, symbolClass) {
        var _this = _super.call(this, data, symbolClass, false) || this;
        _this.graphics = null;
        return _this;
    }
    ShapeSymbol.FromData = function (data, loaderInfo) {
        var symbol = new ShapeSymbol(data, Shape.axClass);
        symbol._setBoundsFromData(data);
        symbol.graphics = Graphics.FromData(data, loaderInfo);
        symbol.processRequires(data.require, loaderInfo);
        return symbol;
    };
    ShapeSymbol.prototype.processRequires = function (dependencies, loaderInfo) {
        if (!dependencies) {
            return;
        }
        var textures = this.graphics.getUsedTextures();
        for (var i = 0; i < dependencies.length; i++) {
            var symbol = loaderInfo.getSymbolById(dependencies[i]);
            if (!symbol) {
                if (dependencies[i] !== 65535) {
                    // Id 65535 is somehow used invalidly in lots of embedded shapes created by the
                    // authoring tool, so don't warn about that.
                    warning("Bitmap symbol " + dependencies[i] + " required by shape, but not defined.");
                }
                textures.push(null);
                // TODO: handle null-textures from invalid SWFs correctly.
                continue;
            }
            textures.push(symbol.getSharedInstance());
        }
    };
    return ShapeSymbol;
}(DisplaySymbol));
export { ShapeSymbol };
