import { InteractiveObject } from "./InteractiveObject";
import { DisplaySymbol, SymbolData } from "../symbol";
import { LoaderInfo } from "./LoaderInfo";
import { PlaceObjectTag } from "../../../parsers/utils/SWFTags";
import { DisplayObjectFlags, HitTestingResult, HitTestingType, DisplayObject } from "./DisplayObject";
import { Bounds } from "../../base/utilities";
import { SoundTransform } from "../media/SoundTransform";
import { AXClass } from '../../avm2/run/AXClass';
/**
 * Copyright 2014 Mozilla Foundation
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
export declare class SimpleButton extends InteractiveObject {
    static axClass: typeof SimpleButton & AXClass;
    static classInitializer: any;
    _symbol: ButtonSymbol;
    applySymbol(): void;
    protected _initializeFields(): void;
    static classSymbols: string[];
    static instanceSymbols: string[];
    constructor(upState?: DisplayObject, overState?: DisplayObject, downState?: DisplayObject, hitTestState?: DisplayObject);
    _initFrame(advance: boolean): void;
    _constructFrame(): void;
    private _useHandCursor;
    private _enabled;
    private _trackAsMenu;
    private _upState;
    private _overState;
    private _downState;
    private _hitTestState;
    private _currentState;
    get useHandCursor(): boolean;
    set useHandCursor(value: boolean);
    get enabled(): boolean;
    set enabled(value: boolean);
    get trackAsMenu(): boolean;
    set trackAsMenu(value: boolean);
    get upState(): DisplayObject;
    set upState(value: DisplayObject);
    get overState(): DisplayObject;
    set overState(value: DisplayObject);
    get downState(): DisplayObject;
    set downState(value: DisplayObject);
    get hitTestState(): DisplayObject;
    set hitTestState(value: DisplayObject);
    get soundTransform(): SoundTransform;
    set soundTransform(sndTransform: SoundTransform);
    /**
     * Override of DisplayObject#_containsPoint that applies the test on hitTestState if
     * that is defined.
     */
    _containsPoint(globalX: number, globalY: number, localX: number, localY: number, testingType: HitTestingType, objects: DisplayObject[]): HitTestingResult;
    /**
     * Override of DisplayObject#_getChildBounds that retrieves the current hitTestState's bounds.
     */
    _getChildBounds(bounds: Bounds, includeStrokes: boolean): void;
    _propagateFlagsDown(flags: DisplayObjectFlags): void;
    _updateButton(): void;
}
export declare class ButtonState {
    symbol: DisplaySymbol;
    placeObjectTag: PlaceObjectTag;
    constructor(symbol: DisplaySymbol, placeObjectTag: PlaceObjectTag);
}
export declare class ButtonSymbol extends DisplaySymbol {
    upState: ButtonState;
    overState: ButtonState;
    downState: ButtonState;
    hitTestState: ButtonState;
    loaderInfo: LoaderInfo;
    constructor(data: SymbolData, loaderInfo: LoaderInfo);
    static FromData(data: any, loaderInfo: LoaderInfo): ButtonSymbol;
}
