import { __extends } from "tslib";
import { InteractiveObject } from "./InteractiveObject";
import { assert, release, notImplemented } from "../../base/utilities/Debug";
import { SpriteSymbol } from "./Sprite";
import { DisplaySymbol } from "../symbol";
import { DisplayObject } from "./DisplayObject";
import { ActionScriptVersion } from "./ActionScriptVersion";
/**
 * Copyright 2014 Mozilla Foundation
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
// Class: SimpleButton
var SimpleButton = /** @class */ (function (_super) {
    __extends(SimpleButton, _super);
    function SimpleButton(upState, overState, downState, hitTestState) {
        var _this = _super.call(this) || this;
        if (_this._symbol && !_this._fieldsInitialized) {
            _this.applySymbol();
        }
        DisplayObject.axClass._advancableInstances.push(_this);
        if (!_this._fieldsInitialized) {
            _this._initializeFields();
        }
        if (!_this._symbol) {
            if (upState) {
                _this.upState = upState;
            }
            if (overState) {
                _this.overState = overState;
            }
            if (downState) {
                _this.downState = downState;
            }
            if (hitTestState) {
                _this.hitTestState = hitTestState;
            }
            _this._updateButton();
        }
        return _this;
    }
    SimpleButton.prototype.applySymbol = function () {
        release || assert(this._symbol);
        this._initializeFields();
        var symbol = this._symbol;
        if (symbol.upState) {
            this._upState = this.createAnimatedDisplayObject(symbol.upState.symbol, symbol.upState.placeObjectTag, true);
        }
        if (symbol.overState) {
            this._overState = this.createAnimatedDisplayObject(symbol.overState.symbol, symbol.overState.placeObjectTag, true);
        }
        if (symbol.downState) {
            this._downState = this.createAnimatedDisplayObject(symbol.downState.symbol, symbol.downState.placeObjectTag, true);
        }
        if (symbol.hitTestState) {
            this._hitTestState = this.createAnimatedDisplayObject(symbol.hitTestState.symbol, symbol.hitTestState.placeObjectTag, true);
        }
        this._updateButton();
    };
    SimpleButton.prototype._initializeFields = function () {
        _super.prototype._initializeFields.call(this);
        this._useHandCursor = true;
        this._enabled = true;
        this._trackAsMenu = false;
        this._upState = null;
        this._overState = null;
        this._downState = null;
        this._hitTestState = null;
        this._currentState = null;
        this._children = [];
    };
    SimpleButton.prototype._initFrame = function (advance) {
        if (advance) {
            this._updateButton();
        }
    };
    SimpleButton.prototype._constructFrame = function () {
        // ...
    };
    Object.defineProperty(SimpleButton.prototype, "useHandCursor", {
        get: function () {
            return this._useHandCursor;
        },
        set: function (value) {
            this._useHandCursor = !!value;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(SimpleButton.prototype, "enabled", {
        get: function () {
            return this._enabled;
        },
        set: function (value) {
            this._enabled = !!value;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(SimpleButton.prototype, "trackAsMenu", {
        get: function () {
            return this._trackAsMenu;
        },
        set: function (value) {
            value = !!value;
            release || notImplemented("public flash.display.SimpleButton::set trackAsMenu");
            return;
            // this._trackAsMenu = value;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(SimpleButton.prototype, "upState", {
        get: function () {
            return this._upState;
        },
        set: function (value) {
            var old = this._upState;
            if (value._parent) {
                value._parent.removeChild(value);
            }
            this._upState = value;
            if (this._currentState === old) {
                this._updateButton();
            }
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(SimpleButton.prototype, "overState", {
        get: function () {
            return this._overState;
        },
        set: function (value) {
            var old = this._overState;
            if (value._parent) {
                value._parent.removeChild(value);
            }
            this._overState = value;
            if (this._currentState === old) {
                this._updateButton();
            }
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(SimpleButton.prototype, "downState", {
        get: function () {
            return this._downState;
        },
        set: function (value) {
            var old = this._downState;
            if (value._parent) {
                value._parent.removeChild(value);
            }
            this._downState = value;
            if (this._currentState === old) {
                this._updateButton();
            }
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(SimpleButton.prototype, "hitTestState", {
        get: function () {
            return this._hitTestState;
        },
        set: function (value) {
            this._hitTestState = value;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(SimpleButton.prototype, "soundTransform", {
        get: function () {
            release || notImplemented("public flash.display.SimpleButton::get soundTransform");
            return;
            // return this._soundTransform;
        },
        set: function (sndTransform) {
            sndTransform = sndTransform;
            release || notImplemented("public flash.display.SimpleButton::set soundTransform");
            return;
            // this._soundTransform = sndTransform;
        },
        enumerable: true,
        configurable: true
    });
    /**
     * Override of DisplayObject#_containsPoint that applies the test on hitTestState if
     * that is defined.
     */
    SimpleButton.prototype._containsPoint = function (globalX, globalY, localX, localY, testingType, objects) {
        var target = testingType === 3 /* Mouse */ ? this._hitTestState : this._currentState;
        if (!target) {
            return 0 /* None */;
        }
        // Hit testing relies on being able to get combined transforms and all that, so, a parent.
        target._parent = this;
        var result = target._containsGlobalPoint(globalX, globalY, testingType, objects);
        target._parent = null;
        // For mouse target finding, SimpleButtons always return themselves as the hit.
        if (result !== 0 /* None */ && testingType === 3 /* Mouse */ &&
            objects && this._mouseEnabled) {
            objects[0] = this;
            release || assert(objects.length === 1);
        }
        return result;
    };
    /**
     * Override of DisplayObject#_getChildBounds that retrieves the current hitTestState's bounds.
     */
    SimpleButton.prototype._getChildBounds = function (bounds, includeStrokes) {
        if (!this._currentState) {
            return;
        }
        this._currentState._parent = this;
        bounds.unionInPlace(this._currentState._getTransformedBounds(this, includeStrokes));
        this._currentState._parent = null;
    };
    SimpleButton.prototype._propagateFlagsDown = function (flags) {
        if (this._hasFlags(flags)) {
            return;
        }
        this._setFlags(flags);
        this._upState && this._upState._propagateFlagsDown(flags);
        this._overState && this._overState._propagateFlagsDown(flags);
        this._downState && this._downState._propagateFlagsDown(flags);
        this._hitTestState && this._hitTestState._propagateFlagsDown(flags);
    };
    SimpleButton.prototype._updateButton = function () {
        var state;
        if (this._mouseOver) {
            state = this._mouseDown ? this._downState : this._overState;
        }
        else {
            state = this._upState;
        }
        if (state === this._currentState) {
            return;
        }
        if (this._currentState) {
            // TODO dispatch removedFromStage event
        }
        this._currentState = state;
        if (this._stage) {
            // TODO dispatch addedToStage event
        }
        if (state) {
            this._children[0] = state;
        }
        else {
            this._children.length = 0;
        }
        this._setDirtyFlags(2 /* DirtyChildren */);
        this._invalidateFillAndLineBounds(true, true);
    };
    // Called whenever the class is initialized.
    SimpleButton.classInitializer = null;
    // List of static symbols to link.
    SimpleButton.classSymbols = null; // [];
    // List of instance symbols to link.
    SimpleButton.instanceSymbols = null; // [];
    return SimpleButton;
}(InteractiveObject));
export { SimpleButton };
var ButtonState = /** @class */ (function () {
    function ButtonState(symbol, placeObjectTag) {
        this.symbol = symbol;
        this.placeObjectTag = placeObjectTag;
    }
    return ButtonState;
}());
export { ButtonState };
var ButtonSymbol = /** @class */ (function (_super) {
    __extends(ButtonSymbol, _super);
    function ButtonSymbol(data, loaderInfo) {
        var _this = _super.call(this, data, SimpleButton.axClass, true) || this;
        _this.upState = null;
        _this.overState = null;
        _this.downState = null;
        _this.hitTestState = null;
        _this.loaderInfo = loaderInfo;
        return _this;
    }
    ButtonSymbol.FromData = function (data, loaderInfo) {
        var symbol = new ButtonSymbol(data, loaderInfo);
        if (loaderInfo.actionScriptVersion === ActionScriptVersion.ACTIONSCRIPT2) {
            symbol.isAVM1Object = true;
        }
        var states = data.states;
        var character = null;
        var placeObjectTag;
        for (var stateName in states) {
            var controlTags = states[stateName];
            if (controlTags.length === 1) {
                placeObjectTag = controlTags[0];
                character = loaderInfo.getSymbolById(placeObjectTag.symbolId);
                if (!character) {
                    continue;
                }
            }
            else {
                placeObjectTag = {
                    code: 4 /* CODE_PLACE_OBJECT */,
                    flags: 1 /* Move */,
                    depth: 1
                };
                character = new SpriteSymbol({ id: -1, className: null, env: null }, loaderInfo);
                console.log("80pro todo: SWFFrame");
                //(<SpriteSymbol>character).frames.push(new SWFFrame(controlTags));
            }
            symbol[stateName + 'State'] = new ButtonState(character, placeObjectTag);
        }
        return symbol;
    };
    return ButtonSymbol;
}(DisplaySymbol));
export { ButtonSymbol };
