import { DisplayObjectContainer } from "./DisplayObjectContainer";
import { Rectangle } from "../geom/Rectangle";
import { DisplayObject } from "./DisplayObject";
import { InteractiveObject } from "./InteractiveObject";
import { GenericVector } from "../../avm2/natives/GenericVector";
import { MovieClip } from "./MovieClip";
import { Event } from "../events/Event";
import { ASArray } from "../../avm2/nat/ASArray";
import { Transform } from "../geom/Transform";
import { AccessibilityProperties } from "../accessibility/AccessibilityProperties";
import { AccessibilityImplementation } from "../accessibility/AccessibilityImplementation";
import { TextSnapshot } from "../text/TextSnapshot";
import { ContextMenu } from "../ui/ContextMenu";
/**
 * Copyright 2014 Mozilla Foundation
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
export declare class Stage extends DisplayObjectContainer {
    static classInitializer: any;
    static classSymbols: string[];
    static instanceSymbols: string[];
    /**
     * Indicates whether the stage object has changed since the last time it was synchronized.
     */
    _isDirty: boolean;
    constructor();
    private _frameRate;
    private _scaleMode;
    private _align;
    private _stageWidth;
    private _stageHeight;
    private _showDefaultContextMenu;
    private _focus;
    private _colorCorrection;
    private _colorCorrectionSupport;
    private _stageFocusRect;
    private _quality;
    private _displayState;
    private _fullScreenSourceRect;
    private _mouseLock;
    private _stageVideos;
    private _stage3Ds;
    private _colorARGB;
    private _fullScreenWidth;
    private _fullScreenHeight;
    private _wmodeGPU;
    private _softKeyboardRect;
    private _allowsFullScreen;
    private _allowsFullScreenInteractive;
    private _contentsScaleFactor;
    private _displayContextInfo;
    private _timeout;
    private _stageContainerWidth;
    private _stageContainerHeight;
    /**
     * The |invalidate| function was called on the stage. This flag indicates that
     * the |RENDER| event gets fired right before the stage is rendered.
     */
    private _invalidated;
    setRoot(root: MovieClip): void;
    get frameRate(): number;
    set frameRate(value: number);
    get scaleMode(): string;
    set scaleMode(value: string);
    get align(): string;
    set align(value: string);
    get stageWidth(): number;
    set stageWidth(value: number);
    _setInitialName(): void;
    /**
     * Non-AS3-available setter. In AS3, the `stageWidth` setter is silently ignored.
     */
    setStageWidth(value: number): void;
    get stageHeight(): number;
    set stageHeight(value: number);
    /**
     * Non-AS3-available setter. In AS3, the `stageHeight` setter is silently ignored.
     */
    setStageHeight(value: number): void;
    /**
     * Almost the same as color setter, except it preserves alpha channel.
     * @param value
     */
    setStageColor(value: number): void;
    /**
     * Non-AS3-available setter. Notifies the stage that the dimensions of the stage container have changed.
     */
    setStageContainerSize(width: number, height: number, pixelRatio: number): void;
    get showDefaultContextMenu(): boolean;
    set showDefaultContextMenu(value: boolean);
    get focus(): InteractiveObject;
    set focus(newFocus: InteractiveObject);
    get colorCorrection(): string;
    set colorCorrection(value: string);
    get colorCorrectionSupport(): string;
    get stageFocusRect(): boolean;
    set stageFocusRect(on: boolean);
    get quality(): string;
    set quality(value: string);
    get displayState(): string;
    set displayState(value: string);
    get fullScreenSourceRect(): Rectangle;
    set fullScreenSourceRect(value: Rectangle);
    get mouseLock(): boolean;
    set mouseLock(value: boolean);
    get stageVideos(): any;
    get stage3Ds(): GenericVector;
    get color(): number;
    set color(rgb: number);
    set alpha(alpha: number);
    get alpha(): number;
    get fullScreenWidth(): number;
    get fullScreenHeight(): number;
    get wmodeGPU(): boolean;
    get softKeyboardRect(): Rectangle;
    get allowsFullScreen(): boolean;
    get allowsFullScreenInteractive(): boolean;
    get contentsScaleFactor(): number;
    get displayContextInfo(): string;
    removeChildAt(index: number): DisplayObject;
    swapChildrenAt(index1: number, index2: number): void;
    get width(): number;
    set width(value: number);
    get height(): number;
    set height(value: number);
    get mouseChildren(): boolean;
    set mouseChildren(value: boolean);
    get numChildren(): number;
    get tabChildren(): boolean;
    set tabChildren(value: boolean);
    addChild(child: DisplayObject): DisplayObject;
    addChildAt(child: DisplayObject, index: number): DisplayObject;
    setChildIndex(child: DisplayObject, index: number): void;
    addEventListener(type: string, listener: (event: Event) => void, useCapture: boolean, priority: number, useWeakReference: boolean): void;
    hasEventListener(type: string): boolean;
    willTrigger(type: string): boolean;
    dispatchEvent(event: Event): boolean;
    invalidate(): void;
    isFocusInaccessible(): boolean;
    requireOwnerPermissions(): void;
    render(): void;
    get name(): string;
    set name(value: string);
    get mask(): DisplayObject;
    set mask(value: DisplayObject);
    get visible(): boolean;
    set visible(value: boolean);
    get x(): number;
    set x(value: number);
    get y(): number;
    set y(value: number);
    get z(): number;
    set z(value: number);
    get scaleX(): number;
    set scaleX(value: number);
    get scaleY(): number;
    set scaleY(value: number);
    get scaleZ(): number;
    set scaleZ(value: number);
    get rotation(): number;
    set rotation(value: number);
    get rotationX(): number;
    set rotationX(value: number);
    get rotationY(): number;
    set rotationY(value: number);
    get rotationZ(): number;
    set rotationZ(value: number);
    get cacheAsBitmap(): boolean;
    set cacheAsBitmap(value: boolean);
    get opaqueBackground(): any;
    set opaqueBackground(value: any);
    get scrollRect(): Rectangle;
    set scrollRect(value: Rectangle);
    get filters(): ASArray;
    set filters(value: ASArray);
    get blendMode(): string;
    set blendMode(value: string);
    get transform(): Transform;
    set transform(value: Transform);
    get accessibilityProperties(): AccessibilityProperties;
    set accessibilityProperties(value: AccessibilityProperties);
    get scale9Grid(): Rectangle;
    set scale9Grid(value: Rectangle);
    get tabEnabled(): boolean;
    set tabEnabled(value: boolean);
    get tabIndex(): number;
    set tabIndex(value: number);
    get focusRect(): any;
    set focusRect(value: any);
    get mouseEnabled(): boolean;
    set mouseEnabled(value: boolean);
    get accessibilityImplementation(): AccessibilityImplementation;
    set accessibilityImplementation(value: AccessibilityImplementation);
    get textSnapshot(): TextSnapshot;
    get contextMenu(): ContextMenu;
    set contextMenu(value: ContextMenu);
}
