import { __extends } from "tslib";
import { ASObject } from "../../avm2/nat/ASObject";
/**
 * Copyright 2014 Mozilla Foundation
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
// Class: Point
var Point = /** @class */ (function (_super) {
    __extends(Point, _super);
    function Point(x, y) {
        if (x === void 0) { x = 0; }
        if (y === void 0) { y = 0; }
        var _this = _super.call(this) || this;
        _this.x = +x;
        _this.y = +y;
        return _this;
    }
    Object.defineProperty(Point.prototype, "native_x", {
        get: function () {
            return this.x;
        },
        set: function (x) {
            this.x = x;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(Point.prototype, "native_y", {
        get: function () {
            return this.y;
        },
        set: function (y) {
            this.y = y;
        },
        enumerable: true,
        configurable: true
    });
    Point.prototype.Point = function (x, y) {
        if (x === void 0) { x = 0; }
        if (y === void 0) { y = 0; }
        this.x = x;
        this.y = y;
    };
    Object.defineProperty(Point.prototype, "length", {
        get: function () {
            return Math.sqrt(this.x * this.x + this.y * this.y);
        },
        enumerable: true,
        configurable: true
    });
    Point.interpolate = function (p1, p2, f) {
        var f1 = 1 - f;
        return new Point(p1.x * f + p2.x * f1, p1.y * f + p2.y * f1);
    };
    Point.distance = function (p1, p2) {
        var dx = p2.x - p1.x;
        var dy = p2.y - p1.y;
        return (dx === 0) ? Math.abs(dy) : (dy === 0) ? Math.abs(dx) : Math.sqrt(dx * dx + dy * dy);
    };
    Point.polar = function (length, angle) {
        length = +length;
        angle = +angle;
        return new Point(length * Math.cos(angle), length * Math.sin(angle));
    };
    Point.prototype.clone = function () {
        return new Point(this.x, this.y);
    };
    Point.prototype.offset = function (dx, dy) {
        this.x += +dx;
        this.y += +dy;
    };
    Point.prototype.equals = function (toCompare) {
        return this.x === toCompare.x && this.y === toCompare.y;
    };
    Point.prototype.subtract = function (v) {
        return new Point(this.x - v.x, this.y - v.y);
    };
    Point.prototype.add = function (v) {
        return new Point(this.x + v.x, this.y + v.y);
    };
    Point.prototype.normalize = function (thickness) {
        if (this.x !== 0 || this.y !== 0) {
            var relativeThickness = +thickness / this.length;
            this.x *= relativeThickness;
            this.y *= relativeThickness;
        }
    };
    Point.prototype.copyFrom = function (sourcePoint) {
        this.x = sourcePoint.x;
        this.y = sourcePoint.y;
    };
    Point.prototype.setTo = function (x, y) {
        this.x = +x;
        this.y = +y;
    };
    Point.prototype.toTwips = function () {
        this.x = (this.x * 20) | 0;
        this.y = (this.y * 20) | 0;
        return this;
    };
    Point.prototype.toPixels = function () {
        this.x /= 20;
        this.y /= 20;
        return this;
    };
    Point.prototype.round = function () {
        this.x = Math.round(this.x);
        this.y = Math.round(this.y);
        return this;
    };
    Point.prototype.toString = function () {
        return "(x=" + this.x + ", y=" + this.y + ")";
    };
    // Called whenever the class is initialized.
    Point.classInitializer = null;
    // List of static symbols to link.
    Point.classSymbols = null; // ["interpolate", "distance", "polar"];
    // List of instance symbols to link.
    Point.instanceSymbols = null; // ["x", "y", "length", "clone", "offset", "equals", "subtract", "add", "normalize", "copyFrom", "setTo", "toString"];
    return Point;
}(ASObject));
export { Point };
