import { __extends } from "tslib";
import { ASObject } from "../../avm2/nat/ASObject";
import { Point } from "./Point";
/**
 * Copyright 2014 Mozilla Foundation
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
// Class: Rectangle
var Rectangle = /** @class */ (function (_super) {
    __extends(Rectangle, _super);
    function Rectangle(x, y, width, height) {
        if (x === void 0) { x = 0; }
        if (y === void 0) { y = 0; }
        if (width === void 0) { width = 0; }
        if (height === void 0) { height = 0; }
        var _this = _super.call(this) || this;
        x = +x;
        y = +y;
        width = +width;
        height = +height;
        _this.x = x;
        _this.y = y;
        _this.width = width;
        _this.height = height;
        return _this;
    }
    Rectangle.FromBounds = function (bounds) {
        var xMin = bounds.xMin;
        var yMin = bounds.yMin;
        return new Rectangle(xMin / 20, yMin / 20, (bounds.xMax - xMin) / 20, (bounds.yMax - yMin) / 20);
    };
    Object.defineProperty(Rectangle.prototype, "native_x", {
        get: function () {
            return this.x;
        },
        set: function (x) {
            this.x = x;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(Rectangle.prototype, "native_y", {
        get: function () {
            return this.y;
        },
        set: function (y) {
            this.y = y;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(Rectangle.prototype, "native_width", {
        get: function () {
            return this.width;
        },
        set: function (width) {
            this.width = +width;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(Rectangle.prototype, "native_height", {
        get: function () {
            return this.height;
        },
        set: function (height) {
            this.height = +height;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(Rectangle.prototype, "left", {
        get: function () {
            return this.x;
        },
        set: function (value) {
            value = +value;
            this.width += this.x - value;
            this.x = value;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(Rectangle.prototype, "right", {
        get: function () {
            return this.x + this.width;
        },
        set: function (value) {
            value = +value;
            this.width = value - this.x;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(Rectangle.prototype, "top", {
        get: function () {
            return this.y;
        },
        set: function (value) {
            value = +value;
            this.height += this.y - value;
            this.y = value;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(Rectangle.prototype, "bottom", {
        get: function () {
            return this.y + this.height;
        },
        set: function (value) {
            value = +value;
            this.height = value - this.y;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(Rectangle.prototype, "topLeft", {
        get: function () {
            return new Point(this.left, this.top);
        },
        set: function (value) {
            this.top = value.y;
            this.left = value.x;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(Rectangle.prototype, "bottomRight", {
        get: function () {
            return new Point(this.right, this.bottom);
        },
        set: function (value) {
            this.bottom = value.y;
            this.right = value.x;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(Rectangle.prototype, "size", {
        get: function () {
            return new Point(this.width, this.height);
        },
        set: function (value) {
            this.width = value.x;
            this.height = value.y;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(Rectangle.prototype, "area", {
        get: function () {
            return this.width * this.height;
        },
        enumerable: true,
        configurable: true
    });
    Rectangle.prototype.clone = function () {
        return new Rectangle(this.x, this.y, this.width, this.height);
    };
    Rectangle.prototype.isEmpty = function () {
        return this.width <= 0 || this.height <= 0;
    };
    Rectangle.prototype.setEmpty = function () {
        this.x = 0;
        this.y = 0;
        this.width = 0;
        this.height = 0;
        return this;
    };
    Rectangle.prototype.inflate = function (dx, dy) {
        dx = +dx;
        dy = +dy;
        this.x -= dx;
        this.y -= dy;
        this.width += (dx * 2);
        this.height += (dy * 2);
    };
    Rectangle.prototype.inflatePoint = function (point) {
        this.inflate(point.x, point.y);
    };
    Rectangle.prototype.offset = function (dx, dy) {
        this.x += +dx;
        this.y += +dy;
    };
    Rectangle.prototype.offsetPoint = function (point) {
        this.offset(point.x, point.y);
    };
    Rectangle.prototype.contains = function (x, y) {
        x = +x;
        y = +y;
        return x >= this.x && x < this.right &&
            y >= this.y && y < this.bottom;
    };
    Rectangle.prototype.containsPoint = function (point) {
        return this.contains(point.x, point.y);
    };
    Rectangle.prototype.containsRect = function (rect) {
        var r1 = rect.x + rect.width;
        var b1 = rect.y + rect.height;
        var r2 = this.x + this.width;
        var b2 = this.y + this.height;
        return (rect.x >= this.x) &&
            (rect.x < r2) &&
            (rect.y >= this.y) &&
            (rect.y < b2) &&
            (r1 > this.x) &&
            (r1 <= r2) &&
            (b1 > this.y) &&
            (b1 <= b2);
    };
    Rectangle.prototype.intersection = function (toIntersect) {
        return this.clone().intersectInPlace(toIntersect);
    };
    Rectangle.prototype.intersects = function (toIntersect) {
        return Math.max(this.x, toIntersect.x) <= Math.min(this.right, toIntersect.right) &&
            Math.max(this.y, toIntersect.y) <= Math.min(this.bottom, toIntersect.bottom);
    };
    Rectangle.prototype.intersectInPlace = function (clipRect) {
        var x0 = this.x;
        var y0 = this.y;
        var x1 = clipRect.x;
        var y1 = clipRect.y;
        var l = Math.max(x0, x1);
        var r = Math.min(x0 + this.width, x1 + clipRect.width);
        if (l <= r) {
            var t = Math.max(y0, y1);
            var b = Math.min(y0 + this.height, y1 + clipRect.height);
            if (t <= b) {
                this.setTo(l, t, r - l, b - t);
                return this;
            }
        }
        this.setEmpty();
        return this;
    };
    Rectangle.prototype.intersectInPlaceInt32 = function (clipRect) {
        var x0 = this.x | 0;
        var y0 = this.y | 0;
        var w0 = this.width | 0;
        var h0 = this.height | 0;
        var x1 = clipRect.x | 0;
        var w1 = clipRect.width | 0;
        var l = Math.max(x0, x1) | 0;
        var r = Math.min(x0 + w0 | 0, x1 + w1 | 0) | 0;
        if (l <= r) {
            var y1 = clipRect.y | 0;
            var h1 = clipRect.height | 0;
            var t = Math.max(y0, y1) | 0;
            var b = Math.min(y0 + h0 | 0, y1 + h1 | 0);
            if (t <= b) {
                this.setTo(l, t, r - l, b - t);
                return this;
            }
        }
        this.setEmpty();
        return this;
    };
    Rectangle.prototype.union = function (toUnion) {
        return this.clone().unionInPlace(toUnion);
    };
    Rectangle.prototype.unionInPlace = function (toUnion) {
        if (toUnion.isEmpty()) {
            return this;
        }
        if (this.isEmpty()) {
            this.copyFrom(toUnion);
            return this;
        }
        var l = Math.min(this.x, toUnion.x);
        var t = Math.min(this.y, toUnion.y);
        this.setTo(l, t, Math.max(this.right, toUnion.right) - l, Math.max(this.bottom, toUnion.bottom) - t);
        return this;
    };
    Rectangle.prototype.equals = function (toCompare) {
        return this === toCompare ||
            this.x === toCompare.x && this.y === toCompare.y
                && this.width === toCompare.width && this.height === toCompare.height;
    };
    Rectangle.prototype.copyFrom = function (sourceRect) {
        this.x = sourceRect.x;
        this.y = sourceRect.y;
        this.width = sourceRect.width;
        this.height = sourceRect.height;
    };
    Rectangle.prototype.setTo = function (x, y, width, height) {
        this.x = +x;
        this.y = +y;
        this.width = +width;
        this.height = +height;
    };
    Rectangle.prototype.toTwips = function () {
        this.x = (this.x * 20) | 0;
        this.y = (this.y * 20) | 0;
        this.width = (this.width * 20) | 0;
        this.height = (this.height * 20) | 0;
        return this;
    };
    Rectangle.prototype.getBaseWidth = function (angle) {
        var u = Math.abs(Math.cos(angle));
        var v = Math.abs(Math.sin(angle));
        return u * this.width + v * this.height;
    };
    Rectangle.prototype.getBaseHeight = function (angle) {
        var u = Math.abs(Math.cos(angle));
        var v = Math.abs(Math.sin(angle));
        return v * this.width + u * this.height;
    };
    Rectangle.prototype.toPixels = function () {
        this.x /= 20;
        this.y /= 20;
        this.width /= 20;
        this.height /= 20;
        return this;
    };
    Rectangle.prototype.snapInPlace = function () {
        var x1 = Math.ceil(this.x + this.width);
        var y1 = Math.ceil(this.y + this.height);
        this.x = Math.floor(this.x);
        this.y = Math.floor(this.y);
        this.width = x1 - this.x;
        this.height = y1 - this.y;
        return this;
    };
    Rectangle.prototype.roundInPlace = function () {
        var x1 = Math.round(this.x + this.width);
        var y1 = Math.round(this.y + this.height);
        this.x = Math.round(this.x);
        this.y = Math.round(this.y);
        this.width = x1 - this.x;
        this.height = y1 - this.y;
        return this;
    };
    Rectangle.prototype.toString = function () {
        return "(x=" + this.x + ", y=" + this.y + ", w=" + this.width + ", h=" + this.height + ")";
    };
    Rectangle.prototype.hashCode = function () {
        var hash = 0;
        hash += this.x * 20 | 0;
        hash *= 37;
        hash += this.y * 20 | 0;
        hash *= 37;
        hash += this.width * 20 | 0;
        hash *= 37;
        hash += this.height * 20 | 0;
        return hash;
    };
    Rectangle.prototype.writeExternal = function (output) {
        output.writeFloat(this.x);
        output.writeFloat(this.y);
        output.writeFloat(this.width);
        output.writeFloat(this.height);
    };
    Rectangle.prototype.readExternal = function (input) {
        this.x = input.readFloat();
        this.y = input.readFloat();
        this.width = input.readFloat();
        this.height = input.readFloat();
    };
    // Called whenever the class is initialized.
    Rectangle.classInitializer = null;
    // List of static symbols to link.
    Rectangle.classSymbols = null; // [];
    // List of instance symbols to link.
    Rectangle.instanceSymbols = null;
    return Rectangle;
}(ASObject));
export { Rectangle };
