import { __extends } from "tslib";
import { ASObject } from "../../avm2/nat/ASObject";
/**
 * Copyright 2014 Mozilla Foundation
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
// Class: Vector3D
var Vector3D = /** @class */ (function (_super) {
    __extends(Vector3D, _super);
    function Vector3D(x, y, z, w) {
        if (x === void 0) { x = 0; }
        if (y === void 0) { y = 0; }
        if (z === void 0) { z = 0; }
        if (w === void 0) { w = 0; }
        var _this = _super.call(this) || this;
        _this.x = +x;
        _this.y = +y;
        _this.z = +z;
        _this.w = +w;
        return _this;
    }
    Vector3D.classInitializer = function () {
        this.X_AXIS = Object.freeze(this.Create(1, 0, 0, 0));
        this.Y_AXIS = Object.freeze(this.Create(1, 0, 0, 0));
        this.Z_AXIS = Object.freeze(this.Create(1, 0, 0, 0));
    };
    Vector3D.Create = function (x, y, z, w) {
        var v = Object.create(this.tPrototype);
        v.x = x;
        v.y = y;
        v.z = z;
        v.w = w;
        return v;
    };
    Object.defineProperty(Vector3D.prototype, "native_x", {
        get: function () {
            return this.x;
        },
        set: function (x) {
            this.x = x;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(Vector3D.prototype, "native_y", {
        get: function () {
            return this.y;
        },
        set: function (y) {
            this.y = y;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(Vector3D.prototype, "native_z", {
        get: function () {
            return this.z;
        },
        set: function (z) {
            this.z = z;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(Vector3D.prototype, "native_w", {
        get: function () {
            return this.w;
        },
        set: function (w) {
            this.w = w;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(Vector3D.prototype, "length", {
        get: function () {
            return Math.sqrt(this.lengthSquared);
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(Vector3D.prototype, "lengthSquared", {
        get: function () {
            return this.x * this.x +
                this.y * this.y +
                this.z * this.z;
        },
        enumerable: true,
        configurable: true
    });
    Vector3D.angleBetween = function (a, b) {
        // http://chemistry.about.com/od/workedchemistryproblems/a/scalar-product-vectors-problem.htm
        return Math.acos(a.dotProduct(b) / (a.length * b.length));
    };
    Vector3D.distance = function (pt1, pt2) {
        // http://en.wikipedia.org/wiki/Euclidean_distance#Three_dimensions
        return pt1.subtract(pt2).length;
    };
    Vector3D.prototype.dotProduct = function (a) {
        return this.x * a.x +
            this.y * a.y +
            this.z * a.z;
    };
    Vector3D.prototype.crossProduct = function (a) {
        return new Vector3D(this.y * a.z - this.z * a.y, this.z * a.x - this.x * a.z, this.x * a.y - this.y * a.x, 1.0);
    };
    Vector3D.prototype.normalize = function () {
        var length = this.length;
        if (length !== 0) {
            this.x /= length;
            this.y /= length;
            this.z /= length;
        }
        else {
            this.x = this.y = this.z = 0;
        }
        return length;
    };
    Vector3D.prototype.scaleBy = function (s) {
        s = +s;
        this.x *= s;
        this.y *= s;
        this.z *= s;
    };
    Vector3D.prototype.incrementBy = function (a) {
        this.x += a.x;
        this.y += a.y;
        this.z += a.z;
    };
    Vector3D.prototype.decrementBy = function (a) {
        this.x -= a.x;
        this.y -= a.y;
        this.z -= a.z;
    };
    Vector3D.prototype.add = function (a) {
        return new Vector3D(this.x + a.x, this.y + a.y, this.z + a.z);
    };
    Vector3D.prototype.subtract = function (a) {
        return new Vector3D(this.x - a.x, this.y - a.y, this.z - a.z);
    };
    Vector3D.prototype.negate = function () {
        this.x = -this.x;
        this.y = -this.y;
        this.z = -this.z;
    };
    Vector3D.prototype.equals = function (toCompare, allFour) {
        return (this.x === toCompare.x)
            && (this.y === toCompare.y)
            && (this.z === toCompare.z)
            && (!allFour || (this.w === toCompare.w));
    };
    Vector3D.prototype.nearEquals = function (toCompare, tolerance, allFour) {
        return (Math.abs(this.x - toCompare.x) < tolerance)
            && (Math.abs(this.y - toCompare.y) < tolerance)
            && (Math.abs(this.z - toCompare.z) < tolerance)
            && (!allFour || (Math.abs(this.w - toCompare.w) < tolerance));
    };
    Vector3D.prototype.project = function () {
        this.x /= this.w;
        this.y /= this.w;
        this.z /= this.w;
    };
    Vector3D.prototype.copyFrom = function (sourceVector3D) {
        this.x = sourceVector3D.x;
        this.y = sourceVector3D.y;
        this.z = sourceVector3D.z;
    };
    Vector3D.prototype.setTo = function (xa, ya, za) {
        this.x = +xa;
        this.y = +ya;
        this.z = +za;
    };
    Vector3D.prototype.clone = function () {
        return new Vector3D(this.x, this.y, this.z, this.w);
    };
    Vector3D.prototype.toString = function () {
        return "Vector3D(" + this.x + ", " + this.y + ", " + this.z + ")";
    };
    return Vector3D;
}(ASObject));
export { Vector3D };
