import { LoaderInfo } from "../display/LoaderInfo";
import { ISecurityDomain } from "../../avm2/nat/ISecurityDomain";
import { SymbolData, Symbol } from "../symbol";
import { EventDispatcher } from "../events/EventDispatcher";
import { ID3Info } from "./ID3Info";
import { URLRequest } from "../net/URLRequest";
import { SoundLoaderContext } from "./SoundLoaderContext";
import { ByteArray } from "../../avm2/natives/byteArray";
import { SoundChannel } from "./SoundChannel";
import { SoundTransform } from "./SoundTransform";
export declare class Sound extends EventDispatcher {
    static classInitializer: any;
    _symbol: SoundSymbol;
    applySymbol(): void;
    static initializeFromPCMData(sec: ISecurityDomain, data: any): Sound;
    static classSymbols: string[];
    static instanceSymbols: string[];
    constructor(stream?: URLRequest, context?: SoundLoaderContext);
    private _playQueue;
    private _soundData;
    private _stream;
    private _url;
    _isURLInaccessible: boolean;
    private _length;
    _isBuffering: boolean;
    private _bytesLoaded;
    private _bytesTotal;
    private _id3;
    get url(): string;
    get isURLInaccessible(): boolean;
    get length(): number;
    get isBuffering(): boolean;
    get bytesLoaded(): number;
    get bytesTotal(): number;
    get id3(): ID3Info;
    loadCompressedDataFromByteArray(bytes: ByteArray, bytesLength: number): void;
    loadPCMFromByteArray(bytes: ByteArray, samples: number, format?: string, stereo?: boolean, sampleRate?: number): void;
    play(startTime?: number, loops?: number, sndTransform?: SoundTransform): SoundChannel;
    close(): void;
    extract(target: ByteArray, length: number, startPosition?: number): number;
    load(request: URLRequest, context?: SoundLoaderContext): void;
}
export declare class SoundSymbol extends Symbol {
    channels: number;
    sampleRate: number;
    pcm: Float32Array;
    packaged: any;
    constructor(data: SymbolData, sec: ISecurityDomain);
    static FromData(data: any, loaderInfo: LoaderInfo): SoundSymbol;
}
