import { __extends } from "tslib";
import { Symbol } from "../symbol";
import { EventDispatcher } from "../events/EventDispatcher";
import { assert, release, somewhatImplemented, notImplemented } from "../../base/utilities/Debug";
import { ID3Info } from "./ID3Info";
import { axCoerceString } from "../../avm2/run/axCoerceString";
function getAudioDescription(soundData, onComplete) {
    var audioElement = document.createElement('audio');
    if (!audioElement.canPlayType(soundData.mimeType)) {
        onComplete({
            duration: 0
        });
        return;
    }
    audioElement.preload = 'metadata'; // for mobile devices
    var blob = new Blob([soundData.data], { type: soundData.mimeType });
    audioElement.src = URL.createObjectURL(blob);
    audioElement.load();
    audioElement.addEventListener("loadedmetadata", function () {
        onComplete({
            duration: this.duration * 1000
        });
    });
}
var SoundData = /** @class */ (function () {
    function SoundData() {
    }
    return SoundData;
}());
var Sound = /** @class */ (function (_super) {
    __extends(Sound, _super);
    function Sound(stream, context) {
        var _this = _super.call(this) || this;
        if (_this._symbol) {
            _this.applySymbol();
        }
        //Telemetry.instance.reportTelemetry({topic: 'feature', feature: Telemetry.Feature.SOUND_FEATURE});
        _this._playQueue = [];
        _this._url = null;
        _this._length = 0;
        _this._bytesTotal = 0;
        _this._bytesLoaded = 0;
        _this._id3 = new ID3Info();
        _this._isURLInaccessible = false;
        _this._isBuffering = false;
        _this.load(stream, context);
        return _this;
    }
    Sound.prototype.applySymbol = function () {
        release || assert(this._symbol);
        this._playQueue = [];
        this._url = null;
        this._length = 0;
        this._bytesTotal = 0;
        this._bytesLoaded = 0;
        this._id3 = new ID3Info();
        this._isURLInaccessible = false;
        this._isBuffering = false;
        var symbol = this._symbol;
        if (symbol) {
            var soundData = new SoundData();
            soundData.sampleRate = symbol.sampleRate;
            soundData.channels = symbol.channels;
            soundData.completed = true;
            if (symbol.pcm) {
                soundData.pcm = symbol.pcm;
                soundData.end = symbol.pcm.length;
            }
            if (symbol.packaged) {
                soundData.data = symbol.packaged.data.buffer;
                soundData.mimeType = symbol.packaged.mimeType;
            }
            var self = this;
            getAudioDescription(soundData, function (description) {
                self._length = description.duration;
            });
            this._soundData = soundData;
        }
    };
    Sound.initializeFromPCMData = function (sec, data) {
        var sound = new Sound();
        sound._symbol = data;
        sound.applySymbol();
        return sound;
    };
    Object.defineProperty(Sound.prototype, "url", {
        get: function () {
            return this._url;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(Sound.prototype, "isURLInaccessible", {
        get: function () {
            release || somewhatImplemented("public flash.media.Sound::get isURLInaccessible");
            return this._isURLInaccessible;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(Sound.prototype, "length", {
        get: function () {
            return this._length;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(Sound.prototype, "isBuffering", {
        get: function () {
            release || somewhatImplemented("public flash.media.Sound::get isBuffering");
            return this._isBuffering;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(Sound.prototype, "bytesLoaded", {
        get: function () {
            return this._bytesLoaded;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(Sound.prototype, "bytesTotal", {
        get: function () {
            return this._bytesTotal;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(Sound.prototype, "id3", {
        get: function () {
            return this._id3;
        },
        enumerable: true,
        configurable: true
    });
    Sound.prototype.loadCompressedDataFromByteArray = function (bytes, bytesLength /*uint*/) {
        bytes = bytes;
        bytesLength = bytesLength >>> 0;
        release || notImplemented("public flash.media.Sound::loadCompressedDataFromByteArray");
        return;
    };
    Sound.prototype.loadPCMFromByteArray = function (bytes, samples /*uint*/, format, stereo, sampleRate) {
        if (format === void 0) { format = "float"; }
        if (stereo === void 0) { stereo = true; }
        if (sampleRate === void 0) { sampleRate = 44100; }
        bytes = bytes;
        samples = samples >>> 0;
        format = axCoerceString(format);
        stereo = !!stereo;
        sampleRate = +sampleRate;
        release || notImplemented("public flash.media.Sound::loadPCMFromByteArray");
        return;
    };
    Sound.prototype.play = function (startTime, loops, sndTransform) {
        if (startTime === void 0) { startTime = 0; }
        if (loops === void 0) { loops = 0; }
        if (sndTransform === void 0) { sndTransform = null; }
        console.log("TODO: Sound.play");
        /*startTime = +startTime; loops = loops | 0;
        var channel = new SoundChannel();
        channel._sound = this;
        channel._soundTransform = isNullOrUndefined(sndTransform) ?
                                  new SoundTransform() :
                                  sndTransform;
        this._playQueue.push({
          channel: channel,
          startTime: startTime
        });
        if (disableAudioOption.value) {
          return channel;
        }
        if (this._soundData) {
          if (!webAudioOption.value && !webAudioMP3Option.value) {
            channel._playSoundDataViaAudio(this._soundData, startTime, loops);
          } else if (!this._soundData.pcm) {
            if (this._soundData.mimeType === 'audio/mpeg' && webAudioMP3Option.value) {
              SWF.MP3DecoderSession.processAll(new Uint8Array(this._soundData.data)).then(function (result) {
                this._soundData.pcm = result.data;
                this._soundData.end = result.data.length;
                channel._playSoundDataViaChannel(this._soundData, startTime, loops);
              }.bind(this), function (reason) {
                warning('Unable to decode MP3 data: ' + reason);
              });
            } else {
              warning('Unable to decode packaged sound data of type: ' + this._soundData.mimeType);
            }
          } else {
            channel._playSoundDataViaChannel(this._soundData, startTime, loops);
          }
        }*/
        return null; //channel;
    };
    Sound.prototype.close = function () {
        release || somewhatImplemented("public flash.media.Sound::close");
    };
    Sound.prototype.extract = function (target, length, startPosition) {
        if (startPosition === void 0) { startPosition = -1; }
        target = target;
        length = +length;
        startPosition = +startPosition;
        release || notImplemented("public flash.media.Sound::extract");
        return;
    };
    Sound.prototype.load = function (request, context) {
        if (!request) {
            return;
        }
        console.log("TODO:Sound.load");
        /*
            var checkPolicyFile: boolean = context ? context.checkPolicyFile : false;
            var bufferTime: number = context ? context.bufferTime : 1000;
        
            var _myThis = this;
            var stream = this._stream = new URLStream();
            var data = new ByteArray();
            var dataPosition = 0;
            var playUsingWebAudio = webAudioOption.value;
            var mp3DecodingSession = null;
            var soundData = new SoundData();
            soundData.completed = false;
        
            stream.addEventListener("progress", function (event) {
              _myThis._bytesLoaded = event.axGetPublicProperty("bytesLoaded");
              _myThis._bytesTotal = event.axGetPublicProperty("bytesTotal");
        
              if (playUsingWebAudio && !mp3DecodingSession) {
                // initialize MP3 decoding
                mp3DecodingSession = decodeMP3(soundData, function (duration, final) {
                  if (_myThis._length === 0) {
                    // once we have some data, trying to play it
                    _myThis._soundData = soundData;
        
                    _myThis._playQueue.forEach(function (item) {
                      item.channel._playSoundDataViaChannel(soundData, item.startTime);
                    });
                  }
                  // estimate duration based on bytesTotal and current loaded data time
                  _myThis._length = final ? duration * 1000 : Math.max(duration,
                    mp3DecodingSession.estimateDuration(_myThis._bytesTotal)) * 1000;
                });
              }
        
              var bytesAvailable = stream.bytesAvailable;
              stream.readBytes(data, dataPosition, bytesAvailable);
              if (mp3DecodingSession) {
                mp3DecodingSession.pushData(new Uint8Array((<any> data)._buffer, dataPosition, bytesAvailable));
              }
              dataPosition += bytesAvailable;
        
              _this.dispatchEvent(event);
            });
        
            stream.addEventListener("complete", function (event) {
              _this.dispatchEvent(event);
              soundData.data = (<any> data)._buffer;
              soundData.mimeType = 'audio/mpeg';
              soundData.completed = true;
        
              if (!playUsingWebAudio) {
                _this._soundData = soundData;
        
                getAudioDescription(soundData, function (description) {
                  _this._length = description.duration;
                });
        
                _this._playQueue.forEach(function (item) {
                  item.channel._playSoundDataViaAudio(soundData, item.startTime);
                });
              }
        
              if (mp3DecodingSession) {
                mp3DecodingSession.close();
              }
            });
        
            stream.load(request);
              */
    };
    // Called whenever the class is initialized.
    Sound.classInitializer = null;
    // List of static symbols to link.
    Sound.classSymbols = null; // [];
    // List of instance symbols to link.
    Sound.instanceSymbols = null; // ["load"];
    return Sound;
}(EventDispatcher));
export { Sound };
var SoundSymbol = /** @class */ (function (_super) {
    __extends(SoundSymbol, _super);
    function SoundSymbol(data, sec) {
        return _super.call(this, data, Sound.axClass) || this;
    }
    SoundSymbol.FromData = function (data, loaderInfo) {
        var symbol = new SoundSymbol(data, loaderInfo.sec);
        symbol.channels = data.channels;
        symbol.sampleRate = data.sampleRate;
        symbol.pcm = data.pcm;
        symbol.packaged = data.packaged;
        return symbol;
    };
    return SoundSymbol;
}(Symbol));
export { SoundSymbol };
