import { __extends } from "tslib";
import { somewhatImplemented, release } from "../../base/utilities/Debug";
import { ASObject } from "../../avm2/nat/ASObject";
/**
 * Copyright 2014 Mozilla Foundation
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
// Class: SoundTransform
var SoundTransform = /** @class */ (function (_super) {
    __extends(SoundTransform, _super);
    function SoundTransform(vol, panning) {
        if (vol === void 0) { vol = 1; }
        if (panning === void 0) { panning = 0; }
        var _this = _super.call(this) || this;
        _this.volume = +vol;
        _this.pan = +panning;
        return _this;
    }
    Object.defineProperty(SoundTransform.prototype, "volume", {
        get: function () {
            return this._volume;
        },
        set: function (volume) {
            volume = +volume;
            this._volume = volume;
            this._updateTransform();
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(SoundTransform.prototype, "leftToLeft", {
        get: function () {
            return this._leftToLeft;
        },
        set: function (leftToLeft) {
            leftToLeft = +leftToLeft;
            this._leftToLeft = leftToLeft;
            this._updateTransform();
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(SoundTransform.prototype, "leftToRight", {
        get: function () {
            return this._leftToRight;
        },
        set: function (leftToRight) {
            leftToRight = +leftToRight;
            this._leftToRight = leftToRight;
            this._updateTransform();
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(SoundTransform.prototype, "rightToRight", {
        get: function () {
            return this._rightToRight;
        },
        set: function (rightToRight) {
            rightToRight = +rightToRight;
            this._rightToRight = rightToRight;
            this._updateTransform();
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(SoundTransform.prototype, "rightToLeft", {
        get: function () {
            return this._rightToLeft;
        },
        set: function (rightToLeft) {
            rightToLeft = +rightToLeft;
            this._rightToLeft = rightToLeft;
            this._updateTransform();
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(SoundTransform.prototype, "pan", {
        get: function () {
            if (this._leftToRight === 0 && this._rightToLeft === 0) {
                return 1 - this._leftToLeft * this._leftToLeft;
            }
            return 0;
        },
        set: function (panning) {
            this.leftToLeft = Math.sqrt(1 - panning);
            this.leftToRight = 0;
            this.rightToRight = Math.sqrt(1 + panning);
            this.rightToLeft = 0;
        },
        enumerable: true,
        configurable: true
    });
    SoundTransform.prototype._updateTransform = function () {
        release || somewhatImplemented("public flash.media.SoundTransform::_updateTransform");
        // TODO dispatch updates to the current audio destinations?
    };
    // Called whenever the class is initialized.
    SoundTransform.classInitializer = null;
    // List of static symbols to link.
    SoundTransform.classSymbols = null; // [];
    // List of instance symbols to link.
    SoundTransform.instanceSymbols = null; // [];
    return SoundTransform;
}(ASObject));
export { SoundTransform };
