import { __extends } from "tslib";
import { DisplayObject } from "../display/DisplayObject";
import { release, assert, somewhatImplemented, notImplemented } from "../../base/utilities/Debug";
import { DisplaySymbol } from "../symbol";
/**
 * Copyright 2014 Mozilla Foundation
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
// Class: Video
var Video = /** @class */ (function (_super) {
    __extends(Video, _super);
    function Video(width, height) {
        if (width === void 0) { width = 320; }
        if (height === void 0) { height = 240; }
        var _this = this;
        width |= 0;
        height |= 0;
        _this = _super.call(this) || this;
        if (_this._symbol && !_this._fieldsInitialized) {
            _this.applySymbol();
        }
        if (!_this._fieldsInitialized) {
            _this._initializeFields();
        }
        if (!_this._symbol) {
            width = width || 320;
            height = height || 240;
            _this._setFillAndLineBoundsFromWidthAndHeight(width * 20, height * 20);
        }
        return _this;
    }
    Video.prototype.applySymbol = function () {
        this._initializeFields();
        var symbol = this._symbol;
        this._deblocking = symbol.deblocking;
        this._smoothing = symbol.smoothing;
        this._setFillAndLineBoundsFromWidthAndHeight(symbol.width * 20, symbol.height * 20);
    };
    Video.prototype._initializeFields = function () {
        _super.prototype._initializeFields.call(this);
        this._deblocking = 0;
        this._smoothing = false;
        this._videoWidth = 0;
        this._videoHeight = 0;
    };
    Object.defineProperty(Video.prototype, "deblocking", {
        get: function () {
            return this._deblocking;
        },
        set: function (value /*int*/) {
            this._deblocking = value | 0;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(Video.prototype, "smoothing", {
        get: function () {
            return this._smoothing;
        },
        set: function (value) {
            this._smoothing = !!value;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(Video.prototype, "videoWidth", {
        get: function () {
            return this._videoWidth;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(Video.prototype, "videoHeight", {
        get: function () {
            return this._videoHeight;
        },
        enumerable: true,
        configurable: true
    });
    Video.prototype._containsPointDirectly = function (localX, localY, globalX, globalY) {
        // If this override is reached, the content bounds have already been checked, which is all
        // we need to do.
        release || assert(this._getContentBounds().contains(localX, localY));
        return true;
    };
    Video.prototype.clear = function () {
        release || somewhatImplemented("public flash.media.Video::clear");
        return;
    };
    Video.prototype.attachNetStream = function (netStream) {
        console.log("80pro: todo: attachNetStream");
        /*
        if (this._netStream === netStream) {
          return;
        }
        if (this._netStream) {
          this._netStream._videoReferrer = null;
        }
        this._netStream = netStream;
        if (this._netStream) {
          netStream._videoReferrer = this;
        }
        this._setDirtyFlags(DisplayObjectDirtyFlags.DirtyNetStream);*/
    };
    Video.prototype.attachCamera = function (camera) {
        release || notImplemented("public flash.media.Video::attachCamera");
        return;
    };
    Video.classInitializer = null;
    Video.classSymbols = null;
    Video.instanceSymbols = null;
    return Video;
}(DisplayObject));
export { Video };
var VideoSymbol = /** @class */ (function (_super) {
    __extends(VideoSymbol, _super);
    function VideoSymbol(data, sec) {
        return _super.call(this, data, Video.axClass, true) || this;
    }
    VideoSymbol.FromData = function (data, loaderInfo) {
        var symbol = new VideoSymbol(data, loaderInfo.sec);
        symbol.width = data.width;
        symbol.height = data.height;
        symbol.deblocking = data.deblocking;
        symbol.smoothing = data.smoothing;
        symbol.codec = data.codec;
        return symbol;
    };
    return VideoSymbol;
}(DisplaySymbol));
export { VideoSymbol };
