import { __extends } from "tslib";
import { EventDispatcher } from "../events/EventDispatcher";
import { axCoerceString } from "../../avm2/run/axCoerceString";
import { DisplayObject } from "../display/DisplayObject";
import { SoundTransform } from "../media/SoundTransform";
import { SoundMixer } from "../media/SoundMixer";
import { NetStreamInfo } from "./NetStreamInfo";
import { notImplemented, release, somewhatImplemented, assert } from "../../base/utilities/Debug";
import { NetStatusEvent } from "../events/NetStatusEvent";
import { PromiseWrapper } from "../../base/utilities";
var NetStream = /** @class */ (function (_super) {
    __extends(NetStream, _super);
    function NetStream(connection, peerID) {
        if (peerID === void 0) { peerID = "connectToFMS"; }
        var _this = _super.call(this) || this;
        _this._connection = connection;
        _this._peerID = axCoerceString(peerID);
        _this._id = DisplayObject.getNextSyncID();
        _this._isDirty = true;
        _this._soundTransform = new SoundTransform();
        _this._contentTypeHint = null;
        _this._checkPolicyFile = true;
        _this._videoStream = new VideoStream(_this);
        _this._videoStream._onEnsurePlay = function () {
            this._notifyVideoControl(9 /* EnsurePlaying */, null);
        }.bind(_this);
        _this._resourceName = null;
        _this._metaData = null;
        return _this;
    }
    // _multicastPushNeighborLimit: number;
    // _multicastWindowDuration: number;
    // _multicastRelayMarginDuration: number;
    // _multicastAvailabilityUpdatePeriod: number;
    // _multicastFetchPeriod: number;
    // _multicastAvailabilitySendToAll: boolean;
    // _farID: string;
    // _nearNonce: string;
    // _farNonce: string;
    // _peerStreams: any [];
    // _audioReliable: boolean;
    // _videoReliable: boolean;
    // _dataReliable: boolean;
    // _audioSampleAccess: boolean;
    // _videoSampleAccess: boolean;
    // _useHardwareDecoder: boolean;
    // _useJitterBuffer: boolean;
    // _videoStreamSettings: flash.media.VideoStreamSettings;
    NetStream.prototype.dispose = function () {
        release || notImplemented("public flash.net.NetStream::dispose");
        return;
    };
    NetStream.prototype._getVideoStreamURL = function () {
        return this._videoStream.url;
    };
    NetStream.prototype.play = function (url) {
        SoundMixer._registerSoundSource(this);
        // (void) -> void ???
        url = axCoerceString(url);
        var service = this.sec.player;
        service.registerEventListener(this._id, this.processVideoEvent.bind(this));
        if (this._connection && this._connection.uri) {
            this._videoStream.playInConnection(this._connection, url);
        }
        else if (url === null) {
            this._videoStream.openInDataGenerationMode();
        }
        else {
            this._videoStream.play(url, this.checkPolicyFile);
        }
        this._notifyVideoControl(1 /* Init */, {
            url: this._videoStream.url
        });
    };
    NetStream.prototype.play2 = function (param) {
        param = param;
        release || notImplemented("public flash.net.NetStream::play2");
        return;
    };
    Object.defineProperty(NetStream.prototype, "info", {
        get: function () {
            release || somewhatImplemented("public flash.net.NetStream::get info");
            var bufferSeconds = 1;
            var playedSeconds = Math.ceil(this._invoke(304, null));
            var audioBytesPerSecond = 32;
            var videoBytesPerSecond = 200;
            var dataBytesPerSecond = 1;
            return new NetStreamInfo(audioBytesPerSecond + videoBytesPerSecond, (audioBytesPerSecond + videoBytesPerSecond + dataBytesPerSecond) * (bufferSeconds + playedSeconds), audioBytesPerSecond + videoBytesPerSecond, audioBytesPerSecond, audioBytesPerSecond * (bufferSeconds + playedSeconds), videoBytesPerSecond, videoBytesPerSecond * (bufferSeconds + playedSeconds), dataBytesPerSecond, dataBytesPerSecond * (bufferSeconds + playedSeconds), (audioBytesPerSecond + videoBytesPerSecond + dataBytesPerSecond) * playedSeconds, 0, audioBytesPerSecond * bufferSeconds, videoBytesPerSecond * bufferSeconds, dataBytesPerSecond * bufferSeconds, bufferSeconds, bufferSeconds, bufferSeconds, 0, 0, 0, this._metaData, null, this._connection.uri, this._resourceName, false);
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(NetStream.prototype, "multicastInfo", {
        get: function () {
            release || notImplemented("public flash.net.NetStream::get multicastInfo");
            return;
            // return this._multicastInfo;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(NetStream.prototype, "soundTransform", {
        get: function () {
            return this._soundTransform;
        },
        set: function (sndTransform) {
            this._soundTransform = sndTransform;
            SoundMixer._updateSoundSource(this);
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(NetStream.prototype, "checkPolicyFile", {
        get: function () {
            return this._checkPolicyFile;
        },
        set: function (state) {
            state = !!state;
            this._checkPolicyFile = state;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(NetStream.prototype, "client", {
        get: function () {
            return this._client;
        },
        set: function (object) {
            release || somewhatImplemented("public flash.net.NetStream::set client");
            this._client = object;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(NetStream.prototype, "objectEncoding", {
        get: function () {
            release || notImplemented("public flash.net.NetStream::get objectEncoding");
            return;
            // return this._objectEncoding;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(NetStream.prototype, "multicastPushNeighborLimit", {
        get: function () {
            release || notImplemented("public flash.net.NetStream::get multicastPushNeighborLimit");
            return;
            // return this._multicastPushNeighborLimit;
        },
        set: function (neighbors) {
            neighbors = +neighbors;
            release || notImplemented("public flash.net.NetStream::set multicastPushNeighborLimit");
            return;
            // this._multicastPushNeighborLimit = neighbors;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(NetStream.prototype, "multicastWindowDuration", {
        get: function () {
            release || notImplemented("public flash.net.NetStream::get multicastWindowDuration");
            return;
            // return this._multicastWindowDuration;
        },
        set: function (seconds) {
            seconds = +seconds;
            release || notImplemented("public flash.net.NetStream::set multicastWindowDuration");
            return;
            // this._multicastWindowDuration = seconds;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(NetStream.prototype, "multicastRelayMarginDuration", {
        get: function () {
            release || notImplemented("public flash.net.NetStream::get multicastRelayMarginDuration");
            return;
            // return this._multicastRelayMarginDuration;
        },
        set: function (seconds) {
            seconds = +seconds;
            release || notImplemented("public flash.net.NetStream::set multicastRelayMarginDuration");
            return;
            // this._multicastRelayMarginDuration = seconds;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(NetStream.prototype, "multicastAvailabilityUpdatePeriod", {
        get: function () {
            release || notImplemented("public flash.net.NetStream::get multicastAvailabilityUpdatePeriod");
            return;
            // return this._multicastAvailabilityUpdatePeriod;
        },
        set: function (seconds) {
            seconds = +seconds;
            release || notImplemented("public flash.net.NetStream::set multicastAvailabilityUpdatePeriod");
            return;
            // this._multicastAvailabilityUpdatePeriod = seconds;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(NetStream.prototype, "multicastFetchPeriod", {
        get: function () {
            release || notImplemented("public flash.net.NetStream::get multicastFetchPeriod");
            return;
            // return this._multicastFetchPeriod;
        },
        set: function (seconds) {
            seconds = +seconds;
            release || notImplemented("public flash.net.NetStream::set multicastFetchPeriod");
            return;
            // this._multicastFetchPeriod = seconds;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(NetStream.prototype, "multicastAvailabilitySendToAll", {
        get: function () {
            release || notImplemented("public flash.net.NetStream::get multicastAvailabilitySendToAll");
            return;
            // return this._multicastAvailabilitySendToAll;
        },
        set: function (value) {
            value = !!value;
            release || notImplemented("public flash.net.NetStream::set multicastAvailabilitySendToAll");
            return;
            // this._multicastAvailabilitySendToAll = value;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(NetStream.prototype, "farID", {
        get: function () {
            release || notImplemented("public flash.net.NetStream::get farID");
            return;
            // return this._farID;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(NetStream.prototype, "nearNonce", {
        get: function () {
            release || notImplemented("public flash.net.NetStream::get nearNonce");
            return;
            // return this._nearNonce;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(NetStream.prototype, "farNonce", {
        get: function () {
            release || notImplemented("public flash.net.NetStream::get farNonce");
            return;
            // return this._farNonce;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(NetStream.prototype, "peerStreams", {
        get: function () {
            release || notImplemented("public flash.net.NetStream::get peerStreams");
            return;
            // return this._peerStreams;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(NetStream.prototype, "audioReliable", {
        get: function () {
            release || notImplemented("public flash.net.NetStream::get audioReliable");
            return;
            // return this._audioReliable;
        },
        set: function (reliable) {
            reliable = !!reliable;
            release || notImplemented("public flash.net.NetStream::set audioReliable");
            return;
            // this._audioReliable = reliable;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(NetStream.prototype, "videoReliable", {
        get: function () {
            release || notImplemented("public flash.net.NetStream::get videoReliable");
            return;
            // return this._videoReliable;
        },
        set: function (reliable) {
            reliable = !!reliable;
            release || notImplemented("public flash.net.NetStream::set videoReliable");
            return;
            // this._videoReliable = reliable;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(NetStream.prototype, "dataReliable", {
        get: function () {
            release || notImplemented("public flash.net.NetStream::get dataReliable");
            return;
            // return this._dataReliable;
        },
        set: function (reliable) {
            reliable = !!reliable;
            release || notImplemented("public flash.net.NetStream::set dataReliable");
            return;
            // this._dataReliable = reliable;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(NetStream.prototype, "audioSampleAccess", {
        get: function () {
            release || notImplemented("public flash.net.NetStream::get audioSampleAccess");
            return;
            // return this._audioSampleAccess;
        },
        set: function (reliable) {
            reliable = !!reliable;
            release || notImplemented("public flash.net.NetStream::set audioSampleAccess");
            return;
            // this._audioSampleAccess = reliable;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(NetStream.prototype, "videoSampleAccess", {
        get: function () {
            release || notImplemented("public flash.net.NetStream::get videoSampleAccess");
            return;
            // return this._videoSampleAccess;
        },
        set: function (reliable) {
            reliable = !!reliable;
            release || notImplemented("public flash.net.NetStream::set videoSampleAccess");
            return;
            // this._videoSampleAccess = reliable;
        },
        enumerable: true,
        configurable: true
    });
    NetStream.prototype.appendBytes = function (bytes) {
        var chunk = new Uint8Array(bytes._buffer, 0, bytes.length);
        // We need to pass cloned data, since the bytes can be reused and
        // VideoStream.appendBytes can hold data for some time.
        this._videoStream.appendBytes(chunk);
    };
    NetStream.prototype.appendBytesAction = function (netStreamAppendBytesAction) {
        this._videoStream.appendBytesAction(netStreamAppendBytesAction);
    };
    Object.defineProperty(NetStream.prototype, "useHardwareDecoder", {
        get: function () {
            release || notImplemented("public flash.net.NetStream::get useHardwareDecoder");
            return;
            // return this._useHardwareDecoder;
        },
        set: function (v) {
            v = !!v;
            release || notImplemented("public flash.net.NetStream::set useHardwareDecoder");
            return;
            // this._useHardwareDecoder = v;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(NetStream.prototype, "useJitterBuffer", {
        get: function () {
            release || notImplemented("public flash.net.NetStream::get useJitterBuffer");
            return;
            // return this._useJitterBuffer;
        },
        set: function (value) {
            value = !!value;
            release || notImplemented("public flash.net.NetStream::set useJitterBuffer");
            return;
            // this._useJitterBuffer = value;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(NetStream.prototype, "videoStreamSettings", {
        get: function () {
            release || notImplemented("public flash.net.NetStream::get videoStreamSettings");
            return;
            // return this._videoStreamSettings;
        },
        set: function (settings) {
            settings = settings;
            release || notImplemented("public flash.net.NetStream::set videoStreamSettings");
            return;
            // this._videoStreamSettings = settings;
        },
        enumerable: true,
        configurable: true
    });
    NetStream.prototype.invoke = function (index /*uint*/) {
        index = index >>> 0;
        return this._invoke(index, Array.prototype.slice.call(arguments, 1));
    };
    NetStream.prototype.invokeWithArgsArray = function (index /*uint*/, p_arguments) {
        index = index >>> 0;
        p_arguments = p_arguments;
        return this._invoke.call(this, index, p_arguments.value);
    };
    Object.defineProperty(NetStream.prototype, "inBufferSeek", {
        get: function () {
            return this._inBufferSeek;
        },
        set: function (value) {
            this._inBufferSeek = !!value;
        },
        enumerable: true,
        configurable: true
    });
    NetStream.prototype._invoke = function (index, args) {
        var simulated = false, result;
        switch (index) {
            case 4: // set bufferTime
                this._videoStream.bufferTime = args[0];
                simulated = true;
                break;
            case 202: // call, e.g. ('pause', null, paused, time)
                switch (args[1]) {
                    case 'pause':
                        simulated = true;
                        this._notifyVideoControl(2 /* Pause */, {
                            paused: !!args[3],
                            time: args[4] / 1000
                        });
                        break;
                    case 'seek':
                        simulated = true;
                        this._notifyVideoControl(3 /* Seek */, {
                            time: args[3] / 1000
                        });
                        break;
                }
                break;
            case 300: // time
                result = this._notifyVideoControl(4 /* GetTime */, null);
                simulated = true;
                break;
            case 302: // get bufferTime
                result = this._videoStream.bufferTime;
                simulated = true;
                break;
            case 303: // get bufferLength
                result = this._notifyVideoControl(5 /* GetBufferLength */, null);
                simulated = true;
                break;
            case 305: // get bytesLoaded
                result = this._notifyVideoControl(7 /* GetBytesLoaded */, null);
                simulated = true;
                break;
            case 306: // get bytesTotal
                result = this._notifyVideoControl(8 /* GetBytesTotal */, null);
                simulated = true;
                break;
        }
        // (index:uint) -> any
        (simulated ? somewhatImplemented : notImplemented)("NetStream._invoke (" + index + ")");
        return result;
    };
    NetStream.prototype._notifyVideoControl = function (eventType, data) {
        var service = this.sec.player;
        return service.notifyVideoControl(this._id, eventType, data);
    };
    NetStream.prototype.processVideoEvent = function (eventType, data) {
        this._videoStream.processVideoPlaybackEvent(eventType, data);
        switch (eventType) {
            case 0 /* Initialized */:
                SoundMixer._updateSoundSource(this);
                break;
            case 2 /* PlayStart */:
                this.dispatchEvent(new NetStatusEvent(NetStatusEvent.NET_STATUS, false, false, this.sec.createObjectFromJS({ code: "NetStream.Play.Start", level: "status" })));
                break;
            case 3 /* PlayStop */:
                this.dispatchEvent(new NetStatusEvent(NetStatusEvent.NET_STATUS, false, false, this.sec.createObjectFromJS({ code: "NetStream.Buffer.Flush", level: "status" })));
                this.dispatchEvent(new NetStatusEvent(NetStatusEvent.NET_STATUS, false, false, this.sec.createObjectFromJS({ code: "NetStream.Play.Stop", level: "status" })));
                SoundMixer._unregisterSoundSource(this);
                break;
            case 5 /* BufferFull */:
                this.dispatchEvent(new NetStatusEvent(NetStatusEvent.NET_STATUS, false, false, this.sec.createObjectFromJS({ code: "NetStream.Buffer.Full", level: "status" })));
                break;
            case 4 /* BufferEmpty */:
                this.dispatchEvent(new NetStatusEvent(NetStatusEvent.NET_STATUS, false, false, this.sec.createObjectFromJS({ code: "NetStream.Buffer.Empty", level: "status" })));
                break;
            case 11 /* Error */:
                var code = data.code === 4 ? "NetStream.Play.NoSupportedTrackFound" :
                    data.code === 3 ? "NetStream.Play.FileStructureInvalid" : "NetStream.Play.StreamNotFound";
                this.dispatchEvent(new NetStatusEvent(NetStatusEvent.NET_STATUS, false, false, this.sec.createObjectFromJS({ code: code, level: "error" })));
                break;
            case 6 /* Pause */:
                this.dispatchEvent(new NetStatusEvent(NetStatusEvent.NET_STATUS, false, false, this.sec.createObjectFromJS({ code: "NetStream.Pause.Notify", level: "status" })));
                break;
            case 7 /* Unpause */:
                this.dispatchEvent(new NetStatusEvent(NetStatusEvent.NET_STATUS, false, false, this.sec.createObjectFromJS({ code: "NetStream.Unpause.Notify", level: "status" })));
                break;
            case 8 /* Seeking */:
                this.dispatchEvent(new NetStatusEvent(NetStatusEvent.NET_STATUS, false, false, this.sec.createObjectFromJS({ code: "NetStream.Seek.Notify", level: "status" })));
                break;
            case 9 /* Seeked */:
                this.dispatchEvent(new NetStatusEvent(NetStatusEvent.NET_STATUS, false, false, this.sec.createObjectFromJS({ code: "NetStream.Seek.Complete", level: "status" })));
                break;
            case 1 /* Metadata */:
                if (this._client) {
                    var metadata = this.sec.createObject();
                    metadata.axSetPublicProperty('width', data.videoWidth);
                    metadata.axSetPublicProperty('height', data.videoHeight);
                    metadata.axSetPublicProperty('duration', data.duration);
                    this._client.axCallPublicProperty('onMetaData', [metadata]);
                }
                break;
        }
    };
    NetStream.prototype.stopSound = function () {
        this.pause();
    };
    NetStream.prototype.updateSoundLevels = function (volume) {
        this._notifyVideoControl(6 /* SetSoundLevels */, {
            volume: volume
        });
    };
    // Called whenever the class is initialized.
    NetStream.classInitializer = null;
    // List of static symbols to link.
    NetStream.classSymbols = null; // [];
    // List of instance symbols to link.
    NetStream.instanceSymbols = null; // ["attach", "close", "attachAudio", "attachCamera", "send", "bufferTime", "bufferTime", "maxPauseBufferTime", "maxPauseBufferTime", "backBufferTime", "backBufferTime", "backBufferLength", "step", "bufferTimeMax", "bufferTimeMax", "receiveAudio", "receiveVideo", "receiveVideoFPS", "pause", "resume", "togglePause", "seek", "publish", "time", "currentFPS", "bufferLength", "liveDelay", "bytesLoaded", "bytesTotal", "decodedFrames", "videoCodec", "audioCodec", "onPeerConnect", "call"];
    // JS -> AS Bindings
    NetStream.DIRECT_CONNECTIONS = "directConnections";
    NetStream.CONNECT_TO_FMS = "connectToFMS";
    return NetStream;
}(EventDispatcher));
export { NetStream };
var FLV_MIME_TYPE = 'video/x-flv';
var MP4_MIME_TYPE = 'video/mp4';
var MP3_MIME_TYPE = 'audio/mpeg';
function buildMimeType(baseType, codecs) {
    var mimeType = baseType;
    if (codecs) {
        mimeType += ';codecs=\"' + codecs.join(',') + '\"';
    }
    return mimeType;
}
var VideoStreamState;
(function (VideoStreamState) {
    VideoStreamState[VideoStreamState["CLOSED"] = 0] = "CLOSED";
    VideoStreamState[VideoStreamState["OPENED"] = 1] = "OPENED";
    VideoStreamState[VideoStreamState["ENDED"] = 2] = "ENDED";
    VideoStreamState[VideoStreamState["OPENED_DATA_GENERATION"] = 3] = "OPENED_DATA_GENERATION";
    VideoStreamState[VideoStreamState["ERROR"] = 4] = "ERROR";
})(VideoStreamState || (VideoStreamState = {}));
/**
 * Helper class that encapsulates VIDEO/MediaSource operations and
 * buffers data before passing to the MSE.
 */
var VideoStream = /** @class */ (function () {
    function VideoStream(netStream) {
        this.sec = netStream.sec;
        this._domReady = new PromiseWrapper();
        this._metadataReady = new PromiseWrapper();
        this._started = false;
        this._buffer = 'empty';
        this._bufferTime = 0.1;
        this._url = null;
        this._mediaSource = null;
        this._mediaSourceBuffer = null;
        this._mediaSourceBufferLock = null;
        this._contentTypeHint = null;
        this._state = VideoStreamState.CLOSED;
        this._head = null;
        this._netStream = netStream;
    }
    Object.defineProperty(VideoStream.prototype, "state", {
        get: function () {
            return this._state;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(VideoStream.prototype, "bufferTime", {
        get: function () {
            return this._bufferTime;
        },
        set: function (value) {
            this._bufferTime = +value;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(VideoStream.prototype, "url", {
        get: function () {
            return this._url;
        },
        enumerable: true,
        configurable: true
    });
    VideoStream.prototype.play = function (url, checkPolicyFile) {
        console.log("todo: NetStream.play");
        /*
        release || assert(this._state === VideoStreamState.CLOSED);
    
        var isMediaSourceEnabled = mediaSourceOption.value;
        if (isMediaSourceEnabled && typeof MediaSource === 'undefined') {
          warning('MediaSource API is not enabled, falling back to regular playback');
          isMediaSourceEnabled = false;
        }
        var forceMediaSource = false;
        if (/\.flv($|\?)/i.test(url)) {
          if (flvOption.value === 'supported') {
            forceMediaSource = true;
          } else if (flvOption.value === 'mock') {
            url = 'resource://shumway/web/noflv.mp4';
          } else {
            setTimeout(() => {
              this._netStream.dispatchEvent(new NetStatusEvent(NetStatusEvent.NET_STATUS,
                false, false, this.sec.createObjectFromJS({code: "NetStream.Play.NoSupportedTrackFound", level: "error"})));
            });
            return;
          }
        }
        if (!forceMediaSource && !isMediaSourceEnabled) {
          release || somewhatImplemented("public flash.net.NetStream::play");
          this._state = VideoStreamState.OPENED;
          this._url = FileLoadingService.resolveUrl(url);
          return;
        }
    
        this.openInDataGenerationMode();
    
        var request = new URLRequest(url);
        request._checkPolicyFile = checkPolicyFile;
        var stream = new URLStream();
        stream.addEventListener('httpStatus', function (e) {
          var responseHeaders = e.axGetPublicProperty('responseHeaders');
          var contentTypeHeader = responseHeaders.filter(function (h) {
            return h.axGetPublicProperty('name') === 'Content-Type';
          })[0];
          if (contentTypeHeader) {
            var hint: string = contentTypeHeader.axGetPublicProperty('value');
            if (hint !== 'application/octet-stream') {
              // this._contentTypeHint = hint;
            }
          }
        }.bind(this));
        stream.addEventListener('progress', function (e) {
          var available = stream.bytesAvailable;
          var bytes = new ByteArray();
          stream.readBytes(bytes, 0, available);
          var chunk = new Uint8Array((<any> bytes)._buffer, 0, bytes.length);
          this.appendBytes(chunk);
        }.bind(this));
        stream.addEventListener('complete', function (e) {
          this.appendBytesAction('endSequence'); // NetStreamAppendBytesAction.END_SEQUENCE
        }.bind(this));
        stream.load(request);
        */
    };
    VideoStream.prototype.playInConnection = function (connection, streamPath) {
        console.log("todo: NetStream.playInConnection");
        /*
        this.openInDataGenerationMode();
    
        var self = this;
        var mux: RtmpJs.MP4.MP4Mux;
        var mp4 = {
          packets: 0,
          init: function (metadata) {
            if (!metadata.axGetPublicProperty('audiocodecid') && !metadata.axGetPublicProperty('videocodecid')) {
              return; // useless metadata?
            }
            var parsedMetadata = RtmpJs.MP4.parseFLVMetadata(metadata);
            mux = new RtmpJs.MP4.MP4Mux(parsedMetadata);
            mux.oncodecinfo = function (mediaCodecs) {
              this._contentTypeHint = buildMimeType(MP4_MIME_TYPE, mediaCodecs);
            };
            mux.ondata = function (data) {
              self.appendBytes(new Uint8Array(data));
            }.bind(this);
          },
          packet: function (type, data, timestamp) {
            mux.pushPacket(type, new Uint8Array(data), timestamp);
          },
          generate: function () {
            mux.flush();
          }
    
        };
    
        connection._createRtmpStream((ns: RtmpJs.INetStream, streamId: number) => {
          ns.ondata = function (message) {
            console.log('#packet (' + message.typeId + '): @' + message.timestamp);
            if (message.data.length > 0) {
              mp4.packet(message.typeId, message.data, message.timestamp);
            }
          };
          ns.oncallback = function () {
            console.log('#callback');
          };
          ns.onscriptdata = function (type, data) {
            console.log('#object: ' + type);
            if (type === 'onMetaData') {
              mp4.init(data);
            }
          };
          ns.play(streamPath);
        });
        */
    };
    VideoStream.prototype.openInDataGenerationMode = function () {
        release || assert(this._state === VideoStreamState.CLOSED);
        this._state = VideoStreamState.OPENED_DATA_GENERATION;
        var mediaSource = new MediaSource();
        mediaSource.addEventListener('sourceopen', function (e) {
            this._ensurePlaying();
        }.bind(this));
        mediaSource.addEventListener('sourceend', function (e) {
            this._mediaSource = null;
        }.bind(this));
        this._mediaSource = mediaSource;
        this._url = URL.createObjectURL(mediaSource);
    };
    VideoStream.prototype.appendBytes = function (bytes) {
        release || assert(this._state === VideoStreamState.OPENED_DATA_GENERATION ||
            this._state === VideoStreamState.OPENED);
        release || assert(this._mediaSource);
        if (this._decoder) {
            this._decoder.push(bytes);
            return;
        }
        // First we need to parse some content to find out mime type and codecs
        // for MediaSource. Caching some data at the beginning until we can tell
        // the type of the content.
        var cached;
        var buffer;
        if (this._head !== null) {
            cached = this._head.length;
            buffer = new Uint8Array(cached + bytes.length);
            buffer.set(bytes, cached);
        }
        else {
            cached = 0;
            buffer = bytes;
        }
        if (!this._decoder) {
            // Trying to create a data decoder.
            var contentType = this._detectContentType(buffer);
            if (contentType === FLV_MIME_TYPE) {
                // FLV data needs to be parsed and wrapped with MP4 tags.
                var flvDecoder = new FlvMp4Decoder(this.sec);
                flvDecoder.onHeader = function (contentType) {
                    this._mediaSourceBuffer = this._mediaSource.addSourceBuffer(contentType);
                    this._mediaSourceBufferLock = Promise.resolve(undefined);
                }.bind(this);
                flvDecoder.onData = this._queueData.bind(this);
                this._decoder = flvDecoder;
            }
            else if (contentType) {
                // Let's use identity decoder for reset of the types.
                this._decoder = {
                    onData: this._queueData.bind(this),
                    onError: function (e) { },
                    push: function (bytes) { this.onData(bytes); },
                    close: function () { }
                };
                this._mediaSourceBuffer = this._mediaSource.addSourceBuffer(contentType);
                this._mediaSourceBufferLock = Promise.resolve(undefined);
            }
        }
        if (this._decoder) {
            // The decoder exists, doing first data push, see also above.
            this._decoder.push(buffer);
            if (cached > 0) {
                this._head = null;
            }
        }
        else {
            // Caching header more header data.
            if (cached === 0) {
                this._head = new Uint8Array(bytes);
            }
            else {
                this._head = buffer;
            }
        }
    };
    VideoStream.prototype._queueData = function (bytes) {
        // We need to chain all appendBuffer operations using 'update' event.
        var buffer = this._mediaSourceBuffer;
        this._mediaSourceBufferLock = this._mediaSourceBufferLock.then(function () {
            buffer.appendBuffer(bytes);
            return new Promise(function (resolve) {
                buffer.addEventListener('update', function updateHandler() {
                    buffer.removeEventListener('update', updateHandler);
                    resolve();
                });
            });
        });
    };
    VideoStream.prototype.appendBytesAction = function (netStreamAppendBytesAction) {
        release || assert(this._state === VideoStreamState.OPENED_DATA_GENERATION ||
            this._state === VideoStreamState.OPENED);
        netStreamAppendBytesAction = axCoerceString(netStreamAppendBytesAction);
        // TODO Ignoring reset actions for now.
        if (netStreamAppendBytesAction === 'endSequence') {
            if (!this._decoder) { // Probably pushed not enough data.
                // REDUX: throw a proper internal error. Or something.
                throw new Error('Internal appendBytes error');
            }
            this._decoder.close();
            this._mediaSourceBufferLock.then(function () {
                if (this._mediaSource) {
                    this._mediaSource.endOfStream();
                }
                this.close();
            }.bind(this));
        }
        release || somewhatImplemented("public flash.net.NetStream::appendBytesAction");
    };
    VideoStream.prototype.close = function () {
        this._state = VideoStreamState.CLOSED;
    };
    VideoStream.prototype._ensurePlaying = function () {
        if (!this._onEnsurePlay) {
            return;
        }
        this._onEnsurePlay();
    };
    VideoStream.prototype._detectContentType = function (bytes) {
        if (bytes.length < 16) {
            return null; // Need more bytes.
        }
        if (bytes[0] === 0x46 /* F */ &&
            bytes[1] === 0x4C /* L */ &&
            bytes[2] === 0x56 /* V */ &&
            bytes[3] === 1 /* version 1 */) {
            // Likely FLV.
            return FLV_MIME_TYPE;
        }
        if (bytes[4] === 0x66 /* f */ &&
            bytes[5] === 0x74 /* t */ &&
            bytes[6] === 0x79 /* y */ &&
            bytes[7] === 0x70 /* p */) {
            if (this._contentTypeHint &&
                /^video\/mp4;\s*codecs=/.test(this._contentTypeHint)) {
                return this._contentTypeHint;
            }
            // TODO check bytes for content type
            return 'video/mp4; codecs="avc1.42E01E, mp4a.40.2"';
        }
        if ((bytes[0] === 0x49 /* I */ &&
            bytes[1] === 0x44 /* D */ &&
            bytes[2] === 0x33 /* 3 */) ||
            (bytes[0] === 0xFF &&
                (bytes[1] & 0xE0) === 0xE0 &&
                (bytes[1] & 0x1E) !== 0x08)) {
            // Maybe MP3.
            return MP3_MIME_TYPE;
        }
        // Just a wild (and wrong) guess
        return this._contentTypeHint || MP4_MIME_TYPE;
    };
    VideoStream.prototype.processVideoPlaybackEvent = function (eventType, data) {
        switch (eventType) {
            case 0 /* Initialized */:
                this._domReady.resolve(undefined);
                break;
            case 2 /* PlayStart */:
                if (this._started) {
                    break;
                }
                this._started = true;
                break;
            case 3 /* PlayStop */:
                this._started = false;
                break;
            case 5 /* BufferFull */:
                this._buffer = 'full';
                break;
            case 10 /* Progress */:
                this._buffer = 'progress';
                break;
            case 4 /* BufferEmpty */:
                this._buffer = 'empty';
                break;
            case 1 /* Metadata */:
                this._metadataReady.resolve({
                    videoWidth: data.videoWidth,
                    videoHeight: data.videoHeight
                });
                break;
        }
    };
    return VideoStream;
}());
// FLV-to-MP4 data transformation.
var FlvMp4Decoder = /** @class */ (function () {
    //private _flvParser: RtmpJs.FLV.FLVParser;
    //private _mp4Mux: RtmpJs.MP4.MP4Mux;
    function FlvMp4Decoder(sec) {
        this.sec = sec;
        console.log("TODO: FlvMp4Decoder");
        /*
        this._flvParser = new RtmpJs.FLV.FLVParser();
        this._flvParser.onHeader = this._onFlvHeader.bind(this);
        this._flvParser.onTag = this._onFlvTag.bind(this);
        this._flvParser.onClose = this._onFlvClose.bind(this);
        this._flvParser.onError = this._onFlvError.bind(this);
        this._mp4Mux = null;*/
    }
    FlvMp4Decoder.prototype._onFlvHeader = function (header /*RtmpJs.FLV.FLVHeader*/) {
        //
    };
    FlvMp4Decoder.prototype._onFlvTag = function (tag /*RtmpJs.FLV.FLVTag*/) {
        /* if (tag.type === 18) {
           var ba = new this.sec.flash.utils.ByteArray();
           ba.writeRawBytes(tag.data);
           ba.position = 0;
           var name = AMF0.read(ba);
           var value = AMF0.read(ba);
           if (name === 'onMetaData') {
             var metadata = RtmpJs.MP4.parseFLVMetadata(value);
             var mp4Mux = new RtmpJs.MP4.MP4Mux(metadata);
             mp4Mux.oncodecinfo = function (codecs: string[]) {
               this.onHeader(buildMimeType(MP4_MIME_TYPE, codecs));
             }.bind(this);
             mp4Mux.ondata = function (data) {
               this.onData.call(null, data);
             }.bind(this);
             this._mp4Mux = mp4Mux;
           }
           return;
         }
         this._mp4Mux.pushPacket(tag.type, new Uint8Array(tag.data), tag.timestamp);
         */
    };
    FlvMp4Decoder.prototype._onFlvClose = function () {
        //this._mp4Mux.flush();
    };
    FlvMp4Decoder.prototype._onFlvError = function (e) {
        if (this.onError) {
            this.onError(e);
        }
    };
    FlvMp4Decoder.prototype.push = function (bytes) {
        /*try {
          this._flvParser.push(bytes);
        } catch (e)  {
          if (this.onError) {
            this.onError(e);
          }
        }*/
    };
    FlvMp4Decoder.prototype.close = function () {
        /*try {
          this._flvParser.close();
        } catch (e)  {
          if (this.onError) {
            this.onError(e);
          }
        }*/
    };
    return FlvMp4Decoder;
}());
