import { __extends } from "tslib";
import { axCoerceString } from "../../avm2/run/axCoerceString";
import { Errors } from "../../avm2/errors";
import { ByteArray } from "../../avm2/natives/byteArray";
import { ASObject } from '../../avm2/nat/ASObject';
/**
 * Copyright 2014 Mozilla Foundation
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
// Class: URLRequest
var URLRequest = /** @class */ (function (_super) {
    __extends(URLRequest, _super);
    function URLRequest(url) {
        if (url === void 0) { url = null; }
        var _this = _super.call(this) || this;
        _this._url = axCoerceString(url);
        _this._method = 'GET';
        _this._data = null;
        _this._digest = null;
        _this._contentType = 'application/x-www-form-urlencoded';
        _this._requestHeaders = [];
        _this._checkPolicyFile = true;
        return _this;
    }
    Object.defineProperty(URLRequest.prototype, "url", {
        get: function () {
            return this._url;
        },
        set: function (value) {
            value = axCoerceString(value);
            this._url = value;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(URLRequest.prototype, "data", {
        get: function () {
            return this._data;
        },
        set: function (value) {
            this._data = value;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(URLRequest.prototype, "method", {
        get: function () {
            return this._method;
        },
        set: function (value) {
            value = axCoerceString(value);
            if (value !== 'get' && value !== 'GET' &&
                value !== 'post' && value !== 'POST') {
                this.sec.throwError('ArgumentError', Errors.InvalidArgumentError);
            }
            this._method = value;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(URLRequest.prototype, "contentType", {
        get: function () {
            return this._contentType;
        },
        set: function (value) {
            value = axCoerceString(value);
            this._contentType = value;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(URLRequest.prototype, "requestHeaders", {
        get: function () {
            return this.sec.createArrayUnsafe(this._requestHeaders);
        },
        set: function (value) {
            if (!this.sec.AXArray.axIsType(value)) {
                this.sec.throwError('ArgumentError', Errors.InvalidArgumentError, "value");
            }
            this._requestHeaders = value.value;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(URLRequest.prototype, "digest", {
        get: function () {
            return this._digest;
        },
        set: function (value) {
            value = axCoerceString(value);
            this._digest = value;
        },
        enumerable: true,
        configurable: true
    });
    URLRequest.prototype._toFileRequest = function () {
        var obj = {};
        obj.url = this._url;
        obj.method = this._method;
        obj.checkPolicyFile = this._checkPolicyFile;
        var data = this._data;
        if (data) {
            obj.mimeType = this._contentType;
            if (ByteArray.axClass.axIsType(data)) {
                obj.data = new Uint8Array(data._buffer, 0, data.length);
            }
            else {
                var dataStr = data.toString();
                if (this._method === 'GET') {
                    var i = obj.url.lastIndexOf('?');
                    obj.url = (i < 0 ? obj.url : obj.url.substring(0, i)) + '?' + dataStr;
                }
                else {
                    obj.data = dataStr;
                }
            }
        }
        return obj;
    };
    // Called whenever the class is initialized.
    URLRequest.classInitializer = null;
    // List of static symbols to link.
    URLRequest.classSymbols = null; // [];
    // List of instance symbols to link.
    URLRequest.bindings = null;
    return URLRequest;
}(ASObject));
export { URLRequest };
