import { __extends } from "tslib";
import { release, notImplemented } from "../../base/utilities/Debug";
import { Errors } from "../../avm2/errors";
import { ByteArray } from "../../avm2/natives/byteArray";
import { EventDispatcher } from "../events/EventDispatcher";
import { FileLoadingService } from "../../base/utilities/FileLoadingService";
import { axCoerceString } from "../../avm2/run/axCoerceString";
import { ProgressEvent } from "../events/ProgressEvent";
import { IOErrorEvent } from "../events/IOErrorEvent";
import { Event } from "../events/Event";
import { HTTPStatusEvent } from "../events/HTTPStatusEvent";
import { URLRequestHeader } from "./URLRequestHeader";
/**
 * Copyright 2014 Mozilla Foundation
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
var URLStream = /** @class */ (function (_super) {
    __extends(URLStream, _super);
    function URLStream() {
        var _this = _super.call(this) || this;
        _this._buffer = new ByteArray();
        _this._writePosition = 0;
        _this._connected = false;
        return _this;
    }
    Object.defineProperty(URLStream.prototype, "connected", {
        // _diskCacheEnabled: boolean;
        get: function () {
            return this._connected;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(URLStream.prototype, "bytesAvailable", {
        get: function () {
            return this._buffer.length - this._buffer.position;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(URLStream.prototype, "objectEncoding", {
        get: function () {
            return this._buffer.objectEncoding;
        },
        set: function (version /*uint*/) {
            version = version >>> 0;
            this._buffer.objectEncoding = version;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(URLStream.prototype, "endian", {
        get: function () {
            return this._buffer.endian;
        },
        set: function (type) {
            type = axCoerceString(type);
            this._buffer.endian = type;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(URLStream.prototype, "diskCacheEnabled", {
        get: function () {
            release || notImplemented("public flash.net.URLStream::get diskCacheEnabled");
            return;
            // return this._diskCacheEnabled;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(URLStream.prototype, "position", {
        get: function () {
            return this._buffer.position;
        },
        set: function (offset) {
            offset = +offset;
            this._buffer.position = offset;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(URLStream.prototype, "length", {
        get: function () {
            return this._buffer.length;
        },
        enumerable: true,
        configurable: true
    });
    URLStream.prototype.load = function (request) {
        var session = FileLoadingService.createSession();
        var self = this;
        var initStream = true;
        session.onprogress = function (data, progressState) {
            var readPosition = self._buffer.position;
            self._buffer.position = self._writePosition;
            self._buffer.writeRawBytes(data);
            self._writePosition = self._buffer.position;
            self._buffer.position = readPosition;
            self.dispatchEvent(new ProgressEvent(ProgressEvent.PROGRESS, false, false, progressState.bytesLoaded, progressState.bytesTotal));
        };
        session.onerror = function (error) {
            self._connected = false;
            self.dispatchEvent(new IOErrorEvent(IOErrorEvent.IO_ERROR, false, false, error));
            var isXDomainError = typeof error === 'string' && error.indexOf('XDOMAIN') >= 0;
            /*Telemetry.instance.reportTelemetry({topic: 'loadResource',
              resultType: isXDomainError ? Telemetry.LoadResource.StreamCrossdomain :
                                            Telemetry.LoadResource.StreamDenied});*/
        };
        session.onopen = function () {
            self._connected = true;
            self.dispatchEvent(new Event(Event.OPEN, false, false));
            /*Telemetry.instance.reportTelemetry({topic: 'loadResource',
              resultType: Telemetry.LoadResource.StreamAllowed});*/
        };
        session.onhttpstatus = function (location, httpStatus, httpHeaders) {
            var httpStatusEvent = new HTTPStatusEvent(HTTPStatusEvent.HTTP_STATUS, false, false, httpStatus);
            var headers = [];
            httpHeaders.split(/(?:\n|\r?\n)/g).forEach(function (h) {
                var m = /^([^:]+): (.*)$/.exec(h);
                if (m) {
                    headers.push(new URLRequestHeader(m[1], m[2]));
                    if (m[1] === 'Location') { // Headers have redirect location
                        location = m[2];
                    }
                }
            });
            var boxedHeaders = self.sec.createArray(headers);
            httpStatusEvent.axSetPublicProperty('responseHeaders', boxedHeaders);
            httpStatusEvent.axSetPublicProperty('responseURL', location);
            self.dispatchEvent(httpStatusEvent);
        };
        session.onclose = function () {
            self._connected = false;
            self.dispatchEvent(new Event(Event.COMPLETE, false, false));
        };
        session.open(request._toFileRequest());
        this._session = session;
    };
    URLStream.prototype.readBytes = function (bytes, offset, length) {
        if (offset === void 0) { offset = 0; }
        if (length === void 0) { length = 0; }
        offset = offset >>> 0;
        length = length >>> 0;
        if (length < 0) {
            this.sec.throwError('ArgumentError', Errors.InvalidArgumentError, "length");
        }
        this._buffer.readBytes(bytes, offset, length);
    };
    URLStream.prototype.readBoolean = function () {
        release || notImplemented("public flash.net.URLStream::readBoolean");
        return;
    };
    URLStream.prototype.readByte = function () {
        return this._buffer.readByte();
    };
    URLStream.prototype.readUnsignedByte = function () {
        release || notImplemented("public flash.net.URLStream::readUnsignedByte");
        return;
    };
    URLStream.prototype.readShort = function () {
        release || notImplemented("public flash.net.URLStream::readShort");
        return;
    };
    URLStream.prototype.readUnsignedShort = function () {
        return this._buffer.readUnsignedShort();
    };
    URLStream.prototype.readUnsignedInt = function () {
        release || notImplemented("public flash.net.URLStream::readUnsignedInt");
        return;
    };
    URLStream.prototype.readInt = function () {
        release || notImplemented("public flash.net.URLStream::readInt");
        return;
    };
    URLStream.prototype.readFloat = function () {
        release || notImplemented("public flash.net.URLStream::readFloat");
        return;
    };
    URLStream.prototype.readDouble = function () {
        release || notImplemented("public flash.net.URLStream::readDouble");
        return;
    };
    URLStream.prototype.readMultiByte = function (length /*uint*/, charSet) {
        length = length >>> 0;
        charSet = axCoerceString(charSet);
        release || notImplemented("public flash.net.URLStream::readMultiByte");
        return;
    };
    URLStream.prototype.readUTF = function () {
        return this._buffer.readUTF();
    };
    URLStream.prototype.readUTFBytes = function (length /*uint*/) {
        return this._buffer.readUTFBytes(length);
    };
    URLStream.prototype.close = function () {
        if (this._session) {
            this._session.close();
        }
    };
    URLStream.prototype.readObject = function () {
        release || notImplemented("public flash.net.URLStream::readObject");
        return;
    };
    URLStream.prototype.stop = function () {
        release || notImplemented("public flash.net.URLStream::stop");
        return;
    };
    // Called whenever the class is initialized.
    URLStream.classInitializer = null;
    // List of static symbols to link.
    URLStream.classSymbols = null; // [];
    // List of instance symbols to link.
    URLStream.instanceSymbols = null; // [];
    return URLStream;
}(EventDispatcher));
export { URLStream };
