import { __extends } from "tslib";
import { ASObject } from "../../avm2/nat/ASObject";
import { axCoerceString } from "../../avm2/run/axCoerceString";
import { getCurrentABC } from "../../avm2/run/getCurrentABC";
import { AXApplicationDomain } from "../../avm2/run/AXApplicationDomain";
import { assert, release, notImplemented } from "../../base/utilities/Debug";
import { Errors } from "../../avm2/errors";
import { Multiname } from "../../avm2/abc/lazy/Multiname";
/**
 * Copyright 2014 Mozilla Foundation
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
// Class: ApplicationDomain
var ApplicationDomain = /** @class */ (function (_super) {
    __extends(ApplicationDomain, _super);
    function ApplicationDomain(parentDomainOrAXDomain) {
        if (parentDomainOrAXDomain === void 0) { parentDomainOrAXDomain = null; }
        var _this = _super.call(this) || this;
        release || assert(!(_this instanceof ApplicationDomain));
        if (parentDomainOrAXDomain instanceof AXApplicationDomain) {
            _this.axDomain = parentDomainOrAXDomain;
            return _this;
        }
        var parentRuntimeDomain = null;
        if (ApplicationDomain.axIsType(parentDomainOrAXDomain)) {
            parentRuntimeDomain = parentDomainOrAXDomain.axDomain;
        }
        else {
            parentRuntimeDomain = _this.sec.application;
        }
        _this.axDomain = new AXApplicationDomain(_this.sec, parentRuntimeDomain);
        return _this;
    }
    Object.defineProperty(ApplicationDomain, "currentDomain", {
        // This must return a new object each time.
        get: function () {
            var currentABC = getCurrentABC();
            var app = currentABC ? currentABC.env.app : this.sec.application;
            return new ApplicationDomain(app);
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(ApplicationDomain, "MIN_DOMAIN_MEMORY_LENGTH", {
        get: function () {
            release || notImplemented("public flash.system.ApplicationDomain::get MIN_DOMAIN_MEMORY_LENGTH");
            return;
            // return this._MIN_DOMAIN_MEMORY_LENGTH;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(ApplicationDomain.prototype, "parentDomain", {
        get: function () {
            var currentABC = getCurrentABC();
            var app = currentABC ? currentABC.env.app : this.sec.application;
            release || assert(app.parent !== undefined);
            return new ApplicationDomain(app.parent);
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(ApplicationDomain.prototype, "domainMemory", {
        get: function () {
            release || notImplemented("public flash.system.ApplicationDomain::get domainMemory");
            return;
            // return this._domainMemory;
        },
        set: function (mem) {
            mem = mem;
            release || notImplemented("public flash.system.ApplicationDomain::set domainMemory");
            return;
            // this._domainMemory = mem;
        },
        enumerable: true,
        configurable: true
    });
    ApplicationDomain.prototype.getDefinition = function (name) {
        var definition = this.getDefinitionImpl(name);
        if (!definition) {
            this.sec.throwError('ReferenceError', Errors.UndefinedVarError, name);
        }
        return definition;
    };
    ApplicationDomain.prototype.hasDefinition = function (name) {
        return !!this.getDefinitionImpl(name);
    };
    ApplicationDomain.prototype.getDefinitionImpl = function (name) {
        name = axCoerceString(name);
        if (!name) {
            this.sec.throwError('TypeError', Errors.NullPointerError, 'definitionName');
        }
        var simpleName = name.replace("::", ".");
        var mn = Multiname.FromFQNString(simpleName, 0 /* Public */);
        return this.axDomain.getProperty(mn, false, false);
    };
    ApplicationDomain.prototype.getQualifiedDefinitionNames = function () {
        release || notImplemented("public flash.system.ApplicationDomain::getQualifiedDefinitionNames");
        return;
    };
    return ApplicationDomain;
}(ASObject));
export { ApplicationDomain };
