import { __extends } from "tslib";
import { assert, release } from "../../base/utilities/Debug";
import { transformASValueToJS } from "../../avm2/nat/transformASValueToJS";
import { transformJSValueToAS } from "../../avm2/nat/transformJSValueToAS";
import { axCoerceString } from "../../avm2/run/axCoerceString";
import { EventDispatcher } from "../events/EventDispatcher";
import { TextFormat } from "./TextFormat";
var StyleSheet = /** @class */ (function (_super) {
    __extends(StyleSheet, _super);
    function StyleSheet() {
        var _this = _super.call(this) || this;
        _this.clear();
        return _this;
    }
    Object.defineProperty(StyleSheet.prototype, "styleNames", {
        get: function () {
            var styles = this._rules;
            var names = [];
            for (var name in styles) {
                if (styles[name]) {
                    names.push(name);
                }
            }
            return this.sec.createArrayUnsafe(names);
        },
        enumerable: true,
        configurable: true
    });
    StyleSheet.prototype.getStyle = function (styleName) {
        styleName = axCoerceString(styleName);
        var style = this._rules[styleName.toLowerCase()];
        if (!style) {
            return this.sec.createObject(); // note that documentation is lying about `null`;
        }
        return transformJSValueToAS(this.sec, style, false);
    };
    StyleSheet.prototype.applyStyle = function (textFormat, styleName) {
        styleName = axCoerceString(styleName);
        var style = this._rules[styleName.toLowerCase()];
        if (style) {
            return textFormat.transform(style);
        }
        return textFormat;
    };
    StyleSheet.prototype.setStyle = function (styleName, styleObject) {
        if (typeof styleObject !== 'object') {
            return;
        }
        styleName = axCoerceString(styleName);
        this._rules[styleName.toLowerCase()] = transformASValueToJS(this.sec, styleObject, false);
    };
    StyleSheet.prototype.hasStyle = function (styleName) {
        return !!this._rules[styleName.toLowerCase()];
    };
    StyleSheet.prototype.clear = function () {
        this._rules = Object.create(null);
    };
    StyleSheet.prototype.transform = function (formatObject) {
        if (typeof formatObject !== 'object') {
            return null;
        }
        formatObject = transformASValueToJS(this.sec, formatObject, false);
        var textFormat = new TextFormat();
        textFormat.transform(formatObject);
        return textFormat;
    };
    StyleSheet.prototype.parseCSS = function (css) {
        css = axCoerceString(css) + '';
        var length = css.length;
        var index = skipWhitespace(css, 0, length);
        // Styles are only added once parsing completed successfully. Invalid syntax anywhere discards all new styles.
        var newStyles = {};
        var currentNames = [];
        var sawWhitespace = false;
        var name = '';
        while (index < length) {
            var char = css[index++];
            // Everything except whitespace, command, and '{' is valid in names.
            // Note: if no name is given, the empty string is used.
            switch (char) {
                case '{':
                    sawWhitespace = false;
                    currentNames.push(name);
                    // parse style.
                    index = parseStyle(css, index, length, currentNames, newStyles);
                    if (index === -1) {
                        // Syntax error encountered in style parsing.
                        return;
                    }
                    else if (!release) {
                        assert(css[index - 1] === '}');
                    }
                    currentNames = [];
                    name = '';
                    index = skipWhitespace(css, index, length);
                    break;
                case ',':
                    sawWhitespace = false;
                    currentNames.push(name);
                    name = '';
                    index = skipWhitespace(css, index, length);
                    break;
                case ' ':
                case '\n':
                case '\r':
                case '\t':
                    sawWhitespace = true;
                    index = skipWhitespace(css, index, length);
                    break;
                default:
                    if (sawWhitespace) {
                        return;
                    }
                    name += char;
            }
        }
        var styles = this._rules;
        for (name in newStyles) {
            styles[name.toLowerCase()] = newStyles[name];
        }
    };
    StyleSheet.classInitializer = null;
    StyleSheet.classSymbols = null;
    StyleSheet.instanceSymbols = null;
    return StyleSheet;
}(EventDispatcher));
export { StyleSheet };
function parseStyle(css, index, length, names, newStyles) {
    release || assert(index > 0);
    release || assert(css[index - 1] === '{');
    var style = {};
    var name = '';
    var sawWhitespace = false;
    var upperCase = false;
    index = skipWhitespace(css, index, length);
    // Outer loop parsing property names.
    nameLoop: while (index < length) {
        var char = css[index++];
        switch (char) {
            case '}':
                if (name.length > 0) {
                    return -1;
                }
                break nameLoop;
            case ':':
                var value = '';
                var propertyName = name;
                // Reset outer-loop state.
                name = '';
                sawWhitespace = false;
                upperCase = false;
                // Inner loop parsing property values.
                //80pro todo: this was like this: 
                //valueLoop: while (index < length) {
                while (index < length) {
                    char = css[index];
                    switch (char) {
                        case ';':
                        case '\r':
                        case '\n':
                            index++;
                            index = skipWhitespace(css, index, length);
                        // Fallthrough.
                        case '}':
                            style[propertyName] = value;
                            continue nameLoop;
                        default:
                            index++;
                            value += char;
                    }
                }
                // If we got here, the inner loop ended by exhausting the string, so the definition
                // wasn't properly closed.
                return -1;
            case '-':
                if (css[index] === ':') {
                    name += char;
                }
                else {
                    upperCase = true;
                }
                break;
            case ' ':
            case '\n':
            case '\r':
            case '\t':
                sawWhitespace = true;
                name += char;
                upperCase = false;
                break;
            default:
                // Names that're interrupted by whitespace are invalid.
                if (sawWhitespace) {
                    return -1;
                }
                if (upperCase) {
                    char = char.toUpperCase();
                    upperCase = false;
                }
                name += char;
        }
    }
    if (css[index - 1] !== '}') {
        return -1;
    }
    for (var i = 0; i < names.length; i++) {
        newStyles[names[i]] = style;
    }
    return index;
}
function skipWhitespace(css, index, length) {
    while (index < length) {
        var char = css[index];
        switch (char) {
            case ' ':
            case '\n':
            case '\r':
            case '\t':
                index++;
                break;
            default:
                return index;
        }
    }
    release || assert(index === length);
    return length;
}
