import { __extends } from "tslib";
import { InteractiveObject } from "../display/InteractiveObject";
import { assert, release, notImplemented, somewhatImplemented, warning } from "../../base/utilities/Debug";
import { AntiAliasType } from "./AntiAliasType";
import { TextFieldAutoSize } from "./TextFieldAutoSize";
import { GridFitType } from "./GridFitType";
import { TextInteractionMode } from "./TextInteractionMode";
import { TextFieldType } from "./TextFieldType";
import { TextFormat } from "./TextFormat";
import { Font } from "./Font";
import { TextFormatAlign } from "./TextFormatAlign";
import { DisplaySymbol } from "../symbol";
import { isNullOrUndefined } from "../../base/utilities";
import { axCoerceString } from "../../avm2/run/axCoerceString";
import { Errors } from "../../avm2/errors";
import { clamp } from "../../base/utilities/NumberUtilities";
import { Rectangle } from "../geom/Rectangle";
import { TextLineMetrics } from "./TextLineMetrics";
import { StaticText } from "./StaticText";
/**
 * Copyright 2014 Mozilla Foundation
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
// Class: TextField
var TextField = /** @class */ (function (_super) {
    __extends(TextField, _super);
    function TextField() {
        var _this = _super.call(this) || this;
        if (!_this._fieldsInitialized) {
            _this._initializeFields();
        }
        if (!_this._symbol) {
            _this._setFillAndLineBoundsFromWidthAndHeight(100 * 20, 100 * 20);
        }
        return _this;
    }
    TextField.prototype.applySymbol = function () {
        this._initializeFields();
        release || assert(this._symbol);
        var symbol = this._symbol;
        this._setFillAndLineBoundsFromSymbol(symbol);
        var defaultTextFormat = this._textContent.defaultTextFormat;
        defaultTextFormat.color = symbol.color;
        defaultTextFormat.size = (symbol.size / 20) | 0;
        defaultTextFormat.font = symbol.face;
        defaultTextFormat.bold = symbol.bold;
        defaultTextFormat.italic = symbol.italic;
        defaultTextFormat.align = symbol.align;
        defaultTextFormat.leftMargin = (symbol.leftMargin / 20) | 0;
        defaultTextFormat.rightMargin = (symbol.rightMargin / 20) | 0;
        defaultTextFormat.indent = (symbol.indent / 20) | 0;
        defaultTextFormat.leading = (symbol.leading / 20) | 0;
        this._multiline = symbol.multiline;
        this._embedFonts = symbol.embedFonts;
        this._selectable = symbol.selectable;
        this._displayAsPassword = symbol.displayAsPassword;
        this._type = symbol.type;
        this._maxChars = symbol.maxChars;
        if (symbol.border) {
            this.background = true;
            this.border = true;
        }
        if (symbol.html) {
            this.htmlText = symbol.initialText;
        }
        else {
            this.text = symbol.initialText;
        }
        this.wordWrap = symbol.wordWrap;
        this.autoSize = symbol.autoSize;
    };
    TextField.prototype._initializeFields = function () {
        _super.prototype._initializeFields.call(this);
        this._alwaysShowSelection = false;
        this._antiAliasType = AntiAliasType.NORMAL;
        this._autoSize = TextFieldAutoSize.NONE;
        this._background = false;
        this._backgroundColor = 0xffffffff;
        this._border = false;
        this._borderColor = 0x000000ff;
        this._bottomScrollV = 1;
        this._caretIndex = 0;
        this._condenseWhite = false;
        this._embedFonts = false;
        this._gridFitType = GridFitType.PIXEL;
        this._htmlText = '';
        this._length = 0;
        this._textInteractionMode = TextInteractionMode.NORMAL;
        this._maxChars = 0;
        this._maxScrollH = 0;
        this._maxScrollV = 1;
        this._mouseWheelEnabled = false;
        this._multiline = false;
        this._numLines = 1;
        this._displayAsPassword = false;
        this._restrict = null;
        this._selectable = true;
        this._selectedText = '';
        this._selectionBeginIndex = 0;
        this._selectionEndIndex = 0;
        this._sharpness = 0;
        this._styleSheet = null;
        this._textColor = null;
        this._textHeight = 0;
        this._textWidth = 0;
        this._thickness = 0;
        this._type = TextFieldType.DYNAMIC;
        this._useRichTextClipboard = false;
        this._lineMetricsData = null;
        var defaultTextFormat = new TextFormat(Font.axClass.DEFAULT_FONT_SERIF, 12, 0, false, false, false, '', '', TextFormatAlign.LEFT);
        defaultTextFormat.letterSpacing = 0;
        defaultTextFormat.kerning = 0;
        console.log("80pro todo: TextContent");
        //this._textContent = new TextContent(this.sec, defaultTextFormat);
    };
    TextField.prototype._setFillAndLineBoundsFromSymbol = function (symbol) {
        _super.prototype._setFillAndLineBoundsFromSymbol.call(this, symbol);
        this._textContent.bounds = this._lineBounds;
        this._invalidateContent();
    };
    TextField.prototype._setFillAndLineBoundsFromWidthAndHeight = function (width, height) {
        _super.prototype._setFillAndLineBoundsFromWidthAndHeight.call(this, width, height);
        this._textContent.bounds = this._lineBounds;
        this._invalidateContent();
    };
    TextField.prototype._canHaveTextContent = function () {
        return true;
    };
    TextField.prototype._getTextContent = function () {
        return this._textContent;
    };
    TextField.prototype._getContentBounds = function (includeStrokes) {
        if (includeStrokes === void 0) { includeStrokes = true; }
        this._ensureLineMetrics();
        return _super.prototype._getContentBounds.call(this, includeStrokes);
    };
    TextField.prototype._containsPointDirectly = function (localX, localY, globalX, globalY) {
        // If this override is reached, the content bounds have already been checked, which is all
        // we need to do.
        release || assert(this._getContentBounds().contains(localX, localY));
        return true;
    };
    TextField.prototype._invalidateContent = function () {
        console.log("80pro todo: _invalidateContent");
        /*
        if (this._textContent.flags & TextContentFlag.Dirty) {
          this._setDirtyFlags(DisplayObjectDirtyFlags.DirtyTextContent);
        }*/
    };
    // JS -> AS Bindings
    //selectedText: string;
    //appendText: (newText: string) => void;
    //getXMLText: (beginIndex: number /*int*/ = 0, endIndex: number /*int*/ = 2147483647) =>
    // string;
    //insertXMLText: (beginIndex: number /*int*/, endIndex: number /*int*/, richText: string,
    // pasting: boolean = false) => void; copyRichText: () => string; pasteRichText: (richText:
    // string) => boolean;
    // AS -> JS Bindings
    TextField.isFontCompatible = function (fontName, fontStyle) {
        fontName = axCoerceString(fontName);
        fontStyle = axCoerceString(fontStyle);
        var font = Font.getByNameAndStyle(fontName, fontStyle);
        if (!font) {
            return false;
        }
        return font.fontStyle === fontStyle;
    };
    Object.defineProperty(TextField.prototype, "alwaysShowSelection", {
        get: function () {
            return this._alwaysShowSelection;
        },
        set: function (value) {
            this._alwaysShowSelection = !!value;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(TextField.prototype, "antiAliasType", {
        get: function () {
            return this._antiAliasType;
        },
        set: function (antiAliasType) {
            if (isNullOrUndefined(antiAliasType)) {
                this.sec.throwError('TypeError', Errors.NullPointerError, 'antiAliasType');
            }
            antiAliasType = axCoerceString(antiAliasType);
            if (AntiAliasType.toNumber(antiAliasType) < 0) {
                this.sec.throwError("ArgumentError", Errors.InvalidParamError, "antiAliasType");
            }
            this._antiAliasType = antiAliasType;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(TextField.prototype, "autoSize", {
        get: function () {
            return this._autoSize;
        },
        set: function (value) {
            if (isNullOrUndefined(value)) {
                this.sec.throwError('TypeError', Errors.NullPointerError, 'autoSize');
            }
            value = axCoerceString(value);
            if (value === this._autoSize) {
                return;
            }
            if (TextFieldAutoSize.toNumber(value) < 0) {
                this.sec.throwError("ArgumentError", Errors.InvalidParamError, "autoSize");
            }
            this._autoSize = value;
            this._textContent.autoSize = TextFieldAutoSize.toNumber(value);
            this._invalidateContent();
            this._ensureLineMetrics();
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(TextField.prototype, "background", {
        get: function () {
            return this._background;
        },
        set: function (value) {
            value = !!value;
            if (value === this._background) {
                return;
            }
            this._background = value;
            this._textContent.backgroundColor = value ? this._backgroundColor : 0;
            this._setDirtyFlags(8 /* DirtyTextContent */);
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(TextField.prototype, "backgroundColor", {
        get: function () {
            return this._backgroundColor >> 8;
        },
        set: function (value /*uint*/) {
            value = ((value << 8) | 0xff) >>> 0;
            if (value === this._backgroundColor) {
                return;
            }
            this._backgroundColor = value;
            if (this._background) {
                this._textContent.backgroundColor = value;
                this._setDirtyFlags(8 /* DirtyTextContent */);
            }
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(TextField.prototype, "border", {
        get: function () {
            return this._border;
        },
        set: function (value) {
            value = !!value;
            if (value === this._border) {
                return;
            }
            this._border = value;
            this._textContent.borderColor = value ? this._borderColor : 0;
            this._setDirtyFlags(8 /* DirtyTextContent */);
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(TextField.prototype, "borderColor", {
        get: function () {
            return this._borderColor >> 8;
        },
        set: function (value /*uint*/) {
            value = ((value << 8) | 0xff) >>> 0;
            if (value === this._borderColor) {
                return;
            }
            this._borderColor = value;
            if (this._border) {
                this._textContent.borderColor = value;
                this._setDirtyFlags(8 /* DirtyTextContent */);
            }
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(TextField.prototype, "bottomScrollV", {
        // Returns bottommost line that is currently visible.
        get: function () {
            return this._bottomScrollV;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(TextField.prototype, "caretIndex", {
        get: function () {
            release || notImplemented("public flash.text.TextField::get caretIndex");
            return;
            // return this._caretIndex;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(TextField.prototype, "condenseWhite", {
        get: function () {
            return this._condenseWhite;
        },
        set: function (value) {
            this._condenseWhite = !!value;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(TextField.prototype, "defaultTextFormat", {
        get: function () {
            return this._textContent.defaultTextFormat.clone();
        },
        set: function (format) {
            if (isNullOrUndefined(format)) {
                this.sec.throwError('TypeError', Errors.NullPointerError, 'format');
            }
            var defaultTextFormat = this._textContent.defaultTextFormat;
            defaultTextFormat.merge(format);
            if (defaultTextFormat.color === null) {
                defaultTextFormat.color = this._textColor;
            }
            else {
                this._textColor = +defaultTextFormat.color;
            }
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(TextField.prototype, "embedFonts", {
        get: function () {
            return this._embedFonts;
        },
        set: function (value) {
            this._embedFonts = !!value;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(TextField.prototype, "gridFitType", {
        get: function () {
            return this._gridFitType;
        },
        set: function (gridFitType) {
            if (isNullOrUndefined(gridFitType)) {
                this.sec.throwError('TypeError', Errors.NullPointerError, 'gridFitType');
            }
            gridFitType = axCoerceString(gridFitType);
            release || assert(GridFitType.toNumber(gridFitType) >= 0);
            this._gridFitType = gridFitType;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(TextField.prototype, "htmlText", {
        get: function () {
            return this._htmlText;
        },
        set: function (value) {
            if (isNullOrUndefined(value)) {
                this.sec.throwError('TypeError', Errors.NullPointerError, 'value');
            }
            value = axCoerceString(value);
            // Flash resets the bold and italic flags when an html value is set on a text field created
            // from a symbol.
            if (this._symbol) {
                this._textContent.defaultTextFormat.bold = false;
                this._textContent.defaultTextFormat.italic = false;
            }
            this._textContent.parseHtml(value, this._styleSheet, this._multiline);
            this._htmlText = value;
            this._invalidateContent();
            this._ensureLineMetrics();
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(TextField.prototype, "length", {
        get: function () {
            return this._length;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(TextField.prototype, "textInteractionMode", {
        get: function () {
            release || notImplemented("public flash.text.TextField::get textInteractionMode");
            return;
            // return this._textInteractionMode;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(TextField.prototype, "maxChars", {
        get: function () {
            return this._maxChars;
        },
        set: function (value /*int*/) {
            this._maxChars = value | 0;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(TextField.prototype, "maxScrollH", {
        get: function () {
            this._ensureLineMetrics();
            return this._maxScrollH;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(TextField.prototype, "maxScrollV", {
        get: function () {
            this._ensureLineMetrics();
            return this._maxScrollV;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(TextField.prototype, "mouseWheelEnabled", {
        get: function () {
            return this._mouseWheelEnabled;
        },
        set: function (value) {
            release || somewhatImplemented("public flash.text.TextField::set mouseWheelEnabled");
            this._mouseWheelEnabled = !!value;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(TextField.prototype, "multiline", {
        get: function () {
            return this._multiline;
        },
        set: function (value) {
            this._multiline = !!value;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(TextField.prototype, "numLines", {
        get: function () {
            return this._numLines;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(TextField.prototype, "displayAsPassword", {
        get: function () {
            return this._displayAsPassword;
        },
        set: function (value) {
            release || somewhatImplemented("public flash.text.TextField::set displayAsPassword");
            this._displayAsPassword = !!value;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(TextField.prototype, "restrict", {
        get: function () {
            return this._restrict;
        },
        set: function (value) {
            release || somewhatImplemented("public flash.text.TextField::set restrict");
            this._restrict = axCoerceString(value);
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(TextField.prototype, "scrollH", {
        // Returns the current vertical scrolling position in lines.
        get: function () {
            return this._textContent.scrollH;
        },
        set: function (value /*int*/) {
            value = value | 0;
            this._ensureLineMetrics();
            this._textContent.scrollH = clamp(Math.abs(value), 0, this._maxScrollH);
            this._invalidateContent();
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(TextField.prototype, "scrollV", {
        // Returns the current horizontal scrolling position in pixels.
        get: function () {
            return this._textContent.scrollV;
        },
        set: function (value /*int*/) {
            value = value | 0;
            this._ensureLineMetrics();
            this._textContent.scrollV = clamp(value, 1, this._maxScrollV);
            this._invalidateContent();
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(TextField.prototype, "selectable", {
        get: function () {
            return this._selectable;
        },
        set: function (value) {
            this._selectable = !!value;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(TextField.prototype, "selectedText", {
        get: function () {
            return this._textContent.plainText.substring(this._selectionBeginIndex, this._selectionEndIndex);
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(TextField.prototype, "selectionBeginIndex", {
        get: function () {
            return this._selectionBeginIndex;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(TextField.prototype, "selectionEndIndex", {
        get: function () {
            return this._selectionEndIndex;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(TextField.prototype, "sharpness", {
        get: function () {
            return this._sharpness;
        },
        set: function (value) {
            this._sharpness = clamp(+value, -400, 400);
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(TextField.prototype, "styleSheet", {
        get: function () {
            return this._styleSheet;
        },
        set: function (value) {
            this._styleSheet = value;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(TextField.prototype, "text", {
        get: function () {
            return this._textContent.plainText;
        },
        set: function (value) {
            if (isNullOrUndefined(value)) {
                this.sec.throwError('TypeError', Errors.NullPointerError, 'value');
            }
            value = axCoerceString(value) || '';
            if (value === this._textContent.plainText) {
                return;
            }
            this._textContent.plainText = value;
            this._invalidateContent();
            this._ensureLineMetrics();
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(TextField.prototype, "textColor", {
        get: function () {
            return this._textColor === null ? +this._textContent.defaultTextFormat.color : this._textColor;
        },
        set: function (value /*uint*/) {
            this._textColor = this._textContent.defaultTextFormat.color = value >>> 0;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(TextField.prototype, "textHeight", {
        get: function () {
            this._ensureLineMetrics();
            return (this._textHeight / 20) | 0;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(TextField.prototype, "textWidth", {
        get: function () {
            this._ensureLineMetrics();
            return (this._textWidth / 20) | 0;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(TextField.prototype, "thickness", {
        get: function () {
            return this._thickness;
        },
        set: function (value) {
            this._thickness = clamp(+value, -200, 200);
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(TextField.prototype, "type", {
        get: function () {
            return this._type;
        },
        set: function (value) {
            this._type = axCoerceString(value);
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(TextField.prototype, "wordWrap", {
        get: function () {
            return this._textContent.wordWrap;
        },
        set: function (value) {
            value = !!value;
            if (value === this._textContent.wordWrap) {
                return;
            }
            this._textContent.wordWrap = !!value;
            this._invalidateContent();
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(TextField.prototype, "useRichTextClipboard", {
        get: function () {
            release || notImplemented("public flash.text.TextField::get useRichTextClipboard");
            return;
            // return this._useRichTextClipboard;
        },
        set: function (value) {
            value = !!value;
            release || notImplemented("public flash.text.TextField::set useRichTextClipboard");
            return;
            // this._useRichTextClipboard = value;
        },
        enumerable: true,
        configurable: true
    });
    TextField.prototype.copyRichText = function () {
        release || notImplemented("public flash.text.TextField::copyRichText");
    };
    TextField.prototype.pasteRichText = function (richText) {
        richText = axCoerceString(richText);
        release || notImplemented("public flash.text.TextField::pasteRichText");
    };
    TextField.prototype.getXMLText = function (beginIndex, endIndex) {
        if (endIndex === void 0) { endIndex = 2147483647; }
        beginIndex = +beginIndex;
        endIndex = +endIndex;
        release || notImplemented("public flash.text.TextField::getXMLText");
        return "";
    };
    TextField.prototype.insertXMLText = function (beginIndex, endIndex, richText, pasting) {
        beginIndex = +beginIndex;
        endIndex = +endIndex;
        richText = axCoerceString(richText);
        pasting = !!pasting;
        release || notImplemented("public flash.text.TextField::insertXMLText");
    };
    TextField.prototype._ensureLineMetrics = function () {
        if (!this._hasDirtyFlags(8 /* DirtyTextContent */)) {
            return;
        }
        var serializer = this.sec.player;
        var lineMetricsData = serializer.syncDisplayObject(this, false);
        var textWidth = lineMetricsData.readInt();
        var textHeight = lineMetricsData.readInt();
        var offsetX = lineMetricsData.readInt();
        var bounds = this._lineBounds;
        if (this._autoSize !== TextFieldAutoSize.NONE) {
            bounds.xMin = offsetX;
            bounds.xMax = offsetX + textWidth + 80;
            bounds.yMax = bounds.yMin + textHeight + 80;
        }
        this._textWidth = textWidth;
        this._textHeight = textHeight;
        this._numLines = lineMetricsData.readInt();
        this._lineMetricsData = lineMetricsData;
        if (this._textHeight > bounds.height) {
            var maxScrollV = 1;
            var bottomScrollV = 1;
            lineMetricsData.position = 16;
            var y = 0;
            for (var i = 0; i < this._numLines; i++) {
                lineMetricsData.position += 8;
                var ascent = lineMetricsData.readInt();
                var descent = lineMetricsData.readInt();
                var leading = lineMetricsData.readInt();
                var height = ascent + descent + leading;
                if (y > bounds.height / 20) {
                    maxScrollV++;
                }
                else {
                    bottomScrollV++;
                }
                y += height;
            }
            this._maxScrollV = maxScrollV;
            this._bottomScrollV = bottomScrollV;
        }
        if (this._textWidth > bounds.width) {
            this._maxScrollH = (((this._textWidth + 80) - bounds.width) / 20) | 0;
        }
        else {
            this._maxScrollH = 0;
        }
    };
    TextField.prototype.appendText = function (newText) {
        this._textContent.appendText(axCoerceString(newText));
    };
    TextField.prototype.getCharBoundaries = function (charIndex /*int*/) {
        charIndex = charIndex | 0;
        release || somewhatImplemented("public flash.text.TextField::getCharBoundaries");
        var fakeCharHeight = this.textHeight, fakeCharWidth = fakeCharHeight * 0.75;
        return new Rectangle(charIndex * fakeCharWidth, 0, fakeCharWidth, fakeCharHeight);
    };
    TextField.prototype.getCharIndexAtPoint = function (x, y) {
        x = +x;
        y = +y;
        release || notImplemented("public flash.text.TextField::getCharIndexAtPoint");
        return;
    };
    TextField.prototype.getFirstCharInParagraph = function (charIndex /*int*/) {
        charIndex = charIndex | 0;
        release || notImplemented("public flash.text.TextField::getFirstCharInParagraph");
        return;
    };
    TextField.prototype.getLineIndexAtPoint = function (x, y) {
        x = +x;
        y = +y;
        release || notImplemented("public flash.text.TextField::getLineIndexAtPoint");
        return;
    };
    TextField.prototype.getLineIndexOfChar = function (charIndex /*int*/) {
        charIndex = charIndex | 0;
        release || notImplemented("public flash.text.TextField::getLineIndexOfChar");
        return;
    };
    TextField.prototype.getLineLength = function (lineIndex /*int*/) {
        lineIndex = lineIndex | 0;
        release || notImplemented("public flash.text.TextField::getLineLength");
        return;
    };
    TextField.prototype.getLineMetrics = function (lineIndex /*int*/) {
        lineIndex = lineIndex | 0;
        if (lineIndex < 0 || lineIndex > this._numLines - 1) {
            this.sec.throwError('RangeError', Errors.ParamRangeError);
        }
        this._ensureLineMetrics();
        var lineMetricsData = this._lineMetricsData;
        lineMetricsData.position = 16 + lineIndex * 20;
        // The lines left position includes the gutter widths (it should also include the the margin
        // and indent, which we don't support yet).
        var x = lineMetricsData.readInt() + this._lineBounds.xMin + 2;
        var width = lineMetricsData.readInt();
        var ascent = lineMetricsData.readInt();
        var descent = lineMetricsData.readInt();
        var leading = lineMetricsData.readInt();
        var height = ascent + descent + leading;
        return new TextLineMetrics(x, width, height, ascent, descent, leading);
    };
    TextField.prototype.getLineOffset = function (lineIndex /*int*/) {
        lineIndex = lineIndex | 0;
        var lines = this._textContent.plainText.split('\r');
        if (lineIndex < 0 || lineIndex >= lines.length) {
            this.sec.throwError('RangeError', Errors.ParamRangeError);
        }
        var offset = 0;
        for (var i = 0; i < lineIndex; i++) {
            offset += lines[i].length + 1; // Length + `\r`
        }
        // TODO:  I've tried modifying the width of the text field so that lines wrap, but this doesn't seem
        // to have any effect on how line offsets are computed. I'm leaving in the |somewhatImplemented| call
        // since this is not fully tested.
        release || release || somewhatImplemented("public flash.text.TextField::getLineOffset");
        return offset;
    };
    TextField.prototype.getLineText = function (lineIndex /*int*/) {
        lineIndex = lineIndex | 0;
        var lines = this._textContent.plainText.split('\r');
        if (lineIndex < 0 || lineIndex >= lines.length) {
            this.sec.throwError('RangeError', Errors.ParamRangeError);
        }
        return lines[lineIndex];
    };
    TextField.prototype.getParagraphLength = function (charIndex /*int*/) {
        charIndex = charIndex | 0;
        release || notImplemented("public flash.text.TextField::getParagraphLength");
        return;
    };
    /**
     * Returns a TextFormat object that contains the intersection of formatting information for the
     * range of text between |beginIndex| and |endIndex|.
     */
    TextField.prototype.getTextFormat = function (beginIndex, endIndex) {
        if (beginIndex === void 0) { beginIndex = -1; }
        if (endIndex === void 0) { endIndex = -1; }
        beginIndex = beginIndex | 0;
        endIndex = endIndex | 0;
        var plainText = this._textContent.plainText;
        var maxIndex = plainText.length;
        if (beginIndex < 0) {
            beginIndex = 0;
            if (endIndex < 0) {
                endIndex = maxIndex;
            }
        }
        else {
            if (endIndex < 0) {
                endIndex = beginIndex + 1;
            }
        }
        if (endIndex <= beginIndex || endIndex > maxIndex) {
            this.sec.throwError('RangeError', Errors.ParamRangeError);
        }
        var format;
        var textRuns = this._textContent.textRuns;
        for (var i = 0; i < textRuns.length; i++) {
            var run = textRuns[i];
            if (run.intersects(beginIndex, endIndex)) {
                if (format) {
                    format.intersect(run.textFormat);
                }
                else {
                    format = run.textFormat.clone();
                }
            }
        }
        return format;
    };
    TextField.prototype.getTextRuns = function (beginIndex, endIndex) {
        if (beginIndex === void 0) { beginIndex = 0; }
        if (endIndex === void 0) { endIndex = 2147483647; }
        var textRuns = this._textContent.textRuns;
        var result = [];
        for (var i = 0; i < textRuns.length; i++) {
            var textRun = textRuns[i];
            if (textRun.beginIndex >= beginIndex && textRun.endIndex <= endIndex) {
                result.push(textRun.clone());
            }
        }
        return this.sec.createArrayUnsafe(result);
    };
    TextField.prototype.getRawText = function () {
        release || notImplemented("public flash.text.TextField::getRawText");
        return;
    };
    TextField.prototype.replaceSelectedText = function (value) {
        value = "" + value;
        this.replaceText(this._selectionBeginIndex, this._selectionEndIndex, value);
    };
    TextField.prototype.replaceText = function (beginIndex /*int*/, endIndex /*int*/, newText) {
        beginIndex = beginIndex | 0;
        endIndex = endIndex | 0;
        newText = "" + newText;
        if (beginIndex < 0 || endIndex < 0) {
            return;
        }
        this._textContent.replaceText(beginIndex, endIndex, newText);
        this._invalidateContent();
        this._ensureLineMetrics();
    };
    TextField.prototype.setSelection = function (beginIndex /*int*/, endIndex /*int*/) {
        this._selectionBeginIndex = beginIndex | 0;
        this._selectionEndIndex = endIndex | 0;
    };
    TextField.prototype.setTextFormat = function (format, beginIndex, endIndex) {
        if (beginIndex === void 0) { beginIndex = -1; }
        if (endIndex === void 0) { endIndex = -1; }
        format = format;
        beginIndex = beginIndex | 0;
        endIndex = endIndex | 0;
        var plainText = this._textContent.plainText;
        var maxIndex = plainText.length;
        if (beginIndex < 0) {
            beginIndex = 0;
            if (endIndex < 0) {
                endIndex = maxIndex;
            }
        }
        else {
            if (endIndex < 0) {
                endIndex = beginIndex + 1;
            }
        }
        if (beginIndex > maxIndex || endIndex > maxIndex) {
            this.sec.throwError('RangeError', Errors.ParamRangeError);
        }
        if (endIndex <= beginIndex) {
            return;
        }
        var subText = plainText.substring(beginIndex, endIndex);
        this._textContent.replaceText(beginIndex, endIndex, subText, format);
        this._invalidateContent();
        this._ensureLineMetrics();
    };
    TextField.prototype.getImageReference = function (id) {
        //id = "" + id;
        release || notImplemented("public flash.text.TextField::getImageReference");
        return;
    };
    TextField.classSymbols = null;
    TextField.instanceSymbols = null;
    TextField.classInitializer = null;
    return TextField;
}(InteractiveObject));
export { TextField };
var TextSymbol = /** @class */ (function (_super) {
    __extends(TextSymbol, _super);
    function TextSymbol(data, sec) {
        var _this = _super.call(this, data, TextField.axClass, true) || this;
        _this.color = 0;
        _this.size = 0;
        _this.face = "";
        _this.bold = false;
        _this.italic = false;
        _this.align = TextFormatAlign.LEFT;
        _this.leftMargin = 0;
        _this.rightMargin = 0;
        _this.indent = 0;
        _this.leading = 0;
        _this.multiline = false;
        _this.wordWrap = false;
        _this.embedFonts = false;
        _this.selectable = true;
        _this.border = false;
        _this.initialText = "";
        _this.html = false;
        _this.displayAsPassword = false;
        _this.type = TextFieldType.DYNAMIC;
        _this.maxChars = 0;
        _this.autoSize = TextFieldAutoSize.NONE;
        _this.variableName = null;
        _this.textContent = null;
        return _this;
    }
    TextSymbol.FromTextData = function (data, loaderInfo) {
        var sec = loaderInfo.sec;
        var symbol = new TextSymbol(data, sec);
        symbol._setBoundsFromData(data);
        var tag = data.tag;
        if (data.static) {
            symbol.dynamic = false;
            symbol.symbolClass = StaticText.axClass;
            if (tag.initialText) {
                console.log("80pro todo: TextContent");
                /*
                var textContent = new TextContent(sec);
                textContent.bounds = symbol.lineBounds;
                textContent.parseHtml(tag.initialText, null, false);
                textContent.matrix = new Matrix();
                textContent.matrix.copyFromUntyped(data.matrix);
                textContent.coords = data.coords;
                symbol.textContent = textContent;*/
            }
        }
        if (tag.flags & 4 /* HasColor */) {
            symbol.color = tag.color >>> 8;
        }
        if (tag.flags & 1 /* HasFont */) {
            symbol.size = tag.fontHeight;
            // Requesting the font symbol guarantees that it's loaded and initialized.
            var fontSymbol = loaderInfo.getSymbolById(tag.fontId);
            if (fontSymbol) {
                symbol.face = tag.flags & 256 /* UseOutlines */ ?
                    fontSymbol.name :
                    'swffont' + fontSymbol.syncId;
                symbol.bold = fontSymbol.bold;
                symbol.italic = fontSymbol.italic;
            }
            else {
                warning("Font " + tag.fontId + " is not defined.");
            }
        }
        if (tag.flags & 8192 /* HasLayout */) {
            symbol.align = TextFormatAlign.fromNumber(tag.align);
            symbol.leftMargin = tag.leftMargin;
            symbol.rightMargin = tag.rightMargin;
            symbol.indent = tag.indent;
            symbol.leading = tag.leading;
        }
        symbol.multiline = !!(tag.flags & 32 /* Multiline */);
        symbol.wordWrap = !!(tag.flags & 64 /* WordWrap */);
        symbol.embedFonts = !!(tag.flags & 256 /* UseOutlines */);
        symbol.selectable = !(tag.flags & 4096 /* NoSelect */);
        symbol.border = !!(tag.flags & 2048 /* Border */);
        if (tag.flags & 128 /* HasText */) {
            symbol.initialText = tag.initialText;
        }
        symbol.html = !!(tag.flags & 512 /* Html */);
        symbol.displayAsPassword = !!(tag.flags & 16 /* Password */);
        symbol.type = tag.flags & 8 /* ReadOnly */ ?
            TextFieldType.DYNAMIC :
            TextFieldType.INPUT;
        if (tag.flags & 2 /* HasMaxLength */) {
            symbol.maxChars = tag.maxLength;
        }
        symbol.autoSize = tag.flags & 16384 /* AutoSize */ ?
            TextFieldAutoSize.LEFT :
            TextFieldAutoSize.NONE;
        symbol.variableName = tag.variableName;
        return symbol;
    };
    /**
     * Turns raw DefineLabel tag data into an object that's consumable as a text symbol and then
     * passes that into `FromTextData`, returning the resulting TextSymbol.
     *
     * This has to be done outside the SWF parser because it relies on any used fonts being
     * available as symbols, which isn't the case in the SWF parser.
     */
    TextSymbol.FromLabelData = function (data, loaderInfo) {
        var bounds = data.fillBounds;
        var records = data.records;
        var coords = data.coords = [];
        var htmlText = '';
        var size = 12;
        var face = 'Times Roman';
        var bold = false;
        var italic = false;
        var color = 0;
        var x = 0;
        var y = 0;
        var codes;
        for (var i = 0; i < records.length; i++) {
            var record = records[i];
            if (record.flags & 8 /* HasFont */) {
                var fontSymbol = loaderInfo.getSymbolById(record.fontId);
                if (fontSymbol) {
                    codes = fontSymbol.codes;
                    size = record.fontHeight;
                    if (!fontSymbol.originalSize) {
                        size /= 20;
                    }
                    face = fontSymbol.metrics ? 'swffont' + fontSymbol.syncId : fontSymbol.name;
                    bold = fontSymbol.bold;
                    italic = fontSymbol.italic;
                }
                else {
                    warning('Label ' + data.id + 'refers to undefined font symbol ' + record.fontId);
                }
            }
            if (record.flags & 4 /* HasColor */) {
                color = record.color >>> 8;
            }
            if (record.flags & 1 /* HasMoveX */) {
                x = record.moveX;
            }
            if (record.flags & 2 /* HasMoveY */) {
                y = record.moveY;
            }
            var text = '';
            var entries = record.entries;
            var j = 0;
            var entry;
            while ((entry = entries[j++])) {
                var code = codes[entry.glyphIndex];
                release || assert(code, 'undefined label glyph');
                var char = String.fromCharCode(code);
                text += charEscapeMap[char] || char;
                coords.push(x, y);
                x += entry.advance;
            }
            if (italic) {
                text = '<i>' + text + '</i>';
            }
            if (bold) {
                text = '<b>' + text + '</b>';
            }
            htmlText += '<font size="' + size + '" face="' + face + '"' + ' color="#' +
                ('000000' + color.toString(16)).slice(-6) + '">' + text + '</font>';
        }
        data.tag.initialText = htmlText;
        return TextSymbol.FromTextData(data, loaderInfo);
    };
    return TextSymbol;
}(DisplaySymbol));
export { TextSymbol };
var charEscapeMap = { '<': '&lt;', '>': '&gt;', '&': '&amp;' };
