import { __extends } from "tslib";
import { axCoerceString } from "../../avm2/run/axCoerceString";
import { roundHalfEven } from "../../base/utilities/NumberUtilities";
import { ASObject } from "../../avm2/nat/ASObject";
import { Errors } from "../../avm2/errors";
import { FontStyle } from "./FontStyle";
import { TextFormatDisplay } from "./TextFormatDisplay";
/**
 * Copyright 2014 Mozilla Foundation
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
// Class: TextFormat
var TextFormat = /** @class */ (function (_super) {
    __extends(TextFormat, _super);
    function TextFormat(font, size, color, bold, italic, underline, url, target, align, leftMargin, rightMargin, indent, leading) {
        if (font === void 0) { font = null; }
        if (size === void 0) { size = null; }
        if (color === void 0) { color = null; }
        if (bold === void 0) { bold = null; }
        if (italic === void 0) { italic = null; }
        if (underline === void 0) { underline = null; }
        if (url === void 0) { url = null; }
        if (target === void 0) { target = null; }
        if (align === void 0) { align = null; }
        if (leftMargin === void 0) { leftMargin = null; }
        if (rightMargin === void 0) { rightMargin = null; }
        if (indent === void 0) { indent = null; }
        if (leading === void 0) { leading = null; }
        var _this = _super.call(this) || this;
        _this.font = font;
        _this.size = size;
        _this.color = color;
        _this.bold = bold;
        _this.italic = italic;
        _this.underline = underline;
        _this.url = url;
        _this.target = target;
        _this.align = align;
        _this.leftMargin = leftMargin;
        _this.rightMargin = rightMargin;
        _this.indent = indent;
        _this.leading = leading;
        _this._blockIndent = null;
        _this._bullet = null;
        _this._display = TextFormatDisplay.BLOCK;
        _this._kerning = null;
        _this._letterSpacing = null;
        _this._tabStops = null;
        return _this;
    }
    Object.defineProperty(TextFormat.prototype, "align", {
        // AS -> JS Bindings
        get: function () {
            return this._align;
        },
        set: function (value) {
            value = axCoerceString(value);
            //if (TextFormatAlign.toNumber(value) < 0) {
            //  this.sec.throwError("ArgumentError", Errors.InvalidEnumError, "align");
            //}
            this._align = value;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(TextFormat.prototype, "blockIndent", {
        get: function () {
            return this._blockIndent;
        },
        set: function (value) {
            this._blockIndent = TextFormat.coerceNumber(value);
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(TextFormat.prototype, "bold", {
        get: function () {
            return this._bold;
        },
        set: function (value) {
            this._bold = TextFormat.coerceBoolean(value);
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(TextFormat.prototype, "bullet", {
        get: function () {
            return this._bullet;
        },
        set: function (value) {
            this._bullet = TextFormat.coerceBoolean(value);
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(TextFormat.prototype, "color", {
        get: function () {
            return this._color;
        },
        set: function (value) {
            this._color = value != null ? +value | 0 : null;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(TextFormat.prototype, "display", {
        get: function () {
            return this._display;
        },
        set: function (value) {
            this._display = axCoerceString(value);
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(TextFormat.prototype, "font", {
        get: function () {
            return this._font;
        },
        set: function (value) {
            this._font = axCoerceString(value);
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(TextFormat.prototype, "style", {
        get: function () {
            if (this._bold && this._italic) {
                return FontStyle.BOLD_ITALIC;
            }
            else if (this._bold) {
                return FontStyle.BOLD;
            }
            else if (this._italic) {
                return FontStyle.ITALIC;
            }
            else {
                return FontStyle.REGULAR;
            }
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(TextFormat.prototype, "indent", {
        get: function () {
            return this._indent;
        },
        set: function (value) {
            this._indent = TextFormat.coerceNumber(value);
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(TextFormat.prototype, "italic", {
        get: function () {
            return this._italic;
        },
        set: function (value) {
            this._italic = TextFormat.coerceBoolean(value);
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(TextFormat.prototype, "kerning", {
        get: function () {
            return this._kerning;
        },
        set: function (value) {
            this._kerning = TextFormat.coerceBoolean(value);
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(TextFormat.prototype, "leading", {
        get: function () {
            return this._leading;
        },
        set: function (value) {
            this._leading = TextFormat.coerceNumber(value);
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(TextFormat.prototype, "leftMargin", {
        get: function () {
            return this._leftMargin;
        },
        set: function (value) {
            this._leftMargin = TextFormat.coerceNumber(value);
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(TextFormat.prototype, "letterSpacing", {
        get: function () {
            return this._letterSpacing;
        },
        set: function (value) {
            this._letterSpacing = TextFormat.coerceNumber(value);
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(TextFormat.prototype, "rightMargin", {
        get: function () {
            return this._rightMargin;
        },
        set: function (value) {
            this._rightMargin = TextFormat.coerceNumber(value);
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(TextFormat.prototype, "size", {
        get: function () {
            return this._size;
        },
        set: function (value) {
            this._size = TextFormat.coerceNumber(value);
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(TextFormat.prototype, "tabStops", {
        get: function () {
            return this.sec.createArrayUnsafe(this._tabStops);
        },
        set: function (value) {
            if (value == null) {
                this._tabStops = null;
                return;
            }
            if (!this.sec.AXArray.axIsType(value)) {
                this.sec.throwError("ArgumentError", Errors.CheckTypeFailedError, value, 'Array');
            }
            this._tabStops = value.value;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(TextFormat.prototype, "target", {
        get: function () {
            return this._target;
        },
        set: function (value) {
            this._target = axCoerceString(value);
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(TextFormat.prototype, "underline", {
        get: function () {
            return this._underline;
        },
        set: function (value) {
            this._underline = TextFormat.coerceBoolean(value);
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(TextFormat.prototype, "url", {
        get: function () {
            return this._url;
        },
        set: function (value) {
            this._url = axCoerceString(value);
        },
        enumerable: true,
        configurable: true
    });
    /**
     * All integer values on TextFormat are typed as Object and coerced to ints using the following
     * "algorithm":
     * - if the supplied value is null or undefined, the field is set to null
     * - else if coercing to number results in NaN or the value is greater than MAX_INT, set to
     *   -0x80000000
     * - else, round the coerced value using half-even rounding
     */
    TextFormat.coerceNumber = function (value) {
        /* tslint:disable */
        if (value == undefined) {
            return null;
        }
        /* tslint:enable */
        if (isNaN(value) || value > 0xfffffff) {
            return -0x80000000;
        }
        return roundHalfEven(value);
    };
    /**
     * Boolean values are only stored as bools if they're not undefined or null. In that case,
     * they're stored as null.
     */
    TextFormat.coerceBoolean = function (value) {
        /* tslint:disable */
        return value == undefined ? null : !!value;
        /* tslint:enable */
    };
    TextFormat.prototype.clone = function () {
        var tf = new TextFormat(this._font, this._size, this._color, this._bold, this._italic, this._underline, this._url, this._target, this._align, this._leftMargin, this._rightMargin, this._indent, this._leading);
        tf._blockIndent = this._blockIndent;
        tf._bullet = this._bullet;
        tf._display = this._display;
        tf._kerning = this._kerning;
        tf._letterSpacing = this._letterSpacing;
        tf._tabStops = this._tabStops;
        return tf;
    };
    TextFormat.prototype.equals = function (other) {
        return this._align === other._align &&
            this._blockIndent === other._blockIndent &&
            this._bold === other._bold &&
            this._bullet === other._bullet &&
            this._color === other._color &&
            this._display === other._display &&
            this._font === other._font &&
            this._indent === other._indent &&
            this._italic === other._italic &&
            this._kerning === other._kerning &&
            this._leading === other._leading &&
            this._leftMargin === other._leftMargin &&
            this._letterSpacing === other._letterSpacing &&
            this._rightMargin === other._rightMargin &&
            this._size === other._size &&
            this._tabStops === other._tabStops &&
            this._target === other._target &&
            this._underline === other._underline &&
            this._url === other._url;
    };
    TextFormat.prototype.merge = function (other) {
        if (other._align !== null) {
            this._align = other._align;
        }
        if (other._blockIndent !== null) {
            this._blockIndent = other._blockIndent;
        }
        if (other._bold !== null) {
            this._bold = other._bold;
        }
        if (other._bullet !== null) {
            this._bullet = other._bullet;
        }
        if (other._color !== null) {
            this._color = other._color;
        }
        if (other._display !== null) {
            this._display = other._display;
        }
        if (other._font) {
            this._font = other._font;
        }
        if (other._indent !== null) {
            this._indent = other._indent;
        }
        if (other._italic !== null) {
            this._italic = other._italic;
        }
        if (other._kerning !== null) {
            this._kerning = other._kerning;
        }
        if (other._leading !== null) {
            this._leading = other._leading;
        }
        if (other._leftMargin !== null) {
            this._leftMargin = other._leftMargin;
        }
        if (other._letterSpacing !== null) {
            this._letterSpacing = other._letterSpacing;
        }
        if (other._rightMargin !== null) {
            this._rightMargin = other._rightMargin;
        }
        if (other._size !== null) {
            this._size = other._size;
        }
        if (other._tabStops !== null) {
            this._tabStops = other._tabStops;
        }
        if (other._target) {
            this._target = other._target;
        }
        if (other._underline !== null) {
            this._underline = other._underline;
        }
        if (other._url) {
            this._url = other._url;
        }
    };
    TextFormat.prototype.intersect = function (other) {
        if (other._align !== this._align) {
            this._align = null;
        }
        if (other._blockIndent !== this._blockIndent) {
            this._blockIndent = null;
        }
        if (other._bold !== this._bold) {
            this._bold = null;
        }
        if (other._bullet !== this._bullet) {
            this._bullet = null;
        }
        if (other._color !== this._color) {
            this._color = null;
        }
        if (other._display !== this._display) {
            this._display = null;
        }
        if (other._font !== this._font) {
            this._font = null;
        }
        if (other._indent !== this._indent) {
            this._indent = null;
        }
        if (other._italic !== this._italic) {
            this._italic = null;
        }
        if (other._kerning !== this._kerning) {
            this._kerning = null;
        }
        if (other._leading !== this._leading) {
            this._leading = null;
        }
        if (other._leftMargin !== this._leftMargin) {
            this._leftMargin = null;
        }
        if (other._letterSpacing !== this._letterSpacing) {
            this._letterSpacing = null;
        }
        if (other._rightMargin !== this._rightMargin) {
            this._rightMargin = null;
        }
        if (other._size !== this._size) {
            this._size = null;
        }
        if (other._tabStops !== this._tabStops) {
            this._tabStops = null;
        }
        if (other._target !== this._target) {
            this._target = null;
        }
        if (other._underline !== this._underline) {
            this._underline = null;
        }
        if (other._url !== this._url) {
            this._url = null;
        }
    };
    TextFormat.prototype.transform = function (formatObject) {
        var v = formatObject.textAlign;
        if (v) {
            this.align = v;
        }
        v = formatObject.fontWeight;
        if (v === "bold") {
            this.bold = true;
        }
        else if (v === "normal") {
            this.bold = false;
        }
        v = formatObject.color;
        if (v) {
            // When parsing colors, whitespace is trimmed away, and all numbers are accepted, as long
            // as they make up the full string after the "#", without any non-numeric pre- or suffix.
            // This implementation is somewhat atrocious, but it should be reasonably fast and works.
            var colorStr = axCoerceString(v).trim().toLowerCase();
            if (colorStr[0] === '#') {
                var numericPart = colorStr.substr(1);
                while (numericPart[0] === '0') {
                    numericPart = numericPart.substr(1);
                }
                var colorVal = parseInt(numericPart, 16);
                if (colorVal.toString(16) === numericPart) {
                    this.color = colorVal;
                }
            }
        }
        v = formatObject.display;
        if (v) {
            this.display = v;
        }
        v = formatObject.fontFamily;
        if (v) {
            // TODO: Sanitize fontFamily string.
            this.font = v.replace('sans-serif', '_sans').replace('serif', '_serif');
        }
        v = formatObject.textIndent;
        if (v) {
            this.indent = parseInt(v);
        }
        v = formatObject.fontStyle;
        if (v === "italic") {
            this.italic = true;
        }
        else if (v === "normal") {
            this.italic = false;
        }
        v = formatObject.kerning;
        if (v === "true") {
            this.kerning = 1;
        }
        else if (v === "false") {
            this.kerning = 0;
        }
        else {
            this.kerning = parseInt(v);
        }
        v = formatObject.leading;
        if (v) {
            this.leading = parseInt(v);
        }
        v = formatObject.marginLeft;
        if (v) {
            this.leftMargin = parseInt(v);
        }
        v = formatObject.letterSpacing;
        if (v) {
            this.letterSpacing = parseFloat(v);
        }
        v = formatObject.marginRight;
        if (v) {
            this.rightMargin = parseInt(v);
        }
        v = formatObject.fontSize;
        if (v) {
            var size = parseInt(v);
            if (size > 0) {
                this.size = size;
            }
        }
        v = formatObject.textDecoration;
        if (v === "none") {
            this.underline = false;
        }
        else if (v === "underline") {
            this.underline = true;
        }
        return this;
    };
    TextFormat.classInitializer = null;
    TextFormat.classSymbols = null; // [];
    TextFormat.instanceSymbols = null; // [];
    return TextFormat;
}(ASObject));
export { TextFormat };
