import { release, notImplemented } from "../../base/utilities/Debug";
import { Socket } from "./Socket";
import { ByteArray } from "../../avm2/natives/byteArray";
import { X509Certificate } from "../security/X509Certificate";

/**
 * Copyright 2014 Mozilla Foundation
 * 
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
// Class: SecureSocket
export class SecureSocket extends Socket {
  
  // Called whenever the class is initialized.
  static classInitializer: any = null;

  // List of static symbols to link.
  static classSymbols: string [] = null; // [];
  
  // List of instance symbols to link.
  static instanceSymbols: string [] = null; // ["serverCertificateStatus", "connect"];
  
  constructor () {
    super(undefined, undefined);
  }
  
  // JS -> AS Bindings
  
  serverCertificateStatus: string;
  connect: (host: string, port: number /*int*/) => void;
  
  // AS -> JS Bindings
  // static _isSupported: boolean;
  get isSupported(): boolean {
    release || notImplemented("public flash.net.SecureSocket::get isSupported"); return;
    // return this._isSupported;
  }
  
  // _serverCertificateStatus: string;
  // _serverCertificate: flash.security.X509Certificate;
  get serverCertificate(): X509Certificate {
    release || notImplemented("public flash.net.SecureSocket::get serverCertificate"); return;
    // return this._serverCertificate;
  }
  addBinaryChainBuildingCertificate(certificate: ByteArray, trusted: boolean): void {
    certificate = certificate; trusted = !!trusted;
    release || notImplemented("public flash.net.SecureSocket::addBinaryChainBuildingCertificate"); return;
  }
}
